//
//  UIView+IAPUtils.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "UIView+IAPUtils.h"
#import <objc/runtime.h>

@implementation UIView (IAPUtils)
- (CGFloat)iapu_left {
  return self.frame.origin.x;
}

- (void)setIapu_left:(CGFloat)x {
  CGRect frame = self.frame;
  frame.origin.x = x;
  self.frame = frame;
}

- (CGFloat)iapu_top {
  return self.frame.origin.y;
}

- (void)setIapu_top:(CGFloat)y {
  CGRect frame = self.frame;
  frame.origin.y = y;
  self.frame = frame;
}

- (CGFloat)iapu_right {
  return self.frame.origin.x + self.frame.size.width;
}

- (void)setIapu_right:(CGFloat)right {
  CGRect frame = self.frame;
  frame.origin.x = right - frame.size.width;
  self.frame = frame;
}

- (CGFloat)iapu_bottom {
  return self.frame.origin.y + self.frame.size.height;
}

- (void)setIapu_bottom:(CGFloat)bottom {
  CGRect frame = self.frame;
  frame.origin.y = bottom - frame.size.height;
  self.frame = frame;
}

- (CGFloat)iapu_width {
  return self.frame.size.width;
}

- (void)setIapu_width:(CGFloat)width {
  CGRect frame = self.frame;
  frame.size.width = width;
  self.frame = frame;
}

- (CGFloat)iapu_height {
  return self.frame.size.height;
}

- (void)setIapu_height:(CGFloat)height {
  CGRect frame = self.frame;
  frame.size.height = height;
  self.frame = frame;
}

- (CGFloat)iapu_centerX {
  return self.center.x;
}

- (void)setIapu_centerX:(CGFloat)centerX {
  self.center = CGPointMake(centerX, self.center.y);
}

- (CGFloat)iapu_centerY {
  return self.center.y;
}

- (void)setIapu_centerY:(CGFloat)centerY {
  self.center = CGPointMake(self.center.x, centerY);
}

- (CGPoint)iapu_origin {
  return self.frame.origin;
}

- (void)setIapu_origin:(CGPoint)origin {
  CGRect frame = self.frame;
  frame.origin = origin;
  self.frame = frame;
}

- (CGSize)iapu_size {
  return self.frame.size;
}

- (void)setIapu_size:(CGSize)size {
  CGRect frame = self.frame;
  frame.size = size;
  self.frame = frame;
}


// 指定倒角
- (CAShapeLayer *)iapu_setCornerRadii:(CGFloat)cornerRadii roundingCorners:(UIRectCorner)roundingCorners {
    if (self.layer.mask) {
        self.layer.mask = nil;
    }
    self.layer.masksToBounds = YES;
    UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:self.bounds
                                                   byRoundingCorners:roundingCorners
                                                         cornerRadii:CGSizeMake(cornerRadii, cornerRadii)];
    CAShapeLayer *maskLayer = [CAShapeLayer layer];
    maskLayer.frame = self.bounds;
    maskLayer.path = maskPath.CGPath;
    self.layer.mask = maskLayer;
    return maskLayer;
}

- (void)iapu_touchUpInside:(void(^)(void))block {
  if (block) {
    objc_setAssociatedObject(self, @selector(iapu_action), block, OBJC_ASSOCIATION_COPY_NONATOMIC);
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(iapu_action)];
    self.userInteractionEnabled = YES;
    [self addGestureRecognizer:tap];
  }
}

- (void)iapu_action {
  void (^action)(void) = objc_getAssociatedObject(self, _cmd);
  if (action) {
      action();
  }
}
@end
