//
//  IAPMultiLanguageUtils.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/11.
//

#import "IAPMultiLanguageUtils.h"
#import "AlipayPlusClientAPI.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"

@implementation IAPMultiLanguageUtils

+ (NSString *)defaultCurrentLanguage {
    return @"en_US";
}

+ (NSString *)formatedCurrentLauguage {
    return [self formatedLanguageCode:[AlipayPlusClient shared].configuration.language];
}

+ (nullable NSString *)formatedLanguageCode:(NSString *)originalLauguageCode {
    if (!originalLauguageCode.length) {
        return nil;
    }
    
    NSString *formatedLanguageCode = originalLauguageCode;
    NSArray *strings = [originalLauguageCode componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"-_"]];
    if (strings.count > 1) {
        formatedLanguageCode = strings[0];
        for (int i = 1; i < strings.count; i++) {
            formatedLanguageCode = [formatedLanguageCode stringByAppendingFormat:@"_%@", strings[i]];
        }
    }
    return formatedLanguageCode;
}

+ (nullable NSArray<NSString *> *)formatedLanguageCodes:(NSArray<NSString *> *)originalLauguageCodes {
    if (!originalLauguageCodes.count) {
        return nil;
    }
    
    NSMutableArray *formatedLanguageCodes = [NSMutableArray arrayWithCapacity:originalLauguageCodes.count];
    for (int i = 0; i < originalLauguageCodes.count; i++) {
        [formatedLanguageCodes addObject:[self formatedLanguageCode:originalLauguageCodes[i]]];
    }
    return formatedLanguageCodes;
}


+ (nullable NSString *)matchedLanguageWithLanguageCodes:(NSArray<NSString *> *)originalLauguageCodes {
    NSArray<NSString *> *formatedLanguageCodes = [self formatedLanguageCodes:originalLauguageCodes];
    NSString *formatedCurrentLauguage = [self formatedCurrentLauguage];
    
    BOOL matched = [formatedLanguageCodes containsObject:formatedCurrentLauguage];
    if (matched) {
        return formatedCurrentLauguage;
    }
    
    if ([formatedCurrentLauguage isEqualToString:@"zh_CN"]
        || [formatedCurrentLauguage hasPrefix:@"zh_Hans"]
        ) {
        formatedCurrentLauguage = @"zh_CN";
    } else if ([formatedCurrentLauguage isEqualToString:@"zh_HK"]
        || [formatedCurrentLauguage isEqualToString:@"zh_TW"]
        || [formatedCurrentLauguage isEqualToString:@"zh_MO"]
        || [formatedCurrentLauguage hasPrefix:@"zh_Hant"]
        || [formatedCurrentLauguage hasPrefix:@"yue"]
        ) {
        formatedCurrentLauguage = @"zh_HK";
    } else {
        formatedCurrentLauguage = @"en_US";
    }
    
    
    matched = [formatedLanguageCodes containsObject:formatedCurrentLauguage];
    if (matched) {
        return formatedCurrentLauguage;
    } else {
        return nil;
    }
}

+ (NSArray<NSString *> *)presetLanguageCodes {
    return @[
        @"zh_CN",
        @"zh_HK",
        @"en_US",
        @"ko_KR",
        @"fil_PH",
        @"ms_MY",
        @"id_ID",
        @"th_TH",
    ];
}

+ (nullable NSString *)matchedLanguageTextWithJSONString:(NSString *)JSONString {
    NSDictionary *jsonDictionary = [IAPJSONUtils JSONDictionaryFromString:JSONString];
    if (jsonDictionary.count) {
        NSString *currentMatchedLanguage = [self matchedLanguageWithLanguageCodes:jsonDictionary.allKeys] ?: @"en_US";
        NSString *matchedLanguageText = [jsonDictionary iapu_stringForKey:currentMatchedLanguage];
        return matchedLanguageText;
    }
    return nil;
}


+ (nullable NSArray<NSString *> *)matchedLanguageTextsWithJSONStringArray:(NSArray<NSString *> *)JSONStringArray {
    NSMutableArray *matchedLanguageTexts = [NSMutableArray arrayWithCapacity:JSONStringArray.count];
    [JSONStringArray enumerateObjectsUsingBlock:^(NSString * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSString *matchedLanguageText = [self matchedLanguageTextWithJSONString:obj];
        if (matchedLanguageText) {
            [matchedLanguageTexts addObject:matchedLanguageText];
        }
    }];
    
    return matchedLanguageTexts.count ? matchedLanguageTexts.copy : nil;
}

@end


