//
//  IAPRequestUtils.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/16.
//

#import "IAPRequestUtils.h"
#import "IAPConstant.h"
#import "NSError+IAPUtils.h"

@interface IAPRequestUtils ()
@property (nonatomic, strong) NSURLSession *session;
@end

@implementation IAPRequestUtils

+ (instancetype)shared {
    static IAPRequestUtils *utils = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        utils = [self new];
    });
    return utils;
}

- (void)asyncDownloadCDNFileWithURLString:(NSString *)urlString
                          timeoutInterval:(NSTimeInterval)timeoutInterval
                        completionHandler:(void (^)(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error))completionHandler {
    if (!completionHandler) {
        return;
    }
    
    if (!urlString.length || ![NSURLComponents componentsWithString:urlString] || (timeoutInterval <= 0) ) {
        dispatch_async(dispatch_get_main_queue(), ^{
            completionHandler(nil, nil, [NSError iapu_paramsIllegalErrorWithExtraMessage:nil]);
        });
        return;
    }
    
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:urlString]];
    [self asyncSendRequest:request timeoutInterval:timeoutInterval completionHandler:completionHandler];
}


- (void)asyncSendRequest:(NSURLRequest *)request
         timeoutInterval:(NSTimeInterval)timeoutInterval
       completionHandler:(void (^)(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error))completionHandler {
    if (!completionHandler) {
        return;
    }
    
    if (!request.URL || (timeoutInterval <= 0) ) {
        dispatch_async(dispatch_get_main_queue(), ^{
            completionHandler(nil, nil, [NSError iapu_paramsIllegalErrorWithExtraMessage:nil]);
        });
        return;
    }
    
    __block BOOL hasCallback = NO;
    NSURLSessionDataTask *task = [self.session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        if (hasCallback) {
            return;
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            hasCallback = YES;
            completionHandler(data, response, error);
        });
    }];
    
    [task resume];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(timeoutInterval * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        if (hasCallback) {
            return;
        }
        
        hasCallback = YES;
        completionHandler(nil, nil, [NSError errorWithDomain:NSURLErrorDomain code:NSURLErrorTimedOut userInfo:@{NSLocalizedDescriptionKey: @"time out"}]);
    });
}

#pragma mark - private

- (instancetype)init {
    if (self = [super init]) {
        _session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    }
    return self;
}


@end
