//
//  IAPStyleUtils.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/14.
//

#import "IAPStyleUtils.h"
#import "UIColor+IAPUtils.h"

@implementation IAPStyleUtils

+ (UIColor *)colorWithLightColor:(UIColor *)lightColor darkColor:(UIColor *)darkColor {
    if (!lightColor || !darkColor) {
        return [UIColor new];
    }
    
    if(@available(iOS 13, *)){
        return [UIColor colorWithDynamicProvider:^UIColor * _Nonnull(UITraitCollection * _Nonnull traitCollection) {
            if (traitCollection.userInterfaceStyle == UIUserInterfaceStyleDark) {
                return darkColor;
            } else {
                return lightColor;
            }
        }];
    } else {
        return lightColor;
    }
}

+ (UIImage *)imageWithLightImage:(UIImage *)lightImage darkImage:(UIImage *)darkImage {
    if (!lightImage || !darkImage) {
        return [UIImage new];
    }
    
    if(@available(iOS 13, *)){
        UIImage *image = [UIImage new];
        UITraitCollection *lightTraitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight];
        
        [image.imageAsset registerImage:lightImage withTraitCollection:lightTraitCollection];
        
        UITraitCollection *darkTraitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark];
        [image.imageAsset registerImage:darkImage withTraitCollection:darkTraitCollection];
        return image;
    } else {
        return lightImage;
    }
}


+ (NSBundle *)bundle {
    static NSBundle *resourceBundle = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSBundle *libBundle = [NSBundle bundleForClass:[self class]];
        NSString *bundlePath = [[libBundle bundlePath] stringByAppendingPathComponent:@"AlipayPlusResource.bundle"];
        resourceBundle = [NSBundle bundleWithPath:bundlePath];
        if (!resourceBundle) {
            NSAssert(YES, @"AlipayPlusResource.bundle is not existed");
        }
    });
    return resourceBundle;
}

+ (UIImage *)bundleImageNamed:(NSString *)name {
    UIImage *image =  [UIImage imageNamed:name inBundle:[self bundle] compatibleWithTraitCollection:nil];
    return image;
}


+ (UIFont *)fontOfSize:(NSInteger)size {
    return [UIFont systemFontOfSize:size];
}

+ (UIFont *)mediumFontOfSize:(NSInteger)size {
    return [UIFont systemFontOfSize:size weight:UIFontWeightMedium];
}

+ (UIFont *)boldFontOfSize:(NSInteger)size {
    return [UIFont systemFontOfSize:size weight:UIFontWeightBold];
}

@end
