//
//  IAPAlipayCNWalletAction.m
//  AlipayPlusClient
//
//  Created by yihan on 2023/1/11.
//

#import "AlipayPlusClient/AlipayPlusClientAPI.h"
#import "IAPAlipayCNWalletAction.h"
#import "NSDictionary+IAPUtils.h"

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
#pragma clang diagnostic ignored "-Wundeclared-selector"


@interface IAPAlipayCNWalletAction ()
@property (nonatomic, copy) void (^ callbackBlock)(IAPPaymentSheetEvent *_Nonnull sheetEvent);
@end

@implementation IAPAlipayCNWalletAction

+ (instancetype)shared {
    static IAPAlipayCNWalletAction *client = nil;
    static dispatch_once_t onceToken;

    dispatch_once(&onceToken, ^{
        client = [IAPAlipayCNWalletAction new];
    });
    return client;
}

- (nullable id)alipayService {

    if ([AlipayPlusClient shared].alipaySDKSService &&
        [self isServiceConformsToAlipayService:[AlipayPlusClient shared].alipaySDKSService]) {
        return [AlipayPlusClient shared].alipaySDKSService;
    }
    
    Class alipaySDK = NSClassFromString(@"AlipaySDK");
    if ([alipaySDK respondsToSelector:@selector(defaultService)]) {
        id service = [alipaySDK performSelector:@selector(defaultService)];
        if ([self isServiceConformsToAlipayService:service]) {
            return service;
        }
    }
    
    return nil;
}

- (BOOL)isServiceConformsToAlipayService:(id)service {
    if (service &&
        [service respondsToSelector:@selector(payOrder:fromScheme:callback:)] &&
        [service respondsToSelector:@selector(processOrderWithPaymentResult:standbyCallback:)]) {
        return YES;
    } else {
        return NO;
    }
}

- (void)handlePayWithWalletData:(IAPPaymentWalletData *)walletData
                       callback:(void (^)(IAPPaymentSheetEvent *_Nonnull sheetEvent))callback {
    self.callbackBlock = callback;
    id alipayService = [self alipayService];

    if (alipayService &&
        walletData.redirectionInfo.orderInfo &&
        [AlipayPlusClient shared].configuration.fromScheme) {
        [self invokePayOrderWithService:alipayService
                               payOrder:walletData.redirectionInfo.orderInfo
                             fromScheme:[AlipayPlusClient shared].configuration.fromScheme
                               callback:[self callbackWithResultDic]];
    } else {
        [super handlePayWithWalletData:walletData callback:callback];
    }
}

- (void)processOrderWithPaymentResult:(NSURL *)resultUrl {
    id alipayService = [self alipayService];
    [alipayService performSelector:@selector(processOrderWithPaymentResult:standbyCallback:) withObject:resultUrl withObject:[self callbackWithResultDic]];
}

#pragma mark - private

- (IAPPaymentSheetEvent *)transformResultToSheetEvent:(NSDictionary *)resultDic {
    if (!resultDic) {
        return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentException message:@"resultDic is nil"];
    }
    NSString *resultStatus = [resultDic iapu_stringForKey:@"resultStatus"];
    NSString *memo = [resultDic iapu_stringForKey:@"memo"];
    NSMutableDictionary *messageDic = [NSMutableDictionary new];
    if (resultStatus) {
        [messageDic setObject:resultStatus forKey:@"resultStatus"];
    }
    if (memo) {
        [messageDic setObject:memo forKey:@"memo"];
    }

    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:messageDic options:NSJSONWritingPrettyPrinted error:nil];
    NSString *jsonString = nil;

    if (jsonData.length > 0 ) {
        jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    }

    if (resultStatus && [resultStatus isKindOfClass:NSString.class]) {
        if ([resultStatus isEqualToString:@"9000"]) {
            return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentSuccess message:jsonString ? : @""];
        } else if ([resultStatus isEqualToString:@"4000"]) {
            return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentFailed message:jsonString ? : @""];
        } else if ([resultStatus isEqualToString:@"6001"]) {
            return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentCanceled message:jsonString ? : @""];
        } else if ([resultStatus isEqualToString:@"8000"]) {
            return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentProcessing message:jsonString ? : @""];
        } else {
            return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentException message:jsonString ? : @""];
        }
    } else {
        return [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventPaymentException message:@""];
    }
}

- (void)callbackWithSheetEvent:(IAPPaymentSheetEvent *)sheetEvent {
    if (self.callbackBlock) {
        self.callbackBlock(sheetEvent);
    }

    self.callbackBlock = nil;
}

- (void (^)(NSDictionary *))callbackWithResultDic {
    return ^(NSDictionary *resultDic) {
        IAPPaymentSheetEvent *event = [self transformResultToSheetEvent:resultDic];
        [self callbackWithSheetEvent:event];
    };
}

- (void)invokePayOrderWithService:(id)alipayService
                         payOrder:(NSString *)payOrder
                       fromScheme:(NSString *)fromScheme
                         callback:(void (^)(NSDictionary *resultDic))callback {
    NSMethodSignature *methodSignature = [[alipayService class] instanceMethodSignatureForSelector:@selector(payOrder:fromScheme:callback:)];

    if (methodSignature == nil) {
        return;
    }

    NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:methodSignature];
    [invocation setTarget:alipayService];
    [invocation setSelector:@selector(payOrder:fromScheme:callback:)];
    [invocation setArgument:&payOrder atIndex:2];
    [invocation setArgument:&fromScheme atIndex:3];
    [invocation setArgument:&callback atIndex:4];

    [invocation invoke];
}

#pragma clang diagnostic pop

@end
