//
//  IAPMultiLanguageUtils+PaymentSheet.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/14.
//

#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPJSONUtils.h"

@implementation IAPMultiLanguageUtils (PaymentSheet)

+ (NSString *)presetPaymentSheetText:(IAPPaymentSheetPresetText)textPosition {
    NSString *currentMatchedLanguage = [self matchedLanguageWithLanguageCodes:[self presetLanguageCodes]];
    NSString *selectorName = [@"presetPaymentSheetTextMap_" stringByAppendingString:currentMatchedLanguage];
    
    SEL sel = NSSelectorFromString(selectorName);
    if (![self respondsToSelector:sel]) {
        sel = @selector(presetPaymentSheetTextMap_en_US);
    }
    
    #pragma clang diagnostic push
    #pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    NSDictionary *presetPaymentSheetTextMap = nil;
    if ([self respondsToSelector:sel]) {
        presetPaymentSheetTextMap = [self performSelector:sel];
    } else {
        NSAssert(YES, @"error logic");
    }
    #pragma clang diagnostic pop
    
    return [presetPaymentSheetTextMap objectForKey:@(textPosition)];
}

#pragma mark - private - preset presetPaymentSheetText
+ (NSDictionary *)presetPaymentSheetTextMap_zh_CN {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"请选择您的支付方式",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"更多支付方式",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"支付总额",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"继续支付",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"请选择支付方式",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"搜索支付方式",
    
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"当前无可用支付方式，请稍后重试 Alipay+™",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"返回商户",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_zh_HK {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"請選擇您的支付方式",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"更多支付方式",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"支付總額",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"繼續支付",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"請選擇支付方式",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"搜索支付方式",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"當前無可用支付方式，請稍後重試 Alipay+™",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"返回商戶",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_en_US {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"Select your payment method",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"More payment methods",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"Total to pay",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"Go to Pay",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"Please select payment method",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"Search payment method",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"Sorry, no payment methods available. Please try again later",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"Return to Merchant",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_ko_KR {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"결제 수단을 선택하세요",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"기타 결제 수단",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"결제 총액",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"결제로 이동",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"결제 수단을 선택해주세요",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"결제 수단 검색",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"죄송합니다. 사용 가능한 결제 수단이 없습니다.나중에 다시 시도하세요",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"판매자 페이지로 돌아가기",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_th_TH {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"เลือกวิธีการชำระเงินของคุณ",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"วิธีการชำระเงินเพิ่มเติม",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"ยอดชำระเงินทั้งหมด",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"ชำระเงินเลยตอนนี้",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"โปรดเลือกวิธีการชำระเงิน",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"ค้นหาวิธีการชำระเงิน",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"ขออภัย ไม่มีวิธีการชำระเงินที่สามารถใช้งานได้ในขณะนี้ โปรดลองอีกครั้งในภายหลัง",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"กลับไปที่ร้านค้า",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_ms_MY {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"Pilih cara bayaran",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"Cara lain",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"Jumlah bayaran",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"Bayar Sekarang",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"Sila pilih cara bayaran anda",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"Cari cara bayaran",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"Harap maaf, tiada cara bayaran didapati untuk pesanan ini. Sila cuba lagi.",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"Kembali ke penjual",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_id_ID {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"Pilih metode pembayaran Anda",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"Metode pembayaran lainnya",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"Total pembayaran",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"Bayar Sekarang",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"Silakan pilih metode pembayaran Anda",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"Cari metode Pembayaran",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"Maaf, tidak ada metode pembayaran tersedia. Harap coba lagi nanti",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"Kembali ke Penjual",
    };
}

+ (NSDictionary *)presetPaymentSheetTextMap_fil_PH {
    return @{
        @(IAPPaymentSheetPresetText_Select_your_payment_method):
            @"Piliin ang iyong paraan ng pagbabayad",
        
        @(IAPPaymentSheetPresetText_More_payment_methods):
            @"Mas maraming paraan ng pagbabayad",
        
        @(IAPPaymentSheetPresetText_Total_to_pay):
            @"Kabuuang babayaran",
        
        @(IAPPaymentSheetPresetText_Go_to_Pay):
            @"Magbayad Ngayon",
        
        @(IAPPaymentSheetPresetText_Toast_Please_select_payment_method):
            @"Pakipili ang iyong paraan ng pagbabayad",
        
        @(IAPPaymentSheetPresetText_Search_payment_method):
            @"Maghanap ng paraan ng pagbabayad",
        
        @(IAPPaymentSheetPresetText_No_payment_methods_available):
            @"Paumanhin, walang magagamit na paraan ng pagbabayad. Pakisubukang muli mamaya",
        
        @(IAPPaymentSheetPresetText_Return_to_merchant):
            @"Bumalik sa Merchant",
    };
}



@end

