//
//  IAPPaymentSheetCheckoutTopView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "IAPPaymentSheetCheckoutTopView.h"
#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"
#import "IAPPaymentSheetEvent+Private.h"
#import "CAGradientLayer+IAPUtils.h"

@interface IAPPaymentSheetCheckoutTopView ()
@property (nonatomic, strong) CAGradientLayer *bgLayer;
@end

@implementation IAPPaymentSheetCheckoutTopView
- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        [self initSubViews];
    }
    return self;
}

- (void)initSubViews {
    self.backgroundColor = [IAPStyleUtils sheetCheckoutTopViewBackgroundColor];
    [self iapu_setCornerRadii:12 roundingCorners:UIRectCornerTopLeft|UIRectCornerTopRight];
//    self.clipsToBounds = YES;
    
    UIImageView *checkoutImgView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 0, 0)];
    [self addSubview:checkoutImgView];
    checkoutImgView.image = [IAPStyleUtils aPlusCheckoutImage];
    checkoutImgView.frame = CGRectMake(14, 0, 146, 16);
    checkoutImgView.iapu_centerY = self.iapu_height / 2;
    
    UIImageView *closeImgView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 0, 0)];
    [self addSubview:closeImgView];
    closeImgView.image = [IAPStyleUtils closeImage];
    closeImgView.frame = CGRectMake(0, 0, 24, 24);
    closeImgView.iapu_centerY = self.iapu_height / 2;
    closeImgView.iapu_right = self.iapu_width - 14;
    @iapu_weakify(self);
    [closeImgView iapu_touchUpInside:^{
        @iapu_strongify(self);
        if (self.closeAction) {
            self.closeAction();
        }
    }];
    
    [self updateGradientLayer];
}

- (void)dealloc {
    NSLog(@"");
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection {
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        BOOL isDiffernt = [self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection];
        if (isDiffernt) {
            [self updateGradientLayer];
        }
    }
}

- (void)updateGradientLayer {
    if (@available(iOS 13.0, *)) {
        if (self.traitCollection.userInterfaceStyle == UIUserInterfaceStyleLight) {
            [self insertGradientLayer];
        } else {
            [self.bgLayer removeFromSuperlayer];
        }
    } else {
        [self insertGradientLayer];
    }
}


- (void)insertGradientLayer {
    if (!self.bgLayer) {
        self.bgLayer = [CAGradientLayer iapu_layerWithColors:@[IAP_COLOR(#F6FAFF), IAP_COLOR(#FFFFFF)] direction:IAPGradientDirectionTop];
        self.bgLayer.frame = self.bounds;
    }
    if (!self.bgLayer.superlayer) {
        [self.layer insertSublayer:self.bgLayer atIndex:0];
    }
}

@end
