//
//  IAPPaymentSheetContentView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "IAPPaymentSheetContentView.h"
#import "IAPPaymentSheetMoreView.h"
#import "IAPPaymentSheetHideKeyBoardView.h"

#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"
#import "IAPPaymentSheetEvent+Private.h"


@interface IAPPaymentSheetContentView ()
@property (nonatomic, strong) IAPPaymentSheetMoreView *moreView;
@property (nonatomic, strong) CAShapeLayer *cornerLayer;

@property (nonatomic, assign) CGFloat originalHeight;
@property (nonatomic, assign) CGFloat originalOperationTop;
@property (nonatomic, assign) CGFloat originalSearchTop;
@property (nonatomic, assign) CGFloat originalWalletTableHeight;

@property (nonatomic, strong) IAPPaymentSheetHideKeyBoardView *hideKeyBoardView;
@property (nonatomic, assign) UIInterfaceOrientation interfaceOrientation;
@end
@implementation IAPPaymentSheetContentView
- (instancetype)initWithFrame:(CGRect)frame paymentData:(IAPPaymentData *)paymentData interfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    if (self = [super initWithFrame:frame]) {
        _paymentData = paymentData;
        _interfaceOrientation = interfaceOrientation;
        [self initSubViews];
    }
    return self;
}

- (void)preload {
    // preload, searchView is slow
    self.searchView = [[IAPPaymentSheetSearchView alloc] initWithFrame:CGRectMake(0, 0, self.iapu_width, 56)];
    @iapu_weakify(self);
    [self.searchView setHandler:^(NSString * _Nonnull text) {
        @iapu_strongify(self);
        [self updateWalletTableWhenSearchTextChanged:text];
    }];
}

- (void)initSubViews {
    self.backgroundColor = [IAPStyleUtils sheetBackgroundColor];
    
    self.checkoutTopView = [[IAPPaymentSheetCheckoutTopView alloc] initWithFrame:CGRectMake(2, 2, self.iapu_width - 4, 54)];
    [self addSubview: self.checkoutTopView];
    
    UILabel *mainTitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(16, self.checkoutTopView.iapu_bottom + 10, self.iapu_width - 32, self.paymentData.hasAvailableWallets ? 24: 48)];
    [self addSubview:mainTitleLabel];
    mainTitleLabel.numberOfLines = 2;
    mainTitleLabel.font = [IAPStyleUtils mediumFontOfSize:16];
    mainTitleLabel.adjustsFontSizeToFitWidth = YES;
    mainTitleLabel.textColor = [IAPStyleUtils sheetMainTitleTextColor];
    mainTitleLabel.text = self.paymentData.hasAvailableWallets ? [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Select_your_payment_method] : [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_No_payment_methods_available];
    
    CGFloat tableViewHeight = 168;
    if (self.interfaceOrientation == UIInterfaceOrientationLandscapeLeft || self.interfaceOrientation == UIInterfaceOrientationLandscapeRight) {
        tableViewHeight = MIN(168, MIN([UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height) - 253);
    }
    
    self.walletTableView = [[IAPPaymentWalletTableView alloc] initWithFrame:CGRectMake(0, mainTitleLabel.iapu_bottom + 13, self.iapu_width, tableViewHeight) style:UITableViewStylePlain oringinalWallets:self.paymentData.wallets interfaceOrientation:self.interfaceOrientation];
    [self addSubview:self.walletTableView];
    [self.walletTableView reloadData];
    
    if (self.paymentData.wallets.count > 3) {
        self.moreView = [[IAPPaymentSheetMoreView alloc] initWithFrame:CGRectMake(0, self.walletTableView.iapu_bottom, self.iapu_width, 60)];
        [self addSubview:self.moreView];
        
        @iapu_weakify(self);
        self.moreView.moreAction = ^{
            @iapu_strongify(self);
            [self clickMore];
        };
    }
    
    BOOL isX = ([[UIScreen mainScreen] bounds].size.height / [[UIScreen mainScreen] bounds].size.width > 2) || ([[UIScreen mainScreen] bounds].size.height / [[UIScreen mainScreen] bounds].size.width < 0.5);
    self.operationView = [[IAPPaymentSheetOperationView alloc] initWithFrame:CGRectMake(0, (self.moreView?:self.walletTableView).iapu_bottom, self.iapu_width, isX ? 92: 70) paymentData:self.paymentData];
    [self addSubview:self.operationView];
    
    
    self.hideKeyBoardView = [[IAPPaymentSheetHideKeyBoardView alloc] initWithFrame:CGRectMake(0, 0, self.iapu_width, 32)];
    self.hideKeyBoardView.iapu_top = self.operationView.iapu_top + 64;
    self.hideKeyBoardView.disabled = YES;
    [self addSubview:self.hideKeyBoardView];
    @iapu_weakify(self);
    [self.hideKeyBoardView setHideAction:^{
        @iapu_strongify(self);
        [self.searchView endEditing:YES];
    }];
    
    
    
    self.iapu_height = self.operationView.iapu_bottom;
    [self updateCorner];
    self.clipsToBounds = YES;
}


- (void)updateCorner {
    if (self.cornerLayer) {
        [self.cornerLayer removeFromSuperlayer];
        self.cornerLayer = nil;
    }
    self.cornerLayer = [self iapu_setCornerRadii:12 roundingCorners:UIRectCornerTopLeft|UIRectCornerTopRight];
}

- (void)clickMore {
    [self.moreView removeFromSuperview];
    [self.walletTableView showMore];
    if (!self.searchView) {
        self.searchView = [[IAPPaymentSheetSearchView alloc] initWithFrame:CGRectMake(0, self.walletTableView.iapu_bottom, self.iapu_width, 56)];
    } else {
        self.searchView.iapu_top = self.walletTableView.iapu_bottom;
    }
    [self addSubview:self.searchView];
    [self bringSubviewToFront:self.operationView];
    [self bringSubviewToFront:self.hideKeyBoardView];
    
    [UIView animateWithDuration:0.3 animations:^{
        self.operationView.iapu_top = self.searchView.iapu_bottom;
        self.hideKeyBoardView.iapu_top = self.operationView.iapu_top + 64;
        self.iapu_height = self.operationView.iapu_bottom;
        self.iapu_bottom = self.superview.iapu_height;
        [self updateCorner];
    } completion:^(BOOL finished) {
        self.originalHeight = self.iapu_height;
        self.originalOperationTop = self.operationView.iapu_top;
        self.originalSearchTop = self.searchView.iapu_top;
        self.originalWalletTableHeight = self.walletTableView.iapu_height;
    }];
    
    //增加监听，当键盘出现或改变时收出消息
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    //增加监听，当键退出时收出消息
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

- (void)keyboardWillShow:(NSNotification *)aNotification {
    NSDictionary *userInfo = [aNotification userInfo];
    NSValue *aValue = [userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    CGRect keyboardRect = [aValue CGRectValue];
    CGRect rect = [self convertRect:keyboardRect fromView:[UIApplication sharedApplication].keyWindow];
    CGFloat cha = (96 + 168 + 56 + 92);
    int height = rect.size.height;
    if (self.iapu_height - height < cha) {
        self.iapu_height = MIN((cha + height), self.superview.iapu_height);
    }
    
    self.operationView.iapu_top = self.iapu_height - height - 96;
    self.hideKeyBoardView.disabled = NO;
    self.hideKeyBoardView.iapu_top = self.operationView.iapu_top + 64;
    self.searchView.iapu_bottom = self.operationView.iapu_top;
    
    self.walletTableView.iapu_height = MAX(self.searchView.iapu_top - self.walletTableView.iapu_top, 56 * 3);
    self.iapu_bottom = self.superview.iapu_height;
}
 
- (void)keyboardWillHide:(NSNotification *)aNotification {
    self.hideKeyBoardView.disabled = YES;
    
    self.iapu_height = self.originalHeight;
    self.iapu_bottom = self.superview.iapu_height;
    self.operationView.iapu_top = self.originalOperationTop;
    self.searchView.iapu_top = self.originalSearchTop;
    self.walletTableView.iapu_height = self.originalWalletTableHeight;
    self.hideKeyBoardView.iapu_top = self.operationView.iapu_top + 64;
}

- (void)updateWalletTableWhenSearchTextChanged:(NSString *)text {
    [self.walletTableView updateWhenSearchTextChanged:text];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    NSLog(@"");
}

@end
