//
//  IAPPaymentSheetOperationView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/21.
//

#import "IAPPaymentSheetOperationView.h"

#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"


@interface IAPPaymentSheetOperationView ()
@property (nonatomic, strong) IAPPaymentData *paymentData;
@property (nonatomic, strong) UIView *returnToMerchantButton;
@end


@implementation IAPPaymentSheetOperationView

- (instancetype)initWithFrame:(CGRect)frame paymentData:(IAPPaymentData *)paymentData {
    if (self = [super initWithFrame:frame]) {
        _paymentData = paymentData;
        [self initSubviews];
    }
    
    return self;
}

- (void)initSubviews {
    self.backgroundColor = [IAPStyleUtils sheetOperationViewBackgroundColor];
    self.layer.shadowOffset = CGSizeMake(3, -3);
    self.layer.shadowOpacity = 0.07;
    
    if (self.paymentData.hasAvailableWallets) {
        [self initSubviewWithAvailableWallets];
    } else {
        [self initSubviewWithNoAvailableWallets];
    }
}

- (void)dealloc {
    NSLog(@"");
}


- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection {
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        BOOL isDiffernt = [self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection];
        if (isDiffernt) {
            [self updateShadow];
            [self updateReturnButtonLayer];
        }
    }
}

- (void)updateReturnButtonLayer {
    if (!self.returnToMerchantButton) {
        return;
    }
    
    if (@available(iOS 13.0, *)) {
        self.returnToMerchantButton.layer.borderColor = [[IAPStyleUtils sheetReturnButtonBorderLayerColor] resolvedColorWithTraitCollection:self.traitCollection].CGColor;
    } else {
        self.returnToMerchantButton.layer.borderColor = [IAPStyleUtils sheetReturnButtonBorderLayerColor].CGColor;
    }
}


- (void)updateShadow {
    if (@available(iOS 13.0, *)) {
        self.layer.shadowColor = [[IAPStyleUtils shadowColor] resolvedColorWithTraitCollection:self.traitCollection].CGColor;
    } else {
        self.layer.shadowColor = [IAPStyleUtils shadowColor].CGColor;
    }
}

- (void)initSubviewWithAvailableWallets {
    UILabel *totalTitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(16, 12, 150, 16)];
    [self addSubview:totalTitleLabel];
    totalTitleLabel.font = [IAPStyleUtils fontOfSize:12];
    totalTitleLabel.textColor = [IAPStyleUtils sheetTotalTitleTextColor];
    totalTitleLabel.text = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Total_to_pay];
    
    UILabel *totalValieLabel = [[UILabel alloc] initWithFrame:CGRectMake(16, 28, 100, 31)];
    [self addSubview:totalValieLabel];
    totalValieLabel.adjustsFontSizeToFitWidth = YES;
    totalValieLabel.font = [IAPStyleUtils boldFontOfSize:20];
    totalValieLabel.textColor = [IAPStyleUtils sheetTotalValueTextColor];
    totalValieLabel.text = [NSString stringWithFormat:@"%@ %@", _paymentData.displayPaymentCurrency, _paymentData.displayPaymentAmount];
    
    UILabel *payView = [[UILabel alloc] initWithFrame:CGRectMake(0, 12, 160, 44)];
    [self addSubview:payView];
    if ([[IAPMultiLanguageUtils formatedCurrentLauguage] isEqualToString:@"fil_PH"]) {
        payView.font = [IAPStyleUtils mediumFontOfSize:14];
    } else {
        payView.font = [IAPStyleUtils mediumFontOfSize:17];
    }
    payView.textAlignment = NSTextAlignmentCenter;
    payView.backgroundColor = [IAPStyleUtils sheetPayButtonBackgroundColor];
    payView.textColor = [IAPStyleUtils sheetPayButtonTextColor];
    payView.text = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    payView.layer.cornerRadius = 22;
    payView.clipsToBounds = YES;
    payView.adjustsFontSizeToFitWidth = YES;
    payView.iapu_right = self.iapu_width - 16;
    totalValieLabel.iapu_width = payView.iapu_left - 10 - totalValieLabel.iapu_left;
    @iapu_weakify(self);
    [payView iapu_touchUpInside:^{
        @iapu_strongify(self);
        if (self.goToPayAction) {
            self.goToPayAction();
        }
    }];
}

- (void)initSubviewWithNoAvailableWallets {
    UILabel *returnToMerchantButton = [[UILabel alloc] initWithFrame:CGRectMake(20, 12, self.iapu_width - 40, 44)];
    [self addSubview:returnToMerchantButton];
    self.returnToMerchantButton = returnToMerchantButton;
    returnToMerchantButton.font = [IAPStyleUtils mediumFontOfSize:16];
    returnToMerchantButton.textAlignment = NSTextAlignmentCenter;
    returnToMerchantButton.backgroundColor = [UIColor whiteColor];
    returnToMerchantButton.textColor = [IAPStyleUtils sheetReturnButtonTextColor];
    returnToMerchantButton.text = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Return_to_merchant];
    returnToMerchantButton.layer.cornerRadius = 22;
    returnToMerchantButton.clipsToBounds = YES;
    returnToMerchantButton.layer.borderWidth = 1;
    returnToMerchantButton.adjustsFontSizeToFitWidth = YES;
    @iapu_weakify(self);
    [returnToMerchantButton iapu_touchUpInside:^{
        @iapu_strongify(self);
        if (self.returnToMerchantAction) {
            self.returnToMerchantAction();
        }
    }];
    
    [self updateReturnButtonLayer];
}

@end
