//
//  IAPPaymentSheetSearchView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/21.
//

#import "IAPPaymentSheetSearchView.h"

#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"

@interface IAPSearchTextField: UITextField
@end
@implementation IAPSearchTextField

- (CGRect)placeholderRectForBounds:(CGRect)bounds {
    return CGRectMake(34, 8, self.iapu_width - 100, 16);
}

- (CGRect)editingRectForBounds:(CGRect)bounds {
    return [self textRectForBounds:bounds];
}

- (CGRect)leftViewRectForBounds:(CGRect)bounds {
    return CGRectMake(13, 10, 13, 13);
}

- (CGRect)rightViewRectForBounds:(CGRect)bounds {
    return CGRectMake(self.iapu_width - 13 - 13, 10, 13, 13);
}

- (CGRect)textRectForBounds:(CGRect)bounds {
    CGRect superRect = [super textRectForBounds:bounds];
    return CGRectMake(superRect.origin.x + 8.5, superRect.origin.y, superRect.size.width - 35, superRect.size.height);
}

@end

@interface IAPPaymentSheetSearchView () <UITextFieldDelegate>
@property (nonatomic, strong) IAPSearchTextField *searchBar;
@end
@implementation IAPPaymentSheetSearchView

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        [self initSubviews];
    }
    return self;
}


- (void)initSubviews {
    self.backgroundColor = [IAPStyleUtils sheetSearchFieldsViewBackgroundColor];
    self.layer.shadowOffset = CGSizeMake(1.5, -1.5);
    self.layer.shadowOpacity = 0.07;
    
    IAPSearchTextField *searchBar = [[IAPSearchTextField alloc] initWithFrame:CGRectMake(16, 11, self.iapu_width - 32, 32)];
    [self addSubview:searchBar];
    self.searchBar = searchBar;
    searchBar.backgroundColor = [IAPStyleUtils sheetSearchFieldBackgroundColor];
    searchBar.borderStyle = UITextBorderStyleNone;
    searchBar.layer.cornerRadius = 16;
    searchBar.clipsToBounds = YES;
    searchBar.font = [IAPStyleUtils fontOfSize:12];
    searchBar.textColor = [IAPStyleUtils sheetSearchFieldTextColor];
    searchBar.returnKeyType = UIReturnKeySearch;
    searchBar.tintColor = IAP_COLOR(#00B6FF);
    self.searchBar.attributedPlaceholder = [[NSAttributedString alloc] initWithString:[IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Search_payment_method]?:@"" attributes:@{NSFontAttributeName:[IAPStyleUtils fontOfSize:12], NSForegroundColorAttributeName:[IAPStyleUtils sheetSearchFieldPlaceholderTextColor]}];
    
    UIImageView *leftView = [[UIImageView alloc] initWithFrame:CGRectZero];
    leftView.image = [IAPStyleUtils sheetSearchLeftImage];
    searchBar.leftView = leftView;
    searchBar.leftViewMode = UITextFieldViewModeAlways;
    
    UIImageView *rightView = [[UIImageView alloc] initWithFrame:CGRectZero];
    
    rightView.image = [IAPStyleUtils sheetSearchRightImage];
    searchBar.rightView = rightView;
    @iapu_weakify(self);
    [rightView iapu_touchUpInside:^{
        @iapu_strongify(self);
        self.searchBar.text = @"";
        NSNotification *notification = [NSNotification notificationWithName:UITextFieldTextDidChangeNotification object:self.searchBar];
        [self searchBarTextDidChange:notification];
    }];
    
    searchBar.delegate = self;

    
    //增加监听，当键盘出现或改变时收出消息
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(searchBarTextDidChange:)
                                                 name:UITextFieldTextDidChangeNotification
                                               object:nil];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    NSLog(@"");
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection {
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        BOOL isDiffernt = [self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection];
        if (isDiffernt) {
            [self updateShadow];
        }
    }
}


- (void)updateShadow {
    if (@available(iOS 13.0, *)) {
        self.layer.shadowColor = [[IAPStyleUtils shadowColor] resolvedColorWithTraitCollection:self.traitCollection].CGColor;
    } else {
        self.layer.shadowColor = [IAPStyleUtils shadowColor].CGColor;
    }
}

#pragma mark- UISearchBarDelegate

- (void)textFieldDidBeginEditing:(UITextField *)textField {
    self.searchBar.attributedPlaceholder = [[NSAttributedString alloc] initWithString:[IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Search_payment_method]?:@"" attributes:@{NSFontAttributeName:[IAPStyleUtils fontOfSize:12], NSForegroundColorAttributeName:IAP_COLOR_A(#98A6BB, 0.5)}];
    
    if (self.searchBar.text.length) {
        self.searchBar.rightViewMode = UITextFieldViewModeWhileEditing;
    } else {
        self.searchBar.rightViewMode = UITextFieldViewModeNever;
    }
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    self.searchBar.attributedPlaceholder = [[NSAttributedString alloc] initWithString:[IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Search_payment_method]?:@"" attributes:@{NSFontAttributeName:[IAPStyleUtils fontOfSize:12], NSForegroundColorAttributeName:[IAPStyleUtils sheetSearchFieldPlaceholderTextColor]}];
}


- (void)searchBarTextDidChange:(NSNotification *)notification {
    
    if (notification.object != self.searchBar) {
        return;
    }
    
    NSString *searchText = self.searchBar.text;
    if (searchText.length) {
        self.searchBar.rightViewMode = UITextFieldViewModeWhileEditing;
    } else {
        self.searchBar.rightViewMode = UITextFieldViewModeNever;
    }
    
    if (self.handler) {
        if (searchText.length > 500) {
            searchText = [searchText substringToIndex:499];
        }
        self.handler(searchText);
    }
}

@end
