//
//  IAPPaymentWalletTableCell.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "IAPPaymentWalletTableCell.h"

#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"

@interface IAPPaymentWalletTableCell ()
@property (nonatomic, strong) UIImageView *logoView;
@property (nonatomic, strong) UILabel *tagLabel;
@property (nonatomic, strong) UIImageView *statusView;
@property (nonatomic, strong) UILabel *placeHolderLogoLabel;
@property (nonatomic, strong) UILabel *disabledLabel;
@end

@implementation IAPPaymentWalletTableCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        [self initSubviews];
    }
    return self;
}

- (void)setWallet:(IAPPaymentWalletData *)wallet {
    if (_wallet == wallet) {
        return;
    }
    
    _wallet = wallet;
    _wallet.bizTableTableCell = self;
    
    [self updateLogo];
    [self updateTagText];
    [self updateSelectedStatus];
}

- (UILabel *)disabledLabel {
    if (!_disabledLabel) {
        _disabledLabel = [[UILabel alloc] initWithFrame:CGRectMake(self.logoView.iapu_right + 12, 12, self.statusView.iapu_left - self.logoView.iapu_right - 12, 32)];
        _disabledLabel.font = [IAPStyleUtils fontOfSize:11];
        _disabledLabel.textColor = [IAPStyleUtils sheetDisableTagTextColor];
    }
    return _disabledLabel;
}



- (void)initSubviews {
    self.iapu_width = [UIScreen mainScreen].bounds.size.width;
    self.backgroundColor = [UIColor clearColor];
    
    self.selectedBackgroundView= [[UIView alloc] initWithFrame:self.bounds];
    self.selectedBackgroundView.backgroundColor = [UIColor clearColor];
    
    self.logoView = [[UIImageView alloc] initWithFrame:CGRectMake(15, 12, 75, 33)];
    [self addSubview:self.logoView];
    self.logoView.backgroundColor = IAP_COLOR(#FCFCFC);
    self.logoView.layer.cornerRadius = 6;
    self.logoView.clipsToBounds = YES;
    self.logoView.layer.borderWidth = 1;
    [self updateLogoLayerColor];
    
    self.tagLabel = [[UILabel alloc] initWithFrame:CGRectMake(self.logoView.iapu_right + 12, 19.5, 0, 18)];
    [self addSubview:self.tagLabel];
    self.tagLabel.font = [IAPStyleUtils boldFontOfSize:10];
    self.tagLabel.textColor = [IAPStyleUtils sheetTagTextColor];
    self.tagLabel.backgroundColor = [IAPStyleUtils sheetTagBackgroundColor];
    self.tagLabel.layer.cornerRadius = 3;
    self.tagLabel.clipsToBounds = YES;
    self.tagLabel.layer.borderWidth = 1;
    self.tagLabel.textAlignment = NSTextAlignmentCenter;
    [self updateTagLayerColor];
    
    self.statusView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 14, 28, 28)];
    [self addSubview:self.statusView];
    self.statusView.iapu_right = self.iapu_width - 14;
    
    UIView *line = [[UIView alloc] initWithFrame:CGRectMake(15, 55, self.iapu_width - 30, 1)];
    [self addSubview:line];
    line.backgroundColor = [IAPStyleUtils sheetTableSeparateLineColor];
}



- (void)updateTagLayerColor {
    if (@available(iOS 13.0, *)) {
        self.tagLabel.layer.borderColor = [[IAPStyleUtils sheetTagBorderLayerColor] resolvedColorWithTraitCollection:self.traitCollection].CGColor;
    } else {
        self.tagLabel.layer.borderColor = [IAPStyleUtils sheetTagBorderLayerColor].CGColor;
    }
}

- (void)updateLogoLayerColor {
    if (@available(iOS 13.0, *)) {
        self.logoView.layer.borderColor = [[IAPStyleUtils sheetLogoBorderLayerColor] resolvedColorWithTraitCollection:self.traitCollection].CGColor;
    } else {
        self.logoView.layer.borderColor = [IAPStyleUtils sheetLogoBorderLayerColor].CGColor;
    }
}


- (void)updateSelectedStatus {
    IAPPaymentWalletData *wallet = self.wallet;
    if (!wallet.enabled) {
        self.statusView.image = [IAPStyleUtils walletDisableImage];
    } else {
        if (wallet.bizSelected) {
            self.statusView.image = [IAPStyleUtils walletIsSelectedImage];
        } else {
            self.statusView.image = [IAPStyleUtils walletIsNotSelectedImage];
        }
    }
}

- (void)updateTagText {
    IAPPaymentWalletData *wallet = self.wallet;
    if (wallet.enabled) {
        if (wallet.promoNames.count) {
            self.tagLabel.hidden = NO;
            self.tagLabel.text = wallet.promoNames.firstObject;
        } else {
            self.tagLabel.text = @"";
            self.tagLabel.hidden = YES;
        }
        NSAttributedString *string = [[NSAttributedString alloc] initWithString:self.tagLabel.text attributes:@{NSFontAttributeName: self.tagLabel.font}];
        CGFloat tagWith = MIN(string.size.width + 12, self.statusView.iapu_left - self.tagLabel.iapu_left - 10);
        self.tagLabel.iapu_width = tagWith;
    
        
        if (_disabledLabel) {
            self.disabledLabel.text = @"";
            self.logoView.alpha = 1;
            [_disabledLabel removeFromSuperview];
        }
    } else {
        self.tagLabel.hidden = YES;
        
        [self addSubview:self.disabledLabel];
        self.logoView.alpha = 0.7;
        self.disabledLabel.text = wallet.disableReason ?: @"Not supported at this store";
    }
}

- (void)updateLogo {
    self.placeHolderLogoLabel.hidden = YES;
    IAPPaymentWalletData *wallet = self.wallet;
    @iapu_weakify(self);
    @iapu_weakify(wallet);
    __block BOOL hasPlaceHolder = NO;
    [wallet bizAsyncFetchLogoWithCallback:^(UIImage * _Nullable logo, BOOL isPlaceHolder) {
        @iapu_strongify(self);
        @iapu_strongify(wallet);
        if (self && wallet && (self == wallet.bizTableTableCell)) {
            if (logo) {
                self.logoView.image = logo;
                self.placeHolderLogoLabel.hidden = YES;
                if (isPlaceHolder) {
                    hasPlaceHolder = isPlaceHolder;
                }
            } else {
                if (!hasPlaceHolder) {
                    [self showPlaceHolderLogoLabel];
                }
            }
        }
    }];
}

#pragma - mark override

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection {
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        BOOL isDiffernt = [self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection];
        if (isDiffernt) {
            [self updateTagLayerColor];
            [self updateLogoLayerColor];
        }
    }
}



- (void)dealloc {
    NSLog(@"");
}

- (void)showPlaceHolderLogoLabel{
    if (!self.placeHolderLogoLabel) {
        self.placeHolderLogoLabel = [[UILabel alloc] initWithFrame:CGRectMake(self.logoView.iapu_left + 1, self.logoView.iapu_top + 1, self.logoView.iapu_width - 2, self.logoView.iapu_height - 2)];
        [self addSubview:self.placeHolderLogoLabel];
        self.placeHolderLogoLabel.textColor = [UIColor blackColor];
        self.placeHolderLogoLabel.textAlignment = NSTextAlignmentCenter;
        self.placeHolderLogoLabel.font = [IAPStyleUtils mediumFontOfSize:12];
        self.placeHolderLogoLabel.adjustsFontSizeToFitWidth = YES;
    }
    
    self.placeHolderLogoLabel.hidden = NO;
    self.placeHolderLogoLabel.text = self.wallet.walletBrandName;
    self.logoView.image = nil;
}

@end
