//
//  IAPPaymentWalletTableView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "IAPPaymentWalletTableView.h"
#import "IAPPaymentWalletTableCell.h"

#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"
#import "IAPWalletSearchUtils.h"
#import "IAPLocalStorage.h"
#import "AlipayPlusClientAPI.h"

@interface IAPPaymentWalletTableView() <UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) NSArray<IAPPaymentWalletData *> *originalWallets;
@property (nonatomic, strong, readwrite) NSArray<IAPPaymentWalletData *> *dataSourceWallets;

@property (nonatomic, assign) BOOL hasShowMore;
@property (nonatomic, strong, readwrite) IAPPaymentWalletData *selectedWallet;
@property (nonatomic, strong, readwrite) NSString *selectedWalletName;
@property (nonatomic, assign) UIInterfaceOrientation interfaceOrientation;
@end

@implementation IAPPaymentWalletTableView

- (instancetype)initWithFrame:(CGRect)frame style:(UITableViewStyle)style oringinalWallets:(NSArray<IAPPaymentWalletData *> *)originalWallets interfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    if (self = [super initWithFrame:frame style:style]) {
        _originalWallets = originalWallets;
        _interfaceOrientation = interfaceOrientation;
        self.backgroundColor = [UIColor clearColor];
        self.separatorStyle = UITableViewCellSeparatorStyleNone;
        [self registerClass:IAPPaymentWalletTableCell.class forCellReuseIdentifier:@"IAPPaymentWalletTableCellID"];
        self.dataSource = self;
        self.delegate = self;
        
        
        [self initDataSourceWallets];
    }
    return self;
}

- (void)initDataSourceWallets {
    self.selectedWalletName = [[NSString alloc] initWithData:[[IAPLocalStorage shared] dataForKey:@"IAPLocalStorage.last.selected.wallet"] encoding:NSUTF8StringEncoding];
   
    if (self.selectedWalletName.length) {
        NSArray *walletNames = [self.originalWallets valueForKey:@"walletName"];
        if ([walletNames containsObject:self.selectedWalletName]) {
            __block IAPPaymentWalletData *lastSelectedWallet = nil;
            NSMutableArray<IAPPaymentWalletData *> *newOriginalWallets = [self.originalWallets mutableCopy];
            [newOriginalWallets enumerateObjectsUsingBlock:^(IAPPaymentWalletData * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                if ([obj.walletName isEqualToString:self.selectedWalletName]) {
                    lastSelectedWallet = obj;
                    *stop = YES;
                }
            }];
            
            if (lastSelectedWallet) {
                [newOriginalWallets removeObject:lastSelectedWallet];
                [newOriginalWallets insertObject:lastSelectedWallet atIndex:0];
                if (lastSelectedWallet.enabled) {
                    _selectedWallet = lastSelectedWallet;
                    _selectedWallet.bizSelected = YES;
                    self.selectedWalletName = lastSelectedWallet.walletName;
                }
            }
            self.originalWallets = newOriginalWallets;
        }
       
    }
    
    if (self.originalWallets.count > 3) {
        _dataSourceWallets = [self.originalWallets subarrayWithRange:NSMakeRange(0, 3)];
    } else {
        _dataSourceWallets = self.originalWallets;
    }
}

- (void)setSelectedWallet:(IAPPaymentWalletData *)selectedWallet {
    if (selectedWallet == self.selectedWallet || !selectedWallet.enabled) {
        return;
    }
    
    self.selectedWallet.bizSelected = NO;
    IAPPaymentWalletTableCell *lastCell = self.selectedWallet.bizTableTableCell;
    [lastCell updateSelectedStatus];
    
    _selectedWallet = selectedWallet;
    self.selectedWallet.bizSelected = YES;
    IAPPaymentWalletTableCell *nowCell = self.selectedWallet.bizTableTableCell;
    [nowCell updateSelectedStatus];
    
    self.selectedWalletName = selectedWallet.walletName;
}

- (void)updateWhenSearchTextChanged:(NSString *)text {
    if (!text.length) {
        self.dataSourceWallets = self.originalWallets;
    } else {
        self.dataSourceWallets = [IAPWalletSearchUtils mostMatchedWalletsWithSearchWord:text fromOriginalWallets:self.originalWallets];
    }
    [self reloadData];
}


#pragma mark - UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 56;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    IAPPaymentWalletData *wallet = self.dataSourceWallets[indexPath.row];
    self.selectedWallet = wallet;
}

#pragma mark - UITableViewDataSource

- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    IAPPaymentWalletTableCell *cell = [tableView dequeueReusableCellWithIdentifier:@"IAPPaymentWalletTableCellID" forIndexPath:indexPath];
    cell.wallet = self.dataSourceWallets[indexPath.row];
    return cell;
}

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataSourceWallets.count;
}

- (void)showMore {
    if (self.originalWallets.count <= 3 || self.hasShowMore) {
        return;
    }
    
    self.hasShowMore = YES;
    self.dataSourceWallets = self.originalWallets;
    
   
    CGFloat ScreenHeight = MAX([UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height);
    CGFloat sheetMaxScreenPercentValue = 0.8;
    if (self.interfaceOrientation == UIInterfaceOrientationLandscapeLeft || self.interfaceOrientation == UIInterfaceOrientationLandscapeRight) {
        ScreenHeight = MIN([UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height);
        sheetMaxScreenPercentValue = 1;
    }
    
    self.iapu_height = MIN(self.originalWallets.count * 56, MAX(ScreenHeight * sheetMaxScreenPercentValue - 253, self.iapu_height));
    [self reloadData];
}

- (void)dealloc {
    NSLog(@"");
}

@end
