//
//  IAPWalletSearchUtils.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/21.
//

#import "IAPWalletSearchUtils.h"

@interface IAPWalletMatchedRecord:NSObject
@property (nonatomic, assign) BOOL isContained;
@property (nonatomic, assign) BOOL hasPrefix;
@property (nonatomic, strong) NSString *containedOfString;

@property (nonatomic, assign) float maxMatchedScore;
@property (nonatomic, strong) NSString *maxMatchedScoreString;
@property (nonatomic, strong) id userInfo;
@end

@implementation IAPWalletMatchedRecord
@end

@implementation IAPWalletSearchUtils

+ (NSArray<IAPPaymentWalletData *> *)mostMatchedWalletsWithSearchWord:(NSString *)searchWord fromOriginalWallets:(NSArray<IAPPaymentWalletData *> *)oringinalWallets {
    NSMutableArray<IAPWalletMatchedRecord *> *lowPriorityRecords = [NSMutableArray array];
    NSMutableArray<IAPWalletMatchedRecord *> *highPriorityRecords = [NSMutableArray array];
    NSMutableArray<IAPWalletMatchedRecord *> *highestPriorityRecords = [NSMutableArray array];
    
    NSMutableArray *mostMatchedWallets = [NSMutableArray array];
    
    [oringinalWallets enumerateObjectsUsingBlock:^(IAPPaymentWalletData * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSMutableArray *keywords = [obj.searchKeywords mutableCopy] ?: [NSMutableArray array];
        if (obj.walletName.length) {
            [keywords addObject:obj.walletName];
        }
        
        if (obj.walletBrandName.length) {
            [keywords addObject:obj.walletBrandName];
        }
        
        IAPWalletMatchedRecord *record = [self maxMatchedScoreFromString:searchWord withStrings:keywords];
        if (record.maxMatchedScore > 0) {
            record.userInfo = obj;
            if (record.isContained) {
                if (record.hasPrefix) {
                    [highestPriorityRecords addObject:record];
                } else {
                    [highPriorityRecords addObject:record];
                }
            } else {
                [lowPriorityRecords addObject:record];
            }
        }
        
    }];
    
    NSInteger leftCount = MIN(3 - mostMatchedWallets.count, highestPriorityRecords.count);
    [highestPriorityRecords sortUsingComparator:^NSComparisonResult(IAPWalletMatchedRecord * _Nonnull obj1, IAPWalletMatchedRecord * _Nonnull obj2) {
        return obj2.maxMatchedScore > obj1.maxMatchedScore;
    }];
    for (NSInteger i = 0; i < leftCount; i++) {
        [mostMatchedWallets addObject:highestPriorityRecords[i].userInfo];
    }
    
    leftCount = MIN(3 - mostMatchedWallets.count, highPriorityRecords.count);
    if (leftCount) {
        [highPriorityRecords sortUsingComparator:^NSComparisonResult(IAPWalletMatchedRecord * _Nonnull obj1, IAPWalletMatchedRecord * _Nonnull obj2) {
            return obj2.maxMatchedScore > obj1.maxMatchedScore;
        }];
       
        for (NSInteger i = 0; i < leftCount; i++) {
            [mostMatchedWallets addObject:highPriorityRecords[i].userInfo];
        }
    }
    
    leftCount = MIN(3 - mostMatchedWallets.count, lowPriorityRecords.count);
    if (leftCount > 0) {
        [lowPriorityRecords sortUsingComparator:^NSComparisonResult(IAPWalletMatchedRecord * _Nonnull obj1, IAPWalletMatchedRecord * _Nonnull obj2) {
            return obj2.maxMatchedScore > obj1.maxMatchedScore;
        }];
        
        for (NSInteger i = 0; i < leftCount; i++) {
            [mostMatchedWallets addObject:lowPriorityRecords[i].userInfo];
        }
    }
    
    return mostMatchedWallets;
}


#pragma mark - private

+ (IAPWalletMatchedRecord *)maxMatchedScoreFromString:(NSString *)originalString withStrings:(NSArray<NSString *> *)strings {
    __block BOOL isContained = NO;
    __block BOOL hasPrefix = NO;
    __block float maxMatchedScore = 0.0;
    __block NSString *containedOfString = nil;
    __block NSString *maxMatchedScoreString = nil;
    originalString = originalString.lowercaseString;
    [strings enumerateObjectsUsingBlock:^(NSString * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        obj = obj.lowercaseString;
        CGFloat objScore = [self matchedScoreFromString:originalString withString:obj];
        if (objScore > maxMatchedScore) {
            maxMatchedScore = objScore;
            maxMatchedScoreString = obj;
        }
        
        if (!isContained && [obj containsString:originalString]) {
            isContained = YES;
            hasPrefix = [obj hasPrefix:originalString];
            containedOfString = obj;
        }
    }];
    
    IAPWalletMatchedRecord *record = [IAPWalletMatchedRecord new];
    record.isContained = isContained;
    record.hasPrefix = hasPrefix;
    record.maxMatchedScore = maxMatchedScore;
    record.containedOfString = containedOfString;
    record.maxMatchedScoreString = maxMatchedScoreString;
    return record;
}

+ (float)matchedScoreFromString:(NSString *)originalString withString:(NSString *)comparisonString {
    float levenshteinDistance =  [self compareString:originalString withString:comparisonString];
    return (1 - levenshteinDistance/(MAX(originalString.length, comparisonString.length)));
}

+ (float)compareString:(NSString *)originalString withString:(NSString *)comparisonString {
    // Normalize strings
    [originalString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    [comparisonString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];

    originalString = [originalString lowercaseString];
    comparisonString = [comparisonString lowercaseString];

    // Step 1 (Steps follow description at http://www.merriampark.com/ld.htm)
    NSInteger k, i, j, cost, * d, distance;

    NSInteger n = [originalString length];
    NSInteger m = [comparisonString length];

    if( n++ != 0 && m++ != 0 ) {

        d = malloc( sizeof(NSInteger) * m * n );

        // Step 2
        for( k = 0; k < n; k++)
            d[k] = k;

        for( k = 0; k < m; k++)
            d[ k * n ] = k;

        // Step 3 and 4
        for( i = 1; i < n; i++ )
            for( j = 1; j < m; j++ ) {

                // Step 5
                if( [originalString characterAtIndex: i-1] ==
                   [comparisonString characterAtIndex: j-1] )
                    cost = 0;
                else
                    cost = 1;

                // Step 6
                d[ j * n + i ] = [self smallestOf: d [ (j - 1) * n + i ] + 1
                andOf: d[ j * n + i - 1 ] +  1
                andOf: d[ (j - 1) * n + i - 1 ] + cost ];

                // This conditional adds Damerau transposition to Levenshtein distance
                if( i>1 && j>1 && [originalString characterAtIndex: i-1] ==
                    [comparisonString characterAtIndex: j-2] &&
                    [originalString characterAtIndex: i-2] ==
                    [comparisonString characterAtIndex: j-1] )
                {
                    d[ j * n + i] = [self smallestOf: d[ j * n + i ]
                    andOf: d[ (j - 2) * n + i - 2 ] + cost ];
                }
            }

        distance = d[ n * m - 1 ];

        free( d );

        return distance;
    }
    return 0.0;
}

// Return the minimum of a, b and c - used by compareString:withString:
+ (NSInteger)smallestOf:(NSInteger)a andOf:(NSInteger)b andOf:(NSInteger)c {
    NSInteger min = a;
    if ( b < min )
        min = b;

    if( c < min )
        min = c;

    return min;
}

+ (NSInteger)smallestOf:(NSInteger)a andOf:(NSInteger)b {
    NSInteger min=a;
    if (b < min)
        min=b;

    return min;
}
@end
