//
//  IAPPaymentOption.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/15.
//

#import "IAPPaymentOption.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPLogo+Private.h"
#import "NSDictionary+IAPUtils.h"

@interface IAPPaymentOption()
@property (nonatomic, assign) BOOL fromCache;
@property (nonatomic, strong) NSArray<NSString *> *originalServerPromoNames;
@property (nonatomic, strong) NSArray<NSDictionary *> *originalLogos;
@end

@implementation IAPPaymentOption
#pragma mark - private
IAP_ENCODE_Selector
IAP_DECODE_Selector

- (nullable instancetype)initWithJSONDictionary:(NSDictionary *)json currency:(NSString *)currency {
    return [self initWithJSONDictionary:json];
}

- (nullable instancetype)initWithJSONDictionary:(NSDictionary *)json {
    if (![json isKindOfClass:NSDictionary.class]) {
        return nil;
    }
    
    if (self = [super init]) {
        _enabled = [json iapu_boolForKey:@"enabled"];
        _paymentMethodType = [json iapu_stringForKey:@"paymentMethodType"];
        
        if (_enabled) {
            self.originalLogos = [json iapu_arrayForKey:@"logos"];
            _brandName = [json iapu_stringForKey:@"brandName"];
            self.originalServerPromoNames = [json iapu_arrayForKey:@"promoNames"];
        } else {
            _disableReason = [json iapu_stringForKey:@"disableReason"] ?: @"NO_PAYMENT_OPTIONS";
        }
    }
    return self;
}

- (void)markFromCache {
    self.fromCache = YES;
    _promoNames = nil;
    _disableReason = nil;
}


- (void)setOriginalServerPromoNames:(NSArray<NSString *> *)originalServerPromoNames {
    _originalServerPromoNames = originalServerPromoNames;
    _promoNames = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:originalServerPromoNames];
}

- (void)setOriginalLogos:(NSArray<NSDictionary *> *)originalLogos {
    _originalLogos = originalLogos;
    NSMutableArray *logos = [NSMutableArray arrayWithCapacity:originalLogos.count];
    [originalLogos enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        IAPLogo *logo = [[IAPLogo alloc] initWithJSONDictionary:obj];
        if (logo) {
            [logos addObject:logo];
        }
    }];
    _logos = logos.mutableCopy;
}

@end
