//
//  IAPInquirePaymentOptionSponsorTests.m
//  AlipayPlusClientTests
//
//  Created by assuner on 2022/3/16.
//

#import <XCTest/XCTest.h>
#import "IAPInquirePaymentOptionSponsor.h"
#import "AlipayPlusClientAPI.h"
#import "IAPPaymentOption.h"
#import "IAPInquirePaymentOptionParams.h"
#import "IAPLocalStorage.h"

@interface IAPLocalStorage (Test2)
- (NSCache *)memoryCache;
- (NSUserDefaults *)diskCache;
- (BOOL)isConfirmedNSCoding:(id)object;
- (id<NSCoding>)memoryCodingObjectForKey:(NSString *)key;
- (id<NSCoding>)diskCodingObjectForKey:(NSString *)key;
@end


@interface IAPInquirePaymentOptionSponsorTests : XCTestCase

@end

@implementation IAPInquirePaymentOptionSponsorTests

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testInquireOption1 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:[IAPInquirePaymentOptionParams new] completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        if (error.code == IAPErrorParamIllegal) {
            [exp fulfill];
        }
    }];
    
    [self waitForExpectationsWithTimeout:0.01 handler:^(NSError * _Nullable error) {
        
    }];
}


- (void)testInquireOption2 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.acquirerId = @"123";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:[IAPInquirePaymentOptionParams new] completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        if (error.code == IAPErrorParamIllegal) {
            [exp fulfill];
        }
    }];
    
    [self waitForExpectationsWithTimeout:0.01 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
}

- (void)testInquireOption3 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.acquirerId = @"123";
    [AlipayPlusClient shared].configuration.merchantId = @"abc";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:[IAPInquirePaymentOptionParams new] completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        if (error.code == IAPErrorParamIllegal) {
            [exp fulfill];
        }
    }];
    
    [self waitForExpectationsWithTimeout:0.01 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
    [AlipayPlusClient shared].configuration.merchantId = @"";
}




- (void)testInquireOption4 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.acquirerId = @"123";
    [AlipayPlusClient shared].configuration.merchantId = @"abc";
    IAPInquirePaymentOptionParams *params = [IAPInquirePaymentOptionParams new];
    params.paymentCurrency = @"USD";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:params completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        if (!paymentOption.enabled && [paymentOption.disableReason isEqualToString:@"NO_PAYMENT_OPTIONS"]) {
            [exp fulfill];
        }
    }];
    
    [self waitForExpectationsWithTimeout:3 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
    [AlipayPlusClient shared].configuration.merchantId = @"";
}

- (void)testInquireOption41 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.acquirerId = @"1    23 ";
    [AlipayPlusClient shared].configuration.merchantId = @"ab c";
    IAPInquirePaymentOptionParams *params = [IAPInquirePaymentOptionParams new];
    params.paymentCurrency = @"USD";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:params completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        if (error.code == IAPErrorParamIllegal) {
            [exp fulfill];
        }
    }];
    
    [self waitForExpectationsWithTimeout:3 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
    [AlipayPlusClient shared].configuration.merchantId = @"";
}

- (void)testInquireOption5 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.acquirerId = @"A071231400000002";
    [AlipayPlusClient shared].configuration.merchantId = @"UNIFIED_dev_90001";
    IAPInquirePaymentOptionParams *params = [IAPInquirePaymentOptionParams new];
    params.paymentCurrency = @"USD";
    params.logoPattern = @"P1-APP-01";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:params completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        BOOL result = paymentOption.enabled;
        NSString *key = [NSString stringWithFormat:@"IAPPaymentOptionCache_%@_%@_%@_%@_%@_%@", @"CASHIER_PAYMENT", [AlipayPlusClient shared].configuration.acquirerId, [AlipayPlusClient shared].configuration.merchantId, params.paymentCurrency, @"defaultregion", params.logoPattern];
        id a = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
        id b = [[IAPLocalStorage.shared diskCache] objectForKey:key];
        id c = [IAPLocalStorage.shared codingObjectForKey:key];
        if (result && [a isKindOfClass:IAPPaymentOption.class] && b && [c isKindOfClass:IAPPaymentOption.class] && (a == paymentOption) && (a == c)) {
            [exp fulfill];
        }
        
        [IAPLocalStorage.shared setCodingObject:nil forKey:key];
    }];
    
    [self waitForExpectationsWithTimeout:3 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
    [AlipayPlusClient shared].configuration.merchantId = @"";
}




- (void)testSandbox0 {
    XCTestExpectation *exp = [self expectationWithDescription:@"xx"];
    [AlipayPlusClient shared].configuration.envType = kIAPEnvSANDBOX;
    
    [AlipayPlusClient shared].configuration.acquirerId = @"1234";
    [AlipayPlusClient shared].configuration.merchantId = @"1234";
    IAPInquirePaymentOptionParams *params = [IAPInquirePaymentOptionParams new];
    params.paymentCurrency = @"USD";
    params.logoPattern = @"P1-APP-01";
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:params completionHandler:^(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error) {
        BOOL result = paymentOption.enabled;
        NSString *key = [NSString stringWithFormat:@"IAPPaymentOptionCache_%@_%@_%@_%@_%@_%@", @"CASHIER_PAYMENT", [AlipayPlusClient shared].configuration.acquirerId, [AlipayPlusClient shared].configuration.merchantId, params.paymentCurrency, @"defaultregion", params.logoPattern];
        id a = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
        id b = [[IAPLocalStorage.shared diskCache] objectForKey:key];
        id c = [IAPLocalStorage.shared codingObjectForKey:key];
        if (result && [a isKindOfClass:IAPPaymentOption.class] && b && [c isKindOfClass:IAPPaymentOption.class] && (a == paymentOption) && (a == c)) {
            [exp fulfill];
        }
        
        [IAPLocalStorage.shared setCodingObject:nil forKey:key];
    }];
    
    [self waitForExpectationsWithTimeout:3 handler:^(NSError * _Nullable error) {
        
    }];
    
    [AlipayPlusClient shared].configuration.acquirerId = @"";
    [AlipayPlusClient shared].configuration.merchantId = @"";
    
    [AlipayPlusClient shared].configuration.envType = kIAPEnvPROD;
}


@end
