//
//  IAPPaymentOptionTests.m
//  AlipayPlusClientTests
//
//  Created by assuner on 2022/3/16.
//

#import <XCTest/XCTest.h>
#import "IAPPaymentOption.h"
#import "IAPPaymentOption+Private.h"
#import "IAPLogo+Private.h"

@interface IAPPaymentOptionTests : XCTestCase

@end

@implementation IAPPaymentOptionTests

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testLogo {
    NSDictionary *json = nil;
    IAPLogo *logo = [[IAPLogo alloc] initWithJSONDictionary:json];
    XCTAssertTrue(!logo, @"");
    
    json = (id)@[];
    logo = [[IAPLogo alloc] initWithJSONDictionary:json];
    XCTAssertTrue(!logo, @"");
    
    json = @{@"logoUrl": @"123", @"logoName": @"abc"};
    logo = [[IAPLogo alloc] initWithJSONDictionary:json];
    XCTAssertTrue([logo.logoUrl isEqualToString:@"123"] && [logo.logoName isEqualToString:@"abc"], @"");
    
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated"
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:logo];
    XCTAssertTrue(data.length, @"");
    IAPLogo *logo1 = [NSKeyedUnarchiver unarchiveObjectWithData:data];
#pragma clang diagnostic pop
    XCTAssertTrue([logo.logoUrl isEqualToString:logo1.logoUrl] && [logo.logoName isEqualToString:logo1.logoName], @"");
    
}

- (void)testOption {
    NSDictionary *json = nil;
    IAPPaymentOption *option = [[IAPPaymentOption alloc] initWithJSONDictionary:json currency:@""];
    XCTAssertTrue(!option, @"");
    
    json = (id)@[];
    option = [[IAPPaymentOption alloc] initWithJSONDictionary:json currency:@"USD"];
    XCTAssertTrue(!option, @"");
    
    json = @{
         @"paymentMethodType": @"CONNECT_WALLET",
         @"enabled": @"true",
         @"logos": @[@{
             @"logoName": @"AlipayPlus",
             @"logoUrl": @"https://www.alipayplus.com/logo/20022222xxx.png",
             @"logoPattern": @"test test usd",
             @"logoWidth": @"120",
             @"logoHeight": @"60",
          },
         ],
         @"promoNames":@[
           @"{\"en_US\":\"up to 20 PHP off\",\"th_TH\":\"คูปองส่วนลด 30 บาท\"}"
         ]
    };
    option = [[IAPPaymentOption alloc] initWithJSONDictionary:json];
    XCTAssertTrue(option.paymentMethodType.length && option.enabled == YES && option.logos.count && [option.logos.firstObject.logoPattern isEqualToString:@"test test usd"] && [option.promoNames.firstObject isEqualToString:@"up to 20 PHP off"], @"");
    [option markFromCache];
    XCTAssertTrue(!option.promoNames.count, @"");
    
    
    json = @{
         @"paymentMethodType": @"CONNECT_WALLET",
         @"enabled": @"false",
         @"logos": @[@{
             @"logoName": @"AlipayPlus",
             @"logoUrl": @"https://www.alipayplus.com/logo/20022222xxx.png",
             @"logoPattern": @"test test",
             @"logoWidth": @"120",
             @"logoHeight": @"60",
          },
         ],
         @"promoNames":@[
           @"{\"en_US\":\"up to 20 PHP off\",\"th_TH\":\"คูปองส่วนลด 30 บาท\"}"
         ]
    };
    option = [[IAPPaymentOption alloc] initWithJSONDictionary:json];
    XCTAssertTrue(option.paymentMethodType.length && option.enabled == NO && !option.logos && !option.promoNames, @"");
    
    json = @{
        @"paymentMethodType": @"CONNECT_WALLET",
        @"enabled": @"false",
        @"disableReason": @"No payment option exists."
    };
    option = [[IAPPaymentOption alloc] initWithJSONDictionary:json];
    XCTAssertTrue(option.paymentMethodType.length && option.enabled == NO && !option.logos && !option.promoNames, @"");
}






@end
