//
//  IAPLocalStorageTests.m
//  AlipayPlusClientTests
//
//  Created by assuner on 2022/3/10.
//

#import <XCTest/XCTest.h>
#import "IAPLocalStorage.h"
#import "IAPInnerDefines.h"


@interface IAPLocalStorage (Test)
- (NSCache *)memoryCache;
- (NSUserDefaults *)diskCache;
- (BOOL)isConfirmedNSCoding:(id)object;
- (id<NSCoding>)memoryCodingObjectForKey:(NSString *)key;
- (id<NSCoding>)diskCodingObjectForKey:(NSString *)key;
@end

@interface NoCodingClass : NSObject
@property (nonatomic, strong) NSString *a;
@property (nonatomic, assign) NSInteger b;
@end
@implementation NoCodingClass
@end

@interface NoCodingClass2 : NSObject <NSCoding>
@property (nonatomic, strong) NoCodingClass *c;
@end
@implementation NoCodingClass2
IAP_ENCODE_Selector
IAP_DECODE_Selector
@end



@interface TestCodingClass : NSObject <NSCoding>
@property (nonatomic, strong) NSString *a;
@property (nonatomic, assign) NSInteger b;
@end

@implementation TestCodingClass
IAP_ENCODE_Selector
IAP_DECODE_Selector
@end




@interface IAPLocalStorageTests : XCTestCase

@end

@implementation IAPLocalStorageTests

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testAccessBool {
    NSString *key = @"";
    BOOL result = NO;
    
    [IAPLocalStorage.shared setBool:YES forKey:key];
    result = [IAPLocalStorage.shared boolForKey:key];
    XCTAssertTrue(!result, @"");
    
    
    key = @([[NSDate date] timeIntervalSince1970]).stringValue;
    [IAPLocalStorage.shared setBool:YES forKey:key];
    result = [IAPLocalStorage.shared boolForKey:key];
    XCTAssertTrue(result, @"");
}


- (void)testAccessString {
    NSString *key = @"";
    NSString *result = nil;
    
    [IAPLocalStorage.shared setString:@"123" forKey:key];
    result = [IAPLocalStorage.shared stringForKey:key];
    XCTAssertTrue(!result, @"");
    
    
    key = @([[NSDate date] timeIntervalSince1970]).stringValue;
    [IAPLocalStorage.shared setString:@"123" forKey:key];
    result = [IAPLocalStorage.shared stringForKey:key];
    XCTAssertTrue([result isEqualToString:@"123"], @"");
    
    [IAPLocalStorage.shared setString:(NSString *)@234 forKey:key];
    result = [IAPLocalStorage.shared stringForKey:key];
    XCTAssertTrue([result isEqualToString:@"123"], @"");
    
    [[IAPLocalStorage.shared diskCache] setObject:@123 forKey:key];
    result = [IAPLocalStorage.shared stringForKey:key];
    XCTAssertTrue(!result, @"");
}

- (void)testIsConfirmNSCoding {
    BOOL result = [IAPLocalStorage.shared isConfirmedNSCoding:NoCodingClass.new];
    XCTAssertTrue(!result, @"");
    
    result = [IAPLocalStorage.shared isConfirmedNSCoding:TestCodingClass.new];
    XCTAssertTrue(result, @"");
}


- (void)testCodingObjects {
    NSString *key = @"";
    id<NSCoding> object = nil;
    id<NSCoding> memoryObject = nil;
    id<NSCoding> diskObject = nil;
    
    [IAPLocalStorage.shared setCodingObject:TestCodingClass.new forKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(!object, @"");
    XCTAssertTrue(!memoryObject, @"");
    XCTAssertTrue(!diskObject, @"");
    
    key = @([[NSDate date] timeIntervalSince1970]).stringValue;
    TestCodingClass *value1 = [TestCodingClass new];
    value1.a = @"12";
    value1.b = 11;
    [IAPLocalStorage.shared setCodingObject:value1 forKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(object == value1, @"");
    XCTAssertTrue(memoryObject == value1, @"");
    XCTAssertTrue(diskObject, @"");
    XCTAssertTrue(object == memoryObject, @"");
    
    [IAPLocalStorage.shared setCodingObject:(id<NSCoding>)NoCodingClass.new forKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    XCTAssertTrue(memoryObject == value1, @"");
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(diskObject, @"");
    XCTAssertTrue(object == value1, @"");
    XCTAssertTrue(object == memoryObject, @"");
    
    NoCodingClass2 *value2 = [NoCodingClass2 new];
    value2.c = [NoCodingClass new];
    value2.c.a = @"33";
    value2.c.b = 33;
    [IAPLocalStorage.shared setCodingObject:(id<NSCoding>)value2 forKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(object == value1, @"");
    XCTAssertTrue(memoryObject, @"");
    XCTAssertTrue(diskObject, @"");
    XCTAssertTrue(object == memoryObject, @"");
    
    [[IAPLocalStorage.shared memoryCache] removeObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    XCTAssertTrue(!memoryObject, @"");
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    XCTAssertTrue(object, @"");
    XCTAssertTrue(diskObject, @"");
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    XCTAssertTrue(memoryObject, @"");
    
    [[IAPLocalStorage.shared memoryCache] setObject:value1 forKey:key];
    [IAPLocalStorage.shared setCodingObject:nil forKey:key];
    object = [IAPLocalStorage.shared codingObjectForKey:key];
    memoryObject = [IAPLocalStorage.shared memoryCodingObjectForKey:key];
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(!object, @"");
    XCTAssertTrue(!memoryObject, @"");
    XCTAssertTrue(!diskObject, @"");
    
    diskObject = @"";
    [[IAPLocalStorage.shared diskCache] setObject:[@"123" dataUsingEncoding:NSUTF8StringEncoding] forKey:key];
    diskObject = [IAPLocalStorage.shared diskCodingObjectForKey:key];
    XCTAssertTrue(!diskObject, @"");
}


- (void)testAccessData {
    NSString *key = @"";
    NSData *data = nil;
    NSData *memoryData = nil;
    NSData *diskData= nil;
    
    [IAPLocalStorage.shared setData:[NSData data] forKey:key];
    data = [IAPLocalStorage.shared dataForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    diskData = [[IAPLocalStorage.shared diskCache] objectForKey:key];
    XCTAssertTrue(!data, @"");
    XCTAssertTrue(!memoryData, @"");
    XCTAssertTrue(!diskData, @"");
    
    key = @([[NSDate date] timeIntervalSince1970]).stringValue;
    NSData *value = [@"222" dataUsingEncoding:NSUTF8StringEncoding];
    [IAPLocalStorage.shared setData:value forKey:key];
    data = [IAPLocalStorage.shared dataForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    diskData = [[IAPLocalStorage.shared diskCache] dataForKey:key];
    XCTAssertTrue([[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    XCTAssertTrue(memoryData == data, @"");
    XCTAssertTrue([[[NSString alloc] initWithData:diskData encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    
    [[IAPLocalStorage.shared memoryCache] removeObjectForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    XCTAssertTrue(!memoryData, @"");
    diskData = [[IAPLocalStorage.shared diskCache] dataForKey:key];
    data = [IAPLocalStorage.shared dataForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    XCTAssertTrue(memoryData, @"");
    XCTAssertTrue([[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    XCTAssertTrue([[[NSString alloc] initWithData:diskData encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    
    [[IAPLocalStorage.shared memoryCache] setObject:value forKey:key];
    [IAPLocalStorage.shared setData:(NSData *)@"123" forKey:key];
    data = [IAPLocalStorage.shared dataForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    diskData = [[IAPLocalStorage.shared diskCache] dataForKey:key];
    XCTAssertTrue([[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    XCTAssertTrue(memoryData == data, @"");
    XCTAssertTrue([[[NSString alloc] initWithData:diskData encoding:NSUTF8StringEncoding] isEqualToString:@"222"], @"");
    
    
    
    [IAPLocalStorage.shared setData:nil forKey:key];
    data = [IAPLocalStorage.shared dataForKey:key];
    memoryData = [[IAPLocalStorage.shared memoryCache] objectForKey:key];
    diskData = [[IAPLocalStorage.shared diskCache] objectForKey:key];
    XCTAssertTrue(!data, @"");
    XCTAssertTrue(!memoryData, @"");
    XCTAssertTrue(!diskData, @"");
}

@end
