//
//  IAPMultiLanguageUtilsTests.m
//  AlipayPlusClientTests
//
//  Created by assuner on 2022/3/12.
//

#import <XCTest/XCTest.h>
#import "IAPMultiLanguageUtils.h"
#import "AlipayPlusClientAPI.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"

@interface IAPMultiLanguageUtils (test)
+ (NSString *)formatedCurrentLauguage;
+ (NSString *)formatedLanguageCode:(NSString *)originalLauguageCode;
+ (NSArray<NSString *> *)formatedLanguageCodes:(NSArray<NSString *> *)originalLauguageCodes;
+ (NSString *)matchedLanguageWithLanguageCodes:(NSArray<NSString *> *)originalLauguageCodes;
+ (NSArray<NSString *> *)presetLanguageCodes;
@end

@interface IAPMultiLanguageUtilsTests : XCTestCase

@end

@implementation IAPMultiLanguageUtilsTests

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testFormatedLanguageCode {
    NSString *oL = @"";
    NSString *result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue(!result, @"");
    
    
    oL = @"en-US";
    result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue([result isEqualToString:@"en_US"], @"");
    
    oL = @"en";
    result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue([result isEqualToString:oL], @"");
    
    oL = @"zh_Hans_US";
    result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue([result isEqualToString:@"zh_Hans_US"], @"");
    
    oL = @"zh_Hans-US";
    result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue([result isEqualToString:@"zh_Hans_US"], @"");
    
    oL = @"zh_Hans_US_A";
    result = [IAPMultiLanguageUtils formatedLanguageCode:oL];
    XCTAssertTrue([result isEqualToString:@"zh_Hans_US_A"], @"");
}

- (void)testFormatedLanguageCodes {
    NSArray *ols = @[];
    NSArray *result = nil;
    
    result = [IAPMultiLanguageUtils formatedLanguageCodes:ols];
    XCTAssertTrue(!result, @"");
    
    
    ols = @[@"zh-Hans_US", @"zh-Hans_US"];
    
    result = [IAPMultiLanguageUtils formatedLanguageCodes:ols];
    XCTAssertTrue([result[0] isEqualToString:@"zh_Hans_US"] && [result[1] isEqualToString:@"zh_Hans_US"], @"");
    
}

- (void)testMatchedLanguageWithLanguageCodes {
    NSString *currentLanguage = nil;
    NSString *matchedLaunguage = nil;
    
    currentLanguage = @"zh_CN";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_CN"] , @"");
    
    currentLanguage = @"zh-CN";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_CN"] , @"");
    
    currentLanguage = @"zh-Hans_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_CN"] , @"");
    
    currentLanguage = @"zh-Hans-US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_CN"] , @"");
    
    currentLanguage = @"zh_HK";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"zh-TW";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"zh-MO";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"zh-Hant_TH";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"yue_CN";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"yue_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    
    currentLanguage = @"en_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"en_US"] , @"");
    
    currentLanguage = @"en_US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"en_US"] , @"");
    
    currentLanguage = @"en-KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"en_US"] , @"");
    
    currentLanguage = @"sh-kR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"en_US"] , @"");
    
    currentLanguage = @"fil_PH";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"fil_PH"] , @"");
    
    [AlipayPlusClient shared].configuration.language = nil;
}

- (void)testPresetPaymentSheetText {
    NSString *currentLanguage = nil;
    NSString *matchedLaunguage = nil;
    NSString *goToPayString = nil;
    
    currentLanguage = @"zh-Hans-US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_CN"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"继续支付"], @"");
    
    
    currentLanguage = @"zh-Hant-US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"zh_HK"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"繼續支付"], @"");
    
    currentLanguage = @"id_ID";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"id_ID"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"Bayar Sekarang"], @"");
    
    currentLanguage = @"es_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"en_US"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"Go to Pay"], @"");
    
    currentLanguage = @"th_TH";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"th_TH"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"ชำระเงินเลยตอนนี้"], @"");
    
    currentLanguage = @"ms_MY";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"ms_MY"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    XCTAssertTrue([goToPayString isEqualToString:@"Bayar Sekarang"], @"");
    

    
    currentLanguage = @"ko_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    matchedLaunguage = [IAPMultiLanguageUtils matchedLanguageWithLanguageCodes:[IAPMultiLanguageUtils presetLanguageCodes]];
    XCTAssertTrue([matchedLaunguage isEqualToString:@"ko_KR"] , @"");
    goToPayString = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Go_to_Pay];
    
    [AlipayPlusClient shared].configuration.language = nil;
}

- (void)testMatchedLanguagePromoNameWithJSONString {
    NSString *currentLanguage = nil;
    NSString *JSONStringArray = @"{\"en_US\":\"up to 20 PHP off\",\"th_TH\":\"คูปองส่วนลด 30 บาท\"}";
    NSString *result = nil;
    
    currentLanguage = @"zh-Hans-US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextWithJSONString:JSONStringArray];
    XCTAssertTrue([result isEqualToString:@"up to 20 PHP off"], @"");
    
    currentLanguage = @"en_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextWithJSONString:JSONStringArray];
    XCTAssertTrue([result isEqualToString:@"up to 20 PHP off"], @"");
    
    currentLanguage = @"th_TH";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextWithJSONString:JSONStringArray];
    XCTAssertTrue([result isEqualToString:@"คูปองส่วนลด 30 บาท"], @"");
    
    
    JSONStringArray = @"{\"th_TH\":\"คูปองส่วนลด 30 บาท\"}";
    currentLanguage = @"en_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextWithJSONString:JSONStringArray];
    XCTAssertTrue(!result, @"");
    
    
    [AlipayPlusClient shared].configuration.language = nil;
}


- (void)testmatchedLanguagePromoNamesWithJSONStringArray {
    NSString *currentLanguage = nil;
    NSArray *JSONString = @[@"{\"en_US\":\"up to 20 PHP off\",\"th_TH\":\"คูปองส่วนลด 30 บาท\"}"];
    NSArray *result = nil;
    
    currentLanguage = @"zh-Hans-US";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:JSONString];
    XCTAssertTrue([result.firstObject isEqualToString:@"up to 20 PHP off"], @"");
    
    currentLanguage = @"en_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:JSONString];
    XCTAssertTrue([result.firstObject isEqualToString:@"up to 20 PHP off"], @"");
    
    currentLanguage = @"th_TH";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:JSONString];
    XCTAssertTrue([result.firstObject isEqualToString:@"คูปองส่วนลด 30 บาท"], @"");
    
    
    JSONString = @[@"{\"th_TH\":\"คูปองส่วนลด 30 บาท\"}"];
    currentLanguage = @"en_KR";
    [AlipayPlusClient shared].configuration.language = currentLanguage;
    result = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:JSONString];
    XCTAssertTrue(!result, @"");
    
    
    [AlipayPlusClient shared].configuration.language = nil;
}
@end
