//
//  IAPPaymentDataTests.m
//  AlipayPlusClientTests
//
//  Created by assuner on 2022/3/20.
//

#import <XCTest/XCTest.h>
#import "IAPPaymentData.h"
#import "IAPJSONUtils.h"

@interface IAPPaymentDataTests : XCTestCase

@end

@implementation IAPPaymentDataTests

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testRedirectionInfo {
    NSDictionary *json = (id)@"";
    IAPPaymentWalletRedirectionInfo *redirectionInfo = [[IAPPaymentWalletRedirectionInfo alloc] initWithJSONDictionary:json codeValue:@"123"];
    XCTAssertTrue(!redirectionInfo, @"");
    
    
    json = @{
        @"appIdentifier": @"DANA",
        @"applinkUrl": @"https://link.dana.id/accashierpayment?ACCodeValue=${codeValue}",
        @"normalUrl": @"https://m.dana.id/m/ac/init?ACCodeValue=${codeValue}",
        @"schemeUrl": @""
    };
    redirectionInfo = [[IAPPaymentWalletRedirectionInfo alloc] initWithJSONDictionary:json codeValue:@"123"];
    XCTAssertTrue(redirectionInfo.applinkUrl.length && redirectionInfo.normalUrl.length && redirectionInfo.schemeUrl, @"");
    
    XCTAssertTrue([redirectionInfo.applinkUrl isEqualToString:@"https://link.dana.id/accashierpayment?ACCodeValue=123"], @"");
    XCTAssertTrue([redirectionInfo.normalUrl isEqualToString:@"https://m.dana.id/m/ac/init?ACCodeValue=123"], @"");
}


- (void)testWallet {
    NSDictionary *json = (id)@"";
    IAPPaymentWalletData *wallet = [[IAPPaymentWalletData alloc] initWithJSONDictionary:json reasonTemplate:@{} codeValue:@"123"];
    XCTAssertTrue(!wallet, @"");
    
    json = @{
        @"enabled": @"true",
        @"walletLogo": @{
            @"logoName": @"DANA",
            @"logoUrl": @"https://gw.alipay.com/icon/medium/default/DANA.svg"
        },
        @"promoNames": @[
            @"{\"en_US\":\"30% off\",\"th_TH\":\"คูปองส่วนลด 30 บาท\"}"
        ],
        @"redirectionInfoList": @{
            @"appIdentifier": @"DANA",
            @"applinkUrl": @"https://link.dana.id/accashierpayment?ACCodeValue=${codeValue}",
            @"normalUrl": @"https://m.dana.id/m/ac/init?ACCodeValue=${codeValue}",
            @"schemeUrl": @""
        },
        @"searchKeywords": @[
            @"dana wallet"
        ],
        @"walletBrandName": @"DANA"
    };
    wallet = [[IAPPaymentWalletData alloc] initWithJSONDictionary:json reasonTemplate:@{} codeValue:@"123"];
    XCTAssertTrue(wallet, @"");
    XCTAssertTrue(wallet.enabled, @"");
    XCTAssertTrue(wallet.walletLogo.logoUrl, @"");
    XCTAssertTrue(wallet.searchKeywords.count, @"");
    XCTAssertTrue(wallet.walletBrandName, @"");
    XCTAssertTrue(![wallet.promoNames.firstObject containsString:@"en_US"], @"");
    
    
    json = @{
        @"enabled": @"false",
        @"walletBrandName": @"DANA",
        @"disableReason": @"STORE_NOT_SUPPORT"
    };
    wallet = [[IAPPaymentWalletData alloc] initWithJSONDictionary:json reasonTemplate:@{@"STORE_NOT_SUPPORT": @"123"} codeValue:@"123"];
    XCTAssertTrue([wallet.disableReason isEqualToString:@"123"], @"");
    
    json = @{
        @"enabled": @"false",
        @"walletBrandName": @"DANA",
        @"disableReason": @"STORE_NOT_SUPPORT"
    };
    wallet = [[IAPPaymentWalletData alloc] initWithJSONDictionary:json reasonTemplate:@{} codeValue:@"123"];
    XCTAssertTrue(!wallet.disableReason, @"");
}


- (void)testPaymentData {
    NSDictionary *json = (id)@"";
    IAPPaymentData *data = [[IAPPaymentData alloc] initWithJSONDictionary:json];
    XCTAssertTrue(!data, @"");
    
    json = @{
        @"codeValue": @"281666042sf4d2tEAu2rZkS2t2mv1kYKx",
        @"displayPaymentAmount": @"100.00",
        @"displayPaymentCurrency": @"USD",
        @"reasonTemplate":@{@"STORE_NOT_SUPPORT":@"{\"en_US\":\"Not applicable at this store\",\"th_TH\":\"ไม่รองรับสำหรับร้านนี้\"}"}
    };
    data = [[IAPPaymentData alloc] initWithJSONDictionary:json];
    XCTAssertTrue(data.codeValue.length, @"");
    XCTAssertTrue(data.displayPaymentAmount.length, @"");
    XCTAssertTrue(data.displayPaymentCurrency.length, @"");
    XCTAssertTrue(![data.reasonTemplate[@"STORE_NOT_SUPPORT"] containsString:@"en_US"], @"");
    
    NSString *jsonString = @"{\"codeValue\":\"281666040099hsC12rZgD1j0E29g2jT6Igqm\",\"displayPaymentAmount\":\"1.00\",\"displayPaymentCurrency\":\"MYR\",\"wallets\":[{\"enabled\":true,\"promoNames\":[\"{\\\"en_US\\\":\\\"英文测试\\\",\\\"id_ID\\\":\\\"印尼与\\\",\\\"zh_CN\\\":\\\"中文测试\\\"}\"],\"redirectionInfo\":{\"applinkUrl\":\"https://m-dev.tngdigital.com.my/s/ac-cashier-intermediate/h5.html?ACCodeValue=${codeValue}\",\"normalUrl\":\"https://cdn.tngdigital.com.my/s/ac-cashier-intermediate/standalone-h5.html?ACCodeValue=${codeValue}\",\"schemeUrl\":\"tngdwallet://client/dl/ACDecode?codeValue=${codeValue}\"},\"walletBrandName\":\"TNG eWallet\",\"walletLogo\":{\"logoName\":\"TNG\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/TNG.png\"},\"walletName\":\"TNG\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://uat.m.gcash.com/gcash-ac-cashier-intermediate-web/h5.html?acCodeValue=${codeValue}&acFlag=true\",\"normalUrl\":\"https://payments.gcash.com/gcash-ac-cashier-intermediate-web/standalone-h5.html?acCodeValue=${codeValue}&acFlag=true\"},\"walletBrandName\":\"GCash\",\"walletLogo\":{\"logoName\":\"GCASH\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/GCASH.png\"},\"walletName\":\"GCASH\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://tmn-dev.app.link/gncscanb?codeValue=${codeValue}\",\"normalUrl\":\"https://acm-ccp-web-qa.public-a-cloud1n.ascendmoney.io/ccp-web/prepare?codeValue=${codeValue}\"},\"walletBrandName\":\"TrueMoney Wallet\",\"walletLogo\":{\"logoName\":\"TRUEMONEY\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/TRUEMONEY.png\"},\"walletName\":\"TRUEMONEY\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://render.alipay.hk/p/w/hk-ulink/?path=/mobile&scheme=alipayhk%3A%2F%2Fplatformapi%2FstartApp%3FappId%3D85200168%26fromSite%3Dapp%26CashierAction%3DtradePay%26orderCode%3D${codeValue}%26paymentRedirectUrl%3D%26terminalType%3DWAP\",\"normalUrl\":\"https://render.alipay.com/p/w/ac-fe-adaptor/?ACCodeValue=${codeValue}&paymentMethodType=ALIPAY_HK&ACPaymentRedirectUrl=&ACAppType=WAP\"},\"walletBrandName\":\"AlipayHK\",\"walletLogo\":{\"logoName\":\"ALIPAY_HK\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/ALIPAY_HK.png\"},\"walletName\":\"ALIPAY_HK\"}]}";
    
    json = [IAPJSONUtils JSONDictionaryFromString:jsonString];
    data = [[IAPPaymentData alloc] initWithJSONDictionary:json];
    XCTAssertTrue(data.wallets.count == 4, @"");
    
    
    jsonString = @"{\"codeValue\":\"281666040099hsC12rZgD1j0E29g2jT6Igqm\",\"displayPaymentAmount\":\"1.00\",\"displayPaymentCurrency\":\"MYR\",\"wallets\":[{\"enabled\":true,\"promoNames\":[\"{\\\"en_US\\\":\\\"英文测试\\\",\\\"id_ID\\\":\\\"印尼与\\\",\\\"zh_CN\\\":\\\"中文测试\\\"}\"],\"redirectionInfo\":\"\",\"walletBrandName\":\"TNG eWallet\",\"walletLogo\":{\"logoName\":\"TNG\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/TNG.png\"},\"walletName\":\"TNG\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://uat.m.gcash.com/gcash-ac-cashier-intermediate-web/h5.html?acCodeValue=${codeValue}&acFlag=true\",\"normalUrl\":\"https://payments.gcash.com/gcash-ac-cashier-intermediate-web/standalone-h5.html?acCodeValue=${codeValue}&acFlag=true\"},\"walletBrandName\":\"GCash\",\"walletLogo\":{\"logoName\":\"GCASH\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/GCASH.png\"},\"walletName\":\"GCASH\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://tmn-dev.app.link/gncscanb?codeValue=${codeValue}\",\"normalUrl\":\"https://acm-ccp-web-qa.public-a-cloud1n.ascendmoney.io/ccp-web/prepare?codeValue=${codeValue}\"},\"walletBrandName\":\"TrueMoney Wallet\",\"walletLogo\":{\"logoName\":\"TRUEMONEY\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/TRUEMONEY.png\"},\"walletName\":\"TRUEMONEY\"},{\"enabled\":true,\"redirectionInfo\":{\"applinkUrl\":\"https://render.alipay.hk/p/w/hk-ulink/?path=/mobile&scheme=alipayhk%3A%2F%2Fplatformapi%2FstartApp%3FappId%3D85200168%26fromSite%3Dapp%26CashierAction%3DtradePay%26orderCode%3D${codeValue}%26paymentRedirectUrl%3D%26terminalType%3DWAP\",\"normalUrl\":\"https://render.alipay.com/p/w/ac-fe-adaptor/?ACCodeValue=${codeValue}&paymentMethodType=ALIPAY_HK&ACPaymentRedirectUrl=&ACAppType=WAP\"},\"walletBrandName\":\"AlipayHK\",\"walletLogo\":{\"logoName\":\"ALIPAY_HK\",\"logoUrl\":\"https://gw.alipay.com/icon/large/rectangle/ALIPAY_HK.png\"},\"walletName\":\"ALIPAY_HK\"}]}";
    
    json = [IAPJSONUtils JSONDictionaryFromString:jsonString];
    data = [[IAPPaymentData alloc] initWithJSONDictionary:json];
    XCTAssertTrue(data.wallets.count == 3, @"");
    
    json = @{
        @"codeValue": @"281666042sf4d2tEAu2rZkS2t2mv1kYKx",
        @"displayPaymentAmount": @"100.00",
        @"displayPaymentCurrency": @"USD",
        @"reasonTemplate":@{@"STORE_NOT_SUPPORT":@"{\"en_US\":\"Not applicable at this store\",\"th_TH\":\"ไม่รองรับสำหรับร้านนี้\"}"},
        @"wallets":
            @[@{
                    @"disableReason": @"STORE_NOT_SUPPORT",
                    @"enabled": @"false",
                    @"walletBrandName": @"KLARNA"
             }]
    };
    data = [[IAPPaymentData alloc] initWithJSONDictionary:json];
    XCTAssertTrue(data.wallets.count == 1, @"");
    XCTAssertTrue([data.wallets.firstObject.disableReason isEqualToString:@"Not applicable at this store"], @"");
}

@end
