package com.iap.basic.alipay.async;

import android.text.TextUtils;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class NamedRunnable implements Runnable, Pool.Poolable {

    public static final NamedRunnablePool TASK_POOL = new NamedRunnablePool(8, 16);
    static final String TAG = "AsyncTaskExecutor";

    ;
    String mThreadName;
    Runnable mTask;
    StandardPipeline.IScheduleNext mScheduleNext;
    int mWeight = 0;

    NamedRunnable(Runnable runnable, String threadName, int weight) {
        mTask = runnable;
        mThreadName = threadName;
        mWeight = weight;
    }

    void setThreadName(String threadName) {
        mThreadName = threadName;
    }

    void setTask(Runnable task) {
        mTask = task;
    }

    NamedRunnable setScheduleNext(StandardPipeline.IScheduleNext scheduleNext) {
        mScheduleNext = scheduleNext;
        return this;
    }

    void setWeight(int weight) {
        mWeight = weight;
    }

    @Override
    public void run() {
        String threadName = null;
        if (!TextUtils.isEmpty(mThreadName)) {
            threadName = Thread.currentThread().getName();
            Thread.currentThread().setName(threadName + "_" + mThreadName);
        }
        try {
            mTask.run();
        } finally {
            if (!TextUtils.isEmpty(mThreadName)) {
                if (null != threadName) {
                    Thread.currentThread().setName(threadName);
                }
            }

            if (null != mScheduleNext) {
                mScheduleNext.scheduleNext();
            }

            TASK_POOL.free(this);
        }
    }

    @Override
    public void reset() {
        mTask = null;
        mThreadName = null;
        mScheduleNext = null;
        mWeight = 0;
    }

    /**
     * 对象池
     */
    public static final class NamedRunnablePool extends Pool<NamedRunnable> {

        private final AtomicInteger mIndex = new AtomicInteger(1);

        public NamedRunnablePool(int initialCapacity, int max) {
            super(initialCapacity, max);
        }

        @Deprecated
        @Override
        public NamedRunnable obtain() {
            return null;
        }

        public synchronized NamedRunnable obtain(Runnable task, String threadName) {
            return obtain(task, threadName, 0);
        }

        public synchronized NamedRunnable obtain(Runnable task, String threadName, int weight) {
            NamedRunnable namedRunnable;
            if (freeObjects.size() == 0) {
                namedRunnable = newObject(task, threadName, weight);
            } else {
                namedRunnable = freeObjects.pop();
                namedRunnable.setTask(task);
                namedRunnable.setThreadName(threadName);
                namedRunnable.setWeight(weight);
            }
            return namedRunnable;
        }

        @Override
        @Deprecated
        protected NamedRunnable newObject() {
            return null;
        }

        NamedRunnable newObject(Runnable task, String threadName, int weight) {
            if (TextUtils.isEmpty(threadName)) {
                threadName = "NamedRunable_" + mIndex.getAndIncrement();
            } else {
                threadName = "NamedRunable_" + mIndex.getAndIncrement() + "_" + threadName;
            }
            return new NamedRunnable(task, threadName, weight);
        }

        @Override
        public synchronized void free(NamedRunnable object) {
            super.free(object);
        }

        @Override
        public synchronized void freeAll(List<NamedRunnable> objects) {
            super.freeAll(objects);
        }

        @Override
        public synchronized void clear() {
            super.clear();
        }
    }
}
