package com.iap.basic.alipay.config;

/**
 * Author kanguihu
 * Data   2022/3/7
 * Desc
 */
public class IAPConfiguration {

    /**
     * The unique ID that is assigned by the ACQP to identify a merchant.
     */
    public String merchantId;

    /**
     * The unique ID that is assigned by A+ to identify an ACQP
     */
    public String acquirerId;

    /**
     * The type of environment where the SDK is installed. The value of this parameter affects the gateway address of the requests that are initiated from the SDK.
     * Valid values:
     * PROD: indicates the production environment.
     * SANDBOX: indicates the sandbox environment.
     * Default value: RPOD
     */
    public String envType = "PROD";

    /**
     * The language that is preferred by the user. The value of the parameter must follow the ISO-639 standard.
     * The default value is the language specified in the device settings.
     */
    public String language;

    public IAPConfiguration() {
    }

    public IAPConfiguration(String merchantId, String acquirerId) {
        this.merchantId = merchantId;
        this.acquirerId = acquirerId;
    }

    private IAPConfiguration(Builder builder) {
        merchantId = builder.merchantId;
        acquirerId = builder.acquirerId;
        envType = builder.envType;
        language = builder.language;
    }

    public static IAPConfiguration getConfiguration(String merchantId, String acquirerId) {
        return new IAPConfiguration(merchantId, acquirerId);
    }


    public static final class Builder {
        private String merchantId;
        private String acquirerId;
        private String envType;
        private String language;

        public Builder() {
        }

        public Builder merchantId(String merchantId) {
            this.merchantId = merchantId;
            return this;
        }

        public Builder acquirerId(String acquirerId) {
            this.acquirerId = acquirerId;
            return this;
        }

        public Builder envType(String envType) {
            this.envType = envType;
            return this;
        }

        public Builder language(String language) {
            this.language = language;
            return this;
        }

        public IAPConfiguration build() {
            return new IAPConfiguration(this);
        }
    }
}
