package com.iap.basic.alipay.http.core;


import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.util.Locale;

public class HttpUtils {

    private static final String TAG = "HttpUtils";

    private HttpUtils() {
    }

    @NonNull
    public static String getLanguage() {
        return Locale.getDefault().toString();
    }

    @Nullable
    public static byte[] readBytes(@Nullable InputStream inputStream) {
        return readBytes(inputStream, true);
    }

    @Nullable
    public static byte[] readBytes(@Nullable InputStream inputStream, boolean closeAfterFinished) {
        if (inputStream == null) {
            return null;
        } else {
            ByteArrayOutputStream byteBuffer = new ByteArrayOutputStream();
            int bufferSize = 1024;
            byte[] buffer = new byte[bufferSize];

            try {
                boolean var5 = false;

                int len;
                while ((len = inputStream.read(buffer)) != -1) {
                    byteBuffer.write(buffer, 0, len);
                }
            } catch (Throwable e) {
                Log.e(TAG, "readBytes exception:" + e.getMessage());
            }

            byte[] bytes = byteBuffer.toByteArray();
            if (closeAfterFinished) {
                close(inputStream);
            }

            return bytes;
        }
    }

    public static void close(@Nullable Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            } catch (IOException exception) {
                Log.w(TAG, "Take it easy, just cannot close stream:" + exception.getMessage());
            }

        }
    }
}
