package com.iap.basic.alipay.http.model;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

public class HttpRequest implements Serializable {
    public static final int DEFAULT_TIMEOUT_MS = 5000;
    @Nullable
    public String data;
    @Nullable
    public Map<String, String> headers;
    @Nullable
    public HttpMethod method;
    public boolean notFollowRedirects;
    public int timeoutMilliseconds;
    public boolean cache;
    @NonNull
    public String url;

    public HttpRequest(@NonNull String url, @NonNull HttpMethod method, @Nullable String data) {
        this(url, method, null, data);
    }

    public HttpRequest(@NonNull String url, @NonNull HttpMethod method, @Nullable Map<String, String> headers, @Nullable String data) {
        this(url, method, headers, data, DEFAULT_TIMEOUT_MS);
    }

    public HttpRequest(@NonNull String url, @NonNull HttpMethod method, @Nullable Map<String, String> headers, @Nullable String data, int timeoutMilliseconds) {
        this.url = url;
        this.method = method;
        this.headers = headers;
        this.data = data;
        this.timeoutMilliseconds = timeoutMilliseconds;
    }

    @Override
    public String toString() {
        return "HttpRequest{" +
                "data='" + data + '\'' +
                ", headers=" + headers +
                ", method=" + method +
                ", notFollowRedirects=" + notFollowRedirects +
                ", timeoutMilliseconds=" + timeoutMilliseconds +
                ", cache=" + cache +
                ", url='" + url + '\'' +
                '}';
    }

    public HttpRequest addHeader(@NonNull String key, @Nullable Object value) {
        if (this.headers == null) {
            this.headers = new HashMap<>();
        }

        if (value == null) {
            this.headers.put(key, null);
        } else {
            this.headers.put(key, String.valueOf(value));
        }
        return this;
    }

    public HttpRequest setCache(boolean cache) {
        this.cache = cache;
        return this;
    }
}
