package com.iap.basic.alipay.image;

import android.content.Context;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.DrawableRes;

/**
 * Image service interface.
 */
public interface ImageService {

    /**
     * Load an image from a url into an {@link ImageView}.
     *
     * @param context   a {@link Context}.
     * @param url       the image URL. The url might be null.
     * @param imageView the {@link ImageView} to display the image.
     * @param callBack  the {@link CallBack} of the loading process
     */
    void loadImage(Context context, String url, ImageView imageView, CallBack callBack);

    void loadImage(Context context, String url, String placeHolder, @DrawableRes int localImage, ImageView imageView, TextView nameView, CallBack callBack);

    /**
     * Callback of the image loading process
     */
    interface CallBack {

        /**
         * Load image success.
         *
         * @param url the image URL. The url might be null.
         */
        void onSuccess(String url);

        /**
         * Load image failed.
         *
         * @param url the image URL. The url might be null.
         */
        void onFail(String url);

    }

}
