package com.iap.basic.alipay.utils;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

/**
 * Author kanguihu
 * Data   2022/3/7
 * Desc
 */
public class SharedPreferenceManager {

    public static String TAG = SharedPreferenceManager.class.getSimpleName();

    public static String SP_PREFIX = "AlipayPlusClinet";

    private static SharedPreferenceManager sharedPreferenceManager;

    private SharedPreferences sharedPreferences;


    public static SharedPreferenceManager getInstance(Context context) {
        if (sharedPreferenceManager == null) {
            synchronized (SharedPreferenceManager.class) {
                if (sharedPreferenceManager == null) {
                    sharedPreferenceManager = new SharedPreferenceManager(context);
                }
            }
        }
        return sharedPreferenceManager;
    }

    public SharedPreferenceManager(Context context) {
        if (null == context) {
            return;
        }
        sharedPreferences = context.getSharedPreferences(SP_PREFIX, Context.MODE_PRIVATE);
    }

    public synchronized boolean save(String key, String value) {
        if (TextUtils.isEmpty(key)) {
            return false;
        }
        try {
            sharedPreferences.edit().putString(key, value).apply();
            return true;
        } catch (Exception exception) {
            return false;
        }
    }

    public synchronized String fetch(String key) {
        if (TextUtils.isEmpty(key)) {
            return null;
        }
        try {
            if (!sharedPreferences.contains(key)) {
                return null;
            }
            return sharedPreferences.getString(key, "");
        } catch (Exception exception) {
            return null;
        }
    }

    public synchronized boolean delete(String key) {
        if (TextUtils.isEmpty(key)) {
            return false;
        }
        try {
            sharedPreferences.edit().remove(key).apply();
            return true;
        } catch (Exception exception) {
            return false;
        }
    }

    public synchronized boolean clear() {
        try {
            sharedPreferences.edit().clear().apply();
            return true;
        } catch (Exception exception) {
            return false;
        }
    }

}
