package com.iap.basic.alipay.view;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.net.Uri;
import android.util.AttributeSet;
import android.util.Log;

import androidx.appcompat.widget.AppCompatImageView;

import com.iap.alipayplusclient.R;


public class RoundRectImageView extends AppCompatImageView {

    public static final String TAG = "RoundedImageView";
    public static final float DEFAULT_RADIUS = 0f;
    public static final float DEFAULT_BORDER_WIDTH = 0f;
    private static final ScaleType[] SCALE_TYPES = {
            ScaleType.MATRIX,
            ScaleType.FIT_XY,
            ScaleType.FIT_START,
            ScaleType.FIT_CENTER,
            ScaleType.FIT_END,
            ScaleType.CENTER,
            ScaleType.CENTER_CROP,
            ScaleType.CENTER_INSIDE
    };

    private float cornerRadius = DEFAULT_RADIUS;
    private float borderWidth = DEFAULT_BORDER_WIDTH;
    private ColorStateList borderColor =
            ColorStateList.valueOf(RoundedDrawable.DEFAULT_BORDER_COLOR);
    private ColorStateList maskColor =
            ColorStateList.valueOf(RoundedDrawable.DEFAULT_MASK_COLOR);
    private boolean isOval = false;
    private boolean mutateBackground = false;

    private int mResource;
    private Drawable mDrawable;
    private Drawable mBackgroundDrawable;

    private ScaleType mScaleType;

    private float letterborderWidth = DEFAULT_BORDER_WIDTH;
    private ColorStateList letterborderColor = ColorStateList.valueOf(Color.TRANSPARENT);
    private int letterColor;
    private int letterBackgroundColor;
    private boolean hasSetLetterBackgoundColor = false;

    private int alpha = 255;

    public RoundRectImageView(Context context) {
        super(context);
    }

    public RoundRectImageView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public RoundRectImageView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);

        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.RoundRectImageView, defStyle, 0);

        int index = a.getInt(R.styleable.RoundRectImageView_android_scaleType, -1);
        if (index >= 0) {
            setScaleType(SCALE_TYPES[index]);
        } else {
            setScaleType(ScaleType.FIT_CENTER);
        }

        cornerRadius = a.getDimensionPixelSize(R.styleable.RoundRectImageView_radius, -1);
        borderWidth = a.getDimensionPixelSize(R.styleable.RoundRectImageView_RoundRectWidth, -1);
        letterborderWidth = a.getDimensionPixelSize(R.styleable.RoundRectImageView_letter_bord_width, -1);

        if (cornerRadius < 0) {
            cornerRadius = DEFAULT_RADIUS;
        }
        if (borderWidth < 0) {
            borderWidth = DEFAULT_BORDER_WIDTH;
        }
        if (letterborderWidth < 0) {
            letterborderWidth = DEFAULT_BORDER_WIDTH;
        }

        borderColor = a.getColorStateList(R.styleable.RoundRectImageView_RoundRectColor);
        if (borderColor == null) {
            borderColor = ColorStateList.valueOf(RoundedDrawable.DEFAULT_BORDER_COLOR);
        }
        maskColor = a.getColorStateList(R.styleable.RoundRectImageView_mask_over_color);
        if (maskColor == null) {
            maskColor = ColorStateList.valueOf(RoundedDrawable.DEFAULT_MASK_COLOR);
        }
        letterborderColor = a.getColorStateList(R.styleable.RoundRectImageView_letter_bord_color);
        if (letterborderColor == null) {
            letterborderColor = ColorStateList.valueOf(Color.TRANSPARENT);
        }

        mutateBackground = a.getBoolean(R.styleable.RoundRectImageView_riv_mutate_background, false);
        isOval = a.getBoolean(R.styleable.RoundRectImageView_riv_oval, false);
        letterBackgroundColor = a.getColor(R.styleable.RoundRectImageView_letter_background, 0xcccccc);
        letterColor = a.getColor(R.styleable.RoundRectImageView_letter_color, 0xffffff);
        if (a.hasValue(R.styleable.RoundRectImageView_letter_background)) {
            hasSetLetterBackgoundColor = true;
        }
        updateDrawableAttrs();
        updateBackgroundDrawableAttrs(true);

        a.recycle();
    }

    @Override
    public ScaleType getScaleType() {
        return mScaleType;
    }

    @Override
    public void setScaleType(ScaleType scaleType) {
        assert scaleType != null;

        if (mScaleType != scaleType) {
            mScaleType = scaleType;

            switch (scaleType) {
                case CENTER:
                case CENTER_CROP:
                case CENTER_INSIDE:
                case FIT_CENTER:
                case FIT_START:
                case FIT_END:
                case FIT_XY:
                    super.setScaleType(ScaleType.FIT_XY);
                    break;
                default:
                    super.setScaleType(scaleType);
                    break;
            }

            updateDrawableAttrs();
            updateBackgroundDrawableAttrs(false);
            invalidate();
        }
    }

    @Override
    public void setImageDrawable(Drawable drawable) {
        try {
            mResource = 0;
            mDrawable = RoundedDrawable.fromDrawable(drawable);
            updateDrawableAttrs();
            super.setImageDrawable(mDrawable);
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
        }
    }

    @Override
    public void setImageBitmap(Bitmap bm) {
        mResource = 0;
        mDrawable = RoundedDrawable.fromBitmap(bm);
        updateDrawableAttrs();
        try {
            super.setImageDrawable(mDrawable);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void setImageResource(int resId) {
        if (mResource != resId) {
            try {
                mResource = resId;
                mDrawable = resolveResource();
                updateDrawableAttrs();
                super.setImageDrawable(mDrawable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void setImageURI(Uri uri) {
        super.setImageURI(uri);
        setImageDrawable(getDrawable());
    }

    private Drawable resolveResource() {
        Resources rsrc = getResources();
        if (rsrc == null) {
            return null;
        }

        Drawable d = null;

        if (mResource != 0) {
            try {
                d = rsrc.getDrawable(mResource);
            } catch (Exception e) {
                mResource = 0;
            } catch (OutOfMemoryError e) {
                mResource = 0;
            }
        }
        return RoundedDrawable.fromDrawable(d);
    }

    @Override
    public void setBackground(Drawable background) {
        setBackgroundDrawable(background);
    }

    private void updateDrawableAttrs() {
        updateAttrs(mDrawable);
    }

    private void updateBackgroundDrawableAttrs(boolean convert) {
        try {
            if (mutateBackground) {
                if (convert) {
                    mBackgroundDrawable = RoundedDrawable.fromDrawable(mBackgroundDrawable);
                }
                updateAttrs(mBackgroundDrawable);
            }
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
        }
    }

    private void updateAttrs(Drawable drawable) {
        if (drawable == null) {
            return;
        }

        if (drawable instanceof RoundedDrawable) {
            ((RoundedDrawable) drawable)
                    .setScaleType(mScaleType)
                    .setCornerRadius(cornerRadius)
                    .setBorderWidth(borderWidth)
                    .setBorderColor(borderColor)
                    .setMaskColor(maskColor)
                    .setOval(isOval)
                    .setAlpha(alpha);
        } else if (drawable instanceof LayerDrawable) {
            LayerDrawable ld = ((LayerDrawable) drawable);
            for (int i = 0, layers = ld.getNumberOfLayers(); i < layers; i++) {
                updateAttrs(ld.getDrawable(i));
            }
        }
    }

    @Override
    @Deprecated
    public void setBackgroundDrawable(Drawable background) {
        mBackgroundDrawable = background;
        updateBackgroundDrawableAttrs(true);
        super.setBackgroundDrawable(mBackgroundDrawable);
    }

    public float getCornerRadius() {
        return cornerRadius;
    }

    public void setCornerRadius(int resId) {
        setCornerRadius(getResources().getDimension(resId));
    }

    public void setCornerRadius(float radius) {
        cornerRadius = radius;
        updateDrawableAttrs();
        updateBackgroundDrawableAttrs(false);
        invalidate();
    }

    public float getBorderWidth() {
        return borderWidth;
    }

    public void setBorderWidth(int resId) {
        setBorderWidth(getResources().getDimension(resId));
    }

    public void setBorderWidth(float width) {
        if (borderWidth == width) {
            return;
        }
        borderWidth = width;
        updateDrawableAttrs();
        updateBackgroundDrawableAttrs(false);
        invalidate();
    }

    public int getBorderColor() {
        return borderColor.getDefaultColor();
    }

    public void setBorderColor(int color) {
        setBorderColor(ColorStateList.valueOf(color));
    }

    public ColorStateList getBorderColors() {
        return borderColor;
    }

    public void setBorderColor(ColorStateList colors) {
        if (borderColor.equals(colors)) {
            return;
        }
        borderColor =
                (colors != null) ? colors : ColorStateList.valueOf(RoundedDrawable.DEFAULT_BORDER_COLOR);
        updateDrawableAttrs();
        updateBackgroundDrawableAttrs(false);
        if (borderWidth > 0) {
            invalidate();
        }
    }

    public boolean isOval() {
        return isOval;
    }

    public void setOval(boolean oval) {
        isOval = oval;
        updateDrawableAttrs();
        updateBackgroundDrawableAttrs(false);
        invalidate();
    }

    public void setDrawableAlpha(float alpha) {
        this.alpha = (int) (255 * alpha);
    }

    public boolean mutatesBackground() {
        return mutateBackground;
    }

    public void mutateBackground(boolean mutate) {
        if (mutateBackground == mutate) {
            return;
        }

        mutateBackground = mutate;
        updateBackgroundDrawableAttrs(true);
        invalidate();
    }



}