package com.iap.basic.alipay.view;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.Log;
import android.util.TypedValue;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatTextView;

import com.iap.basic.alipay.utils.Util;

/**
 * Author kanguihu
 * Data   2022/3/28
 * Desc
 */
public class ZoomTextView extends AppCompatTextView {

    private ComputeListener computeListener;

    private Paint mTextPaint;
    private float mTextSize;

    public ZoomTextView(@NonNull Context context) {
        super(context);
    }

    public ZoomTextView(@NonNull  Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public ZoomTextView(@NonNull  Context context, @Nullable  AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        refitText(this.getText().toString(), this.getWidth());
    }

    private void refitText(String text, int textViewWidth) {
        if (text == null || textViewWidth <= 0)
            return;
        mTextPaint = new Paint();
        mTextPaint.set(this.getPaint());
        int availableTextViewWidth = getWidth() - getPaddingLeft() - getPaddingRight();
        Rect boundsRect = new Rect();
        mTextPaint.getTextBounds(text, 0, text.length(), boundsRect);
        int textWidth = boundsRect.width();
        mTextSize = getTextSize();
        double fraction = textWidth * 1.0d / availableTextViewWidth;
        mTextSize /= fraction;
        mTextPaint.setTextSize(mTextSize);
        if (fraction > 1) {
            this.setTextSize(TypedValue.COMPLEX_UNIT_PX, mTextSize);
        }
        if (computeListener != null) {
            computeListener.onComputeEnd();
        }
    }

    public void setComputeListener(ComputeListener computeListener) {
        this.computeListener = computeListener;
    }

    public interface ComputeListener {

        void onComputeProcess();
        void onComputeEnd();
    }

}
