package com.iap.cashier.core;

import android.app.Activity;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import com.alipay.sdk.app.PayTask;
import com.iap.basic.alipay.async.IAPAsyncTask;
import com.iap.cashier.callback.IAPPaymentSheetEventCallback;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.model.IAPPaymentSheetEvent;
import com.iap.cashier.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Map;

/**
 * Author kanguihu
 * Data   2023/1/9
 * Desc
 */
public class AlipayCNPayAction extends BaseAction {

    public void handlePay(final Context context, PaymentData.Wallet wallet, String codeValue, final IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback) {
        if(Utils.checkClassExist("com.alipay.sdk.app.PayTask") && context instanceof Activity) {
            PaymentData.RedirectionInfo redirectionInfo = wallet.getRedirectionInfo();
            final String orderInfo = redirectionInfo == null ? "" : redirectionInfo.getOrderInfo();
            IAPAsyncTask.asyncTask(new Runnable() {
                @Override
                public void run() {
                    PayTask alipay = new PayTask((Activity) context);
                    Map<String, String> result = alipay.payV2(orderInfo, true);
                    onResult(result, callback);
                }
            });
        } else {
            super.handlePay(context, wallet, codeValue, callback);
        }
    }

    private void onResult(final Map<String, String> result, final IAPPaymentSheetEventCallback callback) {
        PayResult payResult = new PayResult(result);
        String resultStatus = payResult.getResultStatus();
        String name;
        JSONObject messageObj = new JSONObject();
        try {
            messageObj.put("memo", payResult.getMemo());
            messageObj.put("resultStatus",payResult.getResultStatus());
        } catch (JSONException e) {
        }
        String message = messageObj.toString();
        switch (resultStatus) {
            case "9000":
                name = Constants.EVENT_PAYMENT_SUCCESS;
                break;
            case "4000":
                name = Constants.EVENT_PAYMENT_FAILED;
                break;
            case "6001":
                name = Constants.EVENT_PAYMENT_CANCELED;
                break;
            case "8000":
                name = Constants.EVENT_PAYMENT_PROCESSING;
                break;
            default:
                name = Constants.EVENT_PAYMENT_EXCEPTION;
                break;

        }
        final IAPPaymentSheetEvent event = new IAPPaymentSheetEvent();
        event.name = name;
        event.message = message;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    callback.onSheetEvent(event);
                }
            }
        });
    }


}
