package com.iap.cashier.core;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

import com.iap.basic.alipay.utils.SharedPreferenceManager;
import com.iap.cashier.callback.IAPPaymentSheetEventCallback;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.model.IAPPaymentSheetEvent;

/**
 * Author kanguihu
 * Data   2023/1/9
 * Desc
 */
public class BaseAction {

    public static final String KEY_CODE_PLACEHOLDER = "${codeValue}";
    private Context context;
    private String codeValue;


    public void handlePay(Context context, PaymentData.Wallet wallet, String codeValue, IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback) {
        this.context = context;
        this.codeValue = codeValue;
        jumpToWallet(wallet);
    }

    private void jumpToWallet(PaymentData.Wallet wallet) {
        if (wallet == null) {
            return;
        }
        String brandName = wallet.getWalletName();
        SharedPreferenceManager.getInstance(context).save(Constants.KEY_RECENTLY_WALLET_USED, brandName);
        PaymentData.RedirectionInfo redirectionInfo = wallet.getRedirectionInfo();
        String packageName = redirectionInfo.getAppIdentifier();
        String schemeUrl = redirectionInfo.getSchemeUrl();
        String applinkUrl = redirectionInfo.getApplinkUrl();
        String normalUrl = redirectionInfo.getNormalUrl();
        if (!TextUtils.isEmpty(schemeUrl)) {
            openWalletAppWithScheme(packageName, schemeUrl, applinkUrl, normalUrl);
        } else if (!TextUtils.isEmpty(applinkUrl)) {
            openWalletAppWithApplink(packageName, applinkUrl, normalUrl);
        } else {
            openDefaultBrowser(normalUrl);
        }
    }

    private void openWalletAppWithScheme(String packageName, String schemeUrl, String applinkUrl, String normalUrl) {
        try {
            schemeUrl = schemeUrl.replace(KEY_CODE_PLACEHOLDER, codeValue);
            Intent intent= new Intent(Intent.ACTION_VIEW, Uri.parse(schemeUrl));
            if (!TextUtils.isEmpty(packageName)) {
                intent.setPackage(packageName);
            }
            if (!(context instanceof Activity)) {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            }
            context.startActivity(intent);
        } catch (Exception exception) {
            if (!TextUtils.isEmpty(applinkUrl)) {
                openWalletAppWithApplink(packageName, applinkUrl, normalUrl);
            } else {
                openDefaultBrowser(normalUrl);
            }
        }
    }

    private void openWalletAppWithApplink(String packageName, String applinkUrl, String normalUrl) {
        try {
            applinkUrl = applinkUrl.replace(KEY_CODE_PLACEHOLDER, codeValue);
            Intent intent= new Intent(Intent.ACTION_VIEW,Uri.parse(applinkUrl));
            if (!TextUtils.isEmpty(packageName)) {
                intent.setPackage(packageName);
            }
            if (!(context instanceof Activity)) {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            }
            context.startActivity(intent);
        } catch (Exception exception) {
            if (!TextUtils.isEmpty(normalUrl)) {
                openDefaultBrowser(normalUrl);
            }
        }
    }


    private void openDefaultBrowser(String normal) {
        try {
            normal = normal.replace(KEY_CODE_PLACEHOLDER, codeValue);
            Intent intent= new Intent(Intent.ACTION_VIEW,Uri.parse(normal));
            if (!(context instanceof Activity)) {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            }
            context.startActivity(intent);
        } catch (Exception exception) {
            Log.e("BaseAction", exception.getMessage());
        }
    }


}