package com.iap.cashier.core;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.iap.cashier.callback.IAPPaymentSheetEventCallback;
import com.iap.cashier.core.AlipayCNPayAction;
import com.iap.cashier.core.AlipayCashierClient;
import com.iap.cashier.core.BaseAction;
import com.iap.cashier.core.WalletManager;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.model.IAPPaymentSheetEvent;
import com.iap.cashier.utils.JSONConfig;

import java.lang.ref.WeakReference;
import java.util.HashMap;

/**
 * Author kanguihu
 * Data   2023/1/9
 * Desc
 */
public class CashierUtils {

    public static boolean validateSingleWallet(Context context, String paymentData, IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback) {
        if (!(context instanceof Activity)) {
            IAPPaymentSheetEvent event = new IAPPaymentSheetEvent();
            event.name = Constants.EVENT_THROW_EXCEPTION;
            event.message = Constants.EVENT_THROW_EXCEPTION_MESSAGE;
            callback.onSheetEvent(event);
            return true;
        }
        PaymentData iapPaymentData = null;
        try {
            iapPaymentData = JSONConfig.parsePaymentData(paymentData);
        } catch (Exception e) {

        }
        if (isPaymentDataValidate(iapPaymentData)
                && iapPaymentData.getWallets() != null
                && iapPaymentData.getWallets().size() == 1
                && WalletManager.isWalletsEnable(iapPaymentData.getWallets())) {
            PaymentData.Wallet wallet = iapPaymentData.getWallets().get(0);
            String codeValue = iapPaymentData.getCodeValue();
            if (containOrderInfo(wallet)) {
                AlipayCNPayAction action = new AlipayCNPayAction();
                action.handlePay(context, wallet, codeValue, callback);
            } else {
                BaseAction action = new BaseAction();
                action.handlePay(context, wallet, codeValue, callback);
            }
            return true;

        }
        return false;

    }

    public static boolean isPaymentDataValidate(PaymentData iapPaymentData) {
        return iapPaymentData != null
                && !TextUtils.isEmpty(iapPaymentData.getCodeValue())
                && !TextUtils.isEmpty(iapPaymentData.getDisplayPaymentAmount())
                && !TextUtils.isEmpty(iapPaymentData.getDisplayPaymentCurrency());
    }

    public static boolean containOrderInfo(PaymentData.Wallet wallet) {
        return wallet != null
                && wallet.getRedirectionInfo() != null
                && !TextUtils.isEmpty(wallet.getRedirectionInfo().getOrderInfo());
    }


}
