package com.iap.cashier.core;

import android.annotation.TargetApi;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.res.Configuration;
import android.os.Build;
import android.os.LocaleList;
import android.text.TextUtils;

import java.util.Locale;

/**
 * Author kanguihu
 * Data   2022/3/9
 * Desc
 */
public class ContextWrapper2i18n extends ContextWrapper {

    public static final String REGION_CN = "CN";
    public static final String LANGUAGE_ZH = "zh";

    public ContextWrapper2i18n(Context base) {
        super(base);
    }

    @SuppressWarnings("deprecation")
    public static ContextWrapper wrap(Context context, String language) {
        Configuration config = context.getResources().getConfiguration();
        if (!TextUtils.isEmpty(language)) {
            String[] splits = language.split("_");
            if (splits != null && splits.length == 2) {
                String lan = splits[0];
                String region = splits[1];
                if (!TextUtils.isEmpty(lan)
                        && !TextUtils.isEmpty(region)) {
                    Locale locale = getLocale(lan, region);
                    Locale.setDefault(locale);
                    updateConfiguration(context, config, locale);
                }
            }
        } else {
            context = updateConfiguration(context, config, Locale.ENGLISH);
        }
        return new ContextWrapper2i18n(context);
    }

    private static Context updateConfiguration(Context context, Configuration config, Locale locale) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            setSystemLocale(config, locale);
        } else {
            setSystemLocaleLegacy(config, locale);
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            context = context.createConfigurationContext(config);
        } else {
            context.getResources().updateConfiguration(config, context.getResources().getDisplayMetrics());
        }
        return context;
    }

    public static Locale getLocale(String language, String region) {
        if (TextUtils.isEmpty(language) && TextUtils.isEmpty(region)) {
            return Locale.ENGLISH;
        }
        if (LANGUAGE_ZH.equals(language)) {
            if (REGION_CN.equals(region)) {
                return Locale.SIMPLIFIED_CHINESE;
            } else {
                return Locale.TRADITIONAL_CHINESE;
            }
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            return Locale.forLanguageTag(language);
        } else {
            return new Locale(language);
        }
    }

    @SuppressWarnings("deprecation")
    public static void setSystemLocaleLegacy(Configuration config, Locale locale){
        config.locale = locale;
    }

    @TargetApi(Build.VERSION_CODES.N)
    public static void setSystemLocale(Configuration config, Locale locale){
        config.setLocale(locale);
    }

}
