package com.iap.cashier.core;

import android.content.Context;
import android.content.res.Configuration;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;

import com.iap.basic.alipay.config.ConfigurationManager;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * Author kanguihu
 * Data   2022/3/17
 * Desc   https://stackoverflow.com/questions/45556358/locale-getdefault-is-outdated-on-some-android-devices
 */
public class LanguageManager {

    private static final String TAG = LanguageManager.class.getSimpleName();

    private static final String LANGUAGE_DEFAULT = "en_US";
    private static final String LANGUAGE_PHIL = "fil"; //菲律宾语

    private final static Map<String, String> mapConvertPattern = new HashMap<>();

    static {
        mapConvertPattern.put("zh_CN", "zh_CN");
        mapConvertPattern.put("zh_Hans", "zh_CN");
        mapConvertPattern.put("zh_CN_#Hans", "zh_CN");
        mapConvertPattern.put("zh_CN_#Hant", "zh_HK");
        mapConvertPattern.put("zh_Hant", "zh_HK");
        mapConvertPattern.put("zh_tw", "zh_HK");
        mapConvertPattern.put("zh_TW", "zh_HK");
        mapConvertPattern.put("zh_hk", "zh_HK");
        mapConvertPattern.put("zh_HK", "zh_HK");
        mapConvertPattern.put("zh_MO", "zh_HK");
        mapConvertPattern.put("zh_mo", "zh_HK");
        mapConvertPattern.put("in_ID", "id_ID");    //印尼语言有in_ID和id_ID
        mapConvertPattern.put("in_id", "id_ID");
        mapConvertPattern.put("tl_PH", "fil_PH");   //菲律宾语有fil_PH和tl_PH
        mapConvertPattern.put("tl_ph", "fil_PH");
    }


    /**
     * return the matched promotion info
     * @param context
     * @param promoNames
     * @return
     */
    public static List<String> getMatchedPromoName(Context context, List<String> promoNames) {
        List<String> result = new ArrayList<>();
        if (promoNames == null || promoNames.size() == 0) {
            return result;
        }
        String language = getLanguage();
        language = getLanguageFormat(language);
        for (String promoName : promoNames) {
            if (TextUtils.isEmpty(promoName)) {
                continue;
            }
            try {
                JSONObject object = new JSONObject(promoName);
                String promoResult = "";
                if (object.has(language)) {
                    promoResult = object.getString(language);
                } else if (object.has(LANGUAGE_DEFAULT)){
                    promoResult = object.getString(LANGUAGE_DEFAULT);
                }
                if (!TextUtils.isEmpty(promoResult)) {
                    result.add(promoResult);
                }
            } catch (Exception e) {
                Log.i(TAG, " Get promotion exception:" + e.getMessage());
            }
        }
        return result;
    }

    public static String getMatchedDisableReason(Context context, String disableReason) {
        if (TextUtils.isEmpty(disableReason)) {
            return "";
        }
        String language = getLanguage();
        language = getLanguageFormat(language);
        String promoResult = "";
        try {
            JSONObject object = new JSONObject(disableReason);
            if (object.has(language)) {
                promoResult = object.getString(language);
            } else if (object.has(LANGUAGE_DEFAULT)){
                promoResult = object.getString(LANGUAGE_DEFAULT);
            }
        } catch (Exception e) {
            Log.i(TAG, " Get disableReason exception:" + e.getMessage());
        }
        return promoResult;
    }



    public static String getLanguage() {
        String language = ConfigurationManager.getInstance().getConfiguration().language;
        if (TextUtils.isEmpty(language)) {
            language = "en_US";
        }
        return language;
    }

    private static String getLanguageFormat(String language) {
        if (TextUtils.isEmpty(language)) {
            return LANGUAGE_DEFAULT;
        }
        language = language.replace("-", "_");
        String result = mapConvertPattern.get(language);
        if (!TextUtils.isEmpty(result)) {
            return result;
        }
        if (language.startsWith("en")) {
            return LANGUAGE_DEFAULT;
        }
        if (!TextUtils.isEmpty(language)) {
            String[] array = language.split("_");
            if (array.length == 2
                    && !TextUtils.isEmpty(array[0])
                    && !TextUtils.isEmpty(array[1])) {
                language = array[0] + "_" + array[1].toUpperCase();
            }
        }
        return language;
    }

    public static boolean isFilPH() {
        String language = ConfigurationManager.getInstance().getConfiguration().language;
        if (TextUtils.isEmpty(language)) {
            return false;
        }
        return language.startsWith(LANGUAGE_PHIL);
    }

}
