package com.iap.cashier.core;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.iap.alipayplusclient.R;
import com.iap.basic.alipay.image.DefaultImageService;
import com.iap.basic.alipay.image.ImageService;
import com.iap.basic.alipay.utils.SharedPreferenceManager;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.PaymentData;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Author kanguihu
 * Data   2022/3/14
 * Desc
 */
public class PaymentOptionAdapter extends RecyclerView.Adapter {

    private List<Item> itemList = new ArrayList<>();
    private boolean walletEnable = false;
    private Map<String, String> templateMap;

    public PaymentOptionAdapter(boolean walletEnable, Map<String, String> reasonTemplate) {
        this.walletEnable = walletEnable;
        this.templateMap = reasonTemplate;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        switch (ViewType.values()[viewType]) {
            case SHEET_TOP_TIP:
                View topView = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_payment_sheet_top,parent,false);
                return new TopViewHolder(topView);
            case SHEET_BOTTOM_MORE:
                View bottomView = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_payment_sheet_more,parent,false);
                return new BottomViewHolder(bottomView);
            case SHEET_PAYMENT_OPTON:
                View paymentOptionView = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_payment_sheet,parent,false);
                return new PaymentOptionViewHolder(paymentOptionView);
        }
        return null;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        Item item = itemList.get(position);
        if (item == null) {
            return;
        }
        switch (item.viewType) {
            case SHEET_PAYMENT_OPTON:
                ((PaymentOptionViewHolder)holder).bindData(((PaymentOptionItem)item));
                break;
            case SHEET_TOP_TIP:
                ((TopViewHolder)holder).bindData((TopItem) item);
                break;
        }
    }

    @Override
    public int getItemViewType(int position) {
        if (itemList == null || itemList.size() == 0) {
            return ViewType.SHEET_TOP_TIP.ordinal();
        }
        return itemList.get(position).viewType.ordinal();
    }

    @Override
    public int getItemCount() {
        return itemList == null ? 0 : itemList.size();
    }

    public enum ViewType {
        SHEET_TOP_TIP,
        SHEET_PAYMENT_OPTON,
        SHEET_BOTTOM_MORE
    }

    public abstract class Item {
        public ViewType viewType;

        public Item(ViewType viewType) {
            this.viewType = viewType;
        }
    }

    public class TopItem extends Item {

        private int topTip;

        public TopItem(int topTip) {
            super(ViewType.SHEET_TOP_TIP);
            this.topTip = topTip;
        }
    }

    public class BottomItem extends Item {

        public BottomItem() {
            super(ViewType.SHEET_BOTTOM_MORE);
        }
    }

    public class PaymentOptionItem extends Item {

        boolean selected = false;
        public PaymentData.Wallet wallet;

        public PaymentOptionItem(PaymentData.Wallet wallet) {
            super(ViewType.SHEET_PAYMENT_OPTON);
            this.wallet = wallet;
        }
    }

    public class TopViewHolder extends RecyclerView.ViewHolder {

        private TextView topTipView;

        public TopViewHolder(@NonNull View itemView) {
            super(itemView);
            topTipView = itemView.findViewById(R.id.tv_sheet_top);
        }

        public void bindData(TopItem item) {
            topTipView.setText(item.topTip);
        }
    }

    public class BottomViewHolder extends RecyclerView.ViewHolder {

        public BottomViewHolder(@NonNull View itemView) {
            super(itemView);
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (actionListener != null) {
                        actionListener.onExpand();
                    }
                }
            });
        }
    }

    public class PaymentOptionViewHolder extends RecyclerView.ViewHolder {

        private PaymentOptionItem item;
        private TextView promoView;
        private ImageView logoView;
        private ImageView checkView;
        private TextView reasonView;
        private TextView logoName;
        private View dividerView;

        public PaymentOptionViewHolder(@NonNull View itemView) {
            super(itemView);
            promoView = itemView.findViewById(R.id.tv_promo);
            logoView = itemView.findViewById(R.id.iv_logo);
            checkView = itemView.findViewById(R.id.iv_check);
            reasonView = itemView.findViewById(R.id.tv_reason);
            logoName = itemView.findViewById(R.id.tv_logo);
            dividerView = itemView.findViewById(R.id.view_divier);
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (actionListener != null
                            && itemList != null
                            && item != null
                            && item.wallet != null
                            && item.wallet.getEnabled()
                            && !item.selected) {
                        actionListener.onItemClick(item, itemList.indexOf(item));
                    }
                }
            });

        }

        public void bindData(PaymentOptionItem item) {
            this.item = item;
            if (item == null || item.wallet == null) {
                return;
            }
            dividerView.setVisibility(itemList.indexOf(item) == itemList.size() - 1 ? View.GONE : View.VISIBLE);
            PaymentData.Wallet wallet = item.wallet;
            if (wallet.getPromoNames() != null
                    && wallet.getPromoNames().size() > 0
                    && wallet.getPromoNames().get(0) != null) {
                promoView.setVisibility(View.VISIBLE);
                List<String> promoNames = LanguageManager.getMatchedPromoName(itemView.getContext(), wallet.getPromoNames());
                if (promoNames != null && promoNames.size() > 0) {
                    promoView.setText(promoNames.get(0));
                    promoView.setVisibility(View.VISIBLE);
                } else {
                    promoView.setVisibility(View.GONE);
                }
            } else {
                promoView.setVisibility(View.GONE);
            }
            if (wallet.getEnabled()) {
                itemView.setAlpha(1.0f);
                reasonView.setVisibility(View.GONE);
                checkView.setImageResource(item.selected ? R.drawable.checked : R.drawable.unchecked);
            } else {
                itemView.setAlpha(0.7f);
                if (TextUtils.isEmpty(wallet.getDisableReason())) {
                    reasonView.setVisibility(View.GONE);
                } else {
                    reasonView.setVisibility(View.VISIBLE);
                    String reason = wallet.getDisableReason();
                    String realReason = "";
                    if (templateMap != null && templateMap.get(reason) != null) {
                        String template = templateMap.get(reason);
                        realReason = LanguageManager.getMatchedDisableReason(itemView.getContext(), template);
                        reasonView.setText(realReason);
                    }
                    if (TextUtils.isEmpty(realReason)) {
                        reasonView.setVisibility(View.GONE);
                    } else {
                        reasonView.setVisibility(View.VISIBLE);
                    }
                }
                checkView.setImageResource(R.drawable.check_disable);
            }
            PaymentData.WalletLogo logo = wallet.getWalletLogo();
            if (logo != null) {
                logoName.setVisibility(View.INVISIBLE);
                logoView.setVisibility(View.VISIBLE);
                String url = logo.getLogoUrl();
                final String name = logo.getLogoName();
                if (!TextUtils.isEmpty(url)) {
                    int localId = 0;
                    if (!TextUtils.isEmpty(name) &&  Constants.logoMap.get(name.toLowerCase()) != null) {
                        localId = Constants.logoMap.get(name.toLowerCase());
                    }
                    DefaultImageService.getInstance().loadImage(itemView.getContext(),
                            url,
                            name,
                            localId,
                            logoView,
                            logoName,
                            new ImageService.CallBack() {

                                @Override
                                public void onSuccess(String url) {
                                }

                                @Override
                                public void onFail(String url) {
                                }
                            });
                } else if (!TextUtils.isEmpty(name)){
                    logoName.setText(name);
                    logoName.setVisibility(View.VISIBLE);
                    logoView.setVisibility(View.GONE);
                }
            }
        }
    }


    public void setIntialItems(Context context, List<PaymentData.Wallet> wallets) {
        if (itemList == null) {
            itemList = new ArrayList<>();
        }
        itemList.add(new TopItem(walletEnable ? R.string.cashier_select_payment_tip : R.string.cashier_no_payment_methods_available));
        wallets = WalletManager.checkWallet(wallets);
        PaymentOptionItem sortItem = null;
        String walletName = SharedPreferenceManager.getInstance(context).fetch(Constants.KEY_RECENTLY_WALLET_USED);
        if (wallets != null && wallets.size() > 0) {
            List<PaymentOptionItem> temp = new ArrayList<>();
            for (PaymentData.Wallet wallet : wallets) {
                if (wallet == null) {
                    continue;
                }
                PaymentOptionItem item = new PaymentOptionItem(wallet);
                if (TextUtils.equals(wallet.getWalletName(), walletName)) {
                    sortItem = item;
                }
                temp.add(item);

            }
            if (sortItem != null) {
                temp.remove(sortItem);
                temp.add(0, sortItem);
            }
            int count = Math.min(3, temp.size());
            if (count > 0) {
                itemList.addAll(temp.subList(0, count));
            }
            if (actionListener != null && sortItem != null) {
                actionListener.onItemClick(sortItem, 1);
            }
        }
        if (wallets.size() > 3) {
            itemList.add(new BottomItem());
        }
        notifyDataSetChanged();
    }

    public void setExpandItems(Context context, List<PaymentData.Wallet> wallets) {
        if (itemList == null) {
            itemList = new ArrayList<>();
        }
        itemList.clear();
        itemList.add(new TopItem(walletEnable ? R.string.cashier_select_payment_tip : R.string.cashier_no_payment_methods_available));
        wallets = WalletManager.checkWallet(wallets);
        PaymentOptionItem sortItem = null;
        String brandName = SharedPreferenceManager.getInstance(context).fetch(Constants.KEY_RECENTLY_WALLET_USED);
        if (wallets != null) {
            for (PaymentData.Wallet wallet : wallets) {
                PaymentOptionItem item = new PaymentOptionItem(wallet);
                itemList.add(item);
                if (wallet != null && TextUtils.equals(wallet.getWalletName(), brandName)) {
                    sortItem = item;
                }
            }
            if (sortItem != null) {
                itemList.remove(sortItem);
                itemList.add(1, sortItem);
                if (actionListener != null) {
                    actionListener.onItemClick(sortItem, 1);
                }
            }
        }

        notifyDataSetChanged();
    }

    public void setSearchItems(Context context, List<PaymentData.Wallet> wallets) {
        if (itemList == null) {
            itemList = new ArrayList<>();
        }
        itemList.clear();
        itemList.add(new TopItem(walletEnable ? R.string.cashier_select_payment_tip : R.string.cashier_no_payment_methods_available));
        wallets = WalletManager.checkWallet(wallets);
        if (wallets != null) {
            for (PaymentData.Wallet wallet : wallets) {
                PaymentOptionItem item = new PaymentOptionItem(wallet);
                itemList.add(item);
            }
        }
        notifyDataSetChanged();
    }

    public void reset() {
        if (itemList == null) {
            return;
        }
        for (Item item : itemList) {
            if (item == null) {
                continue;
            }
            if (item instanceof PaymentOptionItem) {
                if (((PaymentOptionItem) item).selected) {
                    ((PaymentOptionItem) item).selected = false;
                }
                notifyItemChanged(itemList.indexOf(item));
            }
        }
    }


    private ActionListener actionListener;

    public void setActionListener(ActionListener expandListenr) {
        this.actionListener = expandListenr;
    }
}