package com.iap.cashier.core;

import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.iap.alipayplusclient.R;
import com.iap.basic.alipay.config.ConfigurationManager;
import com.iap.cashier.utils.JSONConfig;
import com.iap.basic.alipay.utils.Util;
import com.iap.basic.alipay.view.ZoomTextView;
import com.iap.cashier.callback.IAPPaymentSheetEventCallback;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.model.IAPPaymentSheetEvent;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class PaymentSheetActivity extends AppCompatActivity implements View.OnClickListener
        , TextWatcher, TextView.OnEditorActionListener {

    public static final String TAG = "PaymentSheetActivity";

    public static boolean showSheet = false;

    public static final String KEY_EXPAND = "KEY_EXPAND";
    public static final String KEY_CONTENT = "KEY_CONTENT";

    public static final String KEY_PAYMENT_DATA = "key_payment_data";
    public static final String KEY_CODE_PLACEHOLDER = "${codeValue}";


    private BottomSheetBehavior behavior;

    private ImageView closeView;
    private TextView payView;
    private ZoomTextView amountView;
    private EditText searchView;
    private View searchParentView;
    private View enableView;
    private View disableView;
    private View returnView;
    private View clearView;
    private RecyclerView recyclerView;
    private PaymentOptionAdapter adapter;

    private PaymentData iapPaymentData;
    private boolean isExpand = false;
    private String searchContent = "";

    private PaymentOptionAdapter.PaymentOptionItem selectedItem;
    private static WeakReference<IAPPaymentSheetEventCallback<IAPPaymentSheetEvent>> callbackWeakReference;
    private static WeakReference<Context> contextWeakReference;

    public static void startActivity(Context context, String paymentData, IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback) {
        if (showSheet) return;
        showSheet = true;
        callbackWeakReference = new WeakReference<>(callback);
        contextWeakReference = new WeakReference<>(context);
        Intent intent = new Intent(context, PaymentSheetActivity.class);
        intent.putExtra(KEY_PAYMENT_DATA, paymentData);
        context.startActivity(intent);
    }

    @Override
    protected void attachBaseContext(Context newBase) {
        String language = ConfigurationManager.getInstance().getConfiguration().language;
        if (!TextUtils.isEmpty(language)) {
            //设置当前页面的语言为用户设置的偏好语言
            super.attachBaseContext(ContextWrapper2i18n.wrap(newBase, language));
            return;
        }
        super.attachBaseContext(newBase);
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        overridePendingTransition(R.anim.fade_in, R.anim.fade_out);
        setContentView(R.layout.activity_payment_sheet);
        Intent intent = getIntent();
        String paymentData = "";
        if (intent != null) {
            paymentData = intent.getStringExtra(KEY_PAYMENT_DATA);
        }
        initData(paymentData);
        View view = findViewById(R.id.bottom_sheet);
        behavior = BottomSheetBehavior.from(view);
        behavior.setDraggable(false);
        behavior.setState(BottomSheetBehavior.STATE_EXPANDED);
        behavior.setHideable(true);
        behavior.setSaveFlags(BottomSheetBehavior.SAVE_ALL);
        int height = getNormalHeight();
        behavior.setPeekHeight(height);
        initView(view);

    }

    @Override
    protected void onSaveInstanceState(@NonNull Bundle outState) {
        outState.putBoolean(KEY_EXPAND, isExpand);
        if (searchView != null && !TextUtils.isEmpty(searchView.getText().toString())) {
            outState.putString(KEY_CONTENT, searchView.getText().toString());
        }
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(@NonNull Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        isExpand = savedInstanceState.getBoolean(KEY_EXPAND);
        searchContent = savedInstanceState.getString(KEY_CONTENT);
        if (iapPaymentData != null) {
            if (isExpand) {
                updateExpand();
                if (!TextUtils.isEmpty(searchContent)) {
                    searchView.setText(searchContent);
                    searchView.setSelection(searchContent.length());
                }
            }
        }
    }

    private void initData(String paymentData) {
        try {
            if (TextUtils.isEmpty(paymentData)) {
                onSheetEvent(Constants.EVENT_THROW_EXCEPTION, Constants.EVENT_THROW_EXCEPTION_MESSAGE);
                logService(Constants.KEY_TYPE_CHECK, "check", "paymentData is null");
                finish();
                return;
            }
            iapPaymentData = JSONConfig.parsePaymentData(paymentData);
            if (iapPaymentData == null
                    || TextUtils.isEmpty(iapPaymentData.getCodeValue())
                    || TextUtils.isEmpty(iapPaymentData.getDisplayPaymentAmount())
                    || TextUtils.isEmpty(iapPaymentData.getDisplayPaymentCurrency())) {
                onSheetEvent(Constants.EVENT_THROW_EXCEPTION, Constants.EVENT_THROW_EXCEPTION_MESSAGE);
                logService(Constants.KEY_TYPE_CHECK, "check", "paymentData is invalid");
                finish();
            } else {
                onSheetEvent(Constants.EVENT_SHOW_SUCCESS, Constants.EVENT_SHOW_SUCCESS_MESSAGE);
            }
        } catch (Exception e) {
            onSheetEvent(Constants.EVENT_THROW_EXCEPTION, Constants.EVENT_THROW_EXCEPTION_MESSAGE);
            logService(Constants.KEY_TYPE_CHECK, "check", "paymentData check exception");
            finish();
        }

    }

    public void initView(final View view) {
        if (iapPaymentData == null) {
            return;
        }
        searchView = view.findViewById(R.id.et_search);
        searchParentView = view.findViewById(R.id.ll_search);
        searchView.setOnEditorActionListener(this);
        closeView = view.findViewById(R.id.iv_close);
        payView = view.findViewById(R.id.tv_order);
        amountView = view.findViewById(R.id.tv_amount);
        enableView = view.findViewById(R.id.rl_enable);
        disableView = view.findViewById(R.id.rl_disable);
        returnView = view.findViewById(R.id.tv_return);
        clearView = view.findViewById(R.id.iv_clear);
        returnView.setOnClickListener(this);
        payView.setOnClickListener(this);
        closeView.setOnClickListener(this);
        clearView.setOnClickListener(this);
        payView.setTextSize(LanguageManager.isFilPH() ? 14 : 17);
        boolean enable = WalletManager.isWalletsEnable(iapPaymentData.getWallets());
        enableView.setVisibility(enable ? View.VISIBLE : View.GONE);
        disableView.setVisibility(enable ? View.GONE : View.VISIBLE);
        recyclerView = view.findViewById(R.id.view_recyclerview);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        linearLayoutManager.setOrientation(RecyclerView.VERTICAL);
        recyclerView.setLayoutManager(linearLayoutManager);
        adapter = new PaymentOptionAdapter(enable, iapPaymentData.getReasonTemplate());
        recyclerView.setAdapter(adapter);
        adapter.setActionListener(new ActionListener<PaymentOptionAdapter.PaymentOptionItem>() {

            @Override
            public void onExpand() {
                updateExpand();
            }

            @Override
            public void onItemClick(PaymentOptionAdapter.PaymentOptionItem item, int index) {
                boolean selected = !item.selected;
                selectedItem = item;
                adapter.reset();
                selectedItem.selected = selected;
                adapter.notifyItemChanged(index);
            }
        });
        amountView.setText(iapPaymentData.getDisplayPaymentCurrency() + " " + iapPaymentData.getDisplayPaymentAmount());
        adapter.setIntialItems(this, iapPaymentData.getWallets());
        searchView.clearFocus();
    }

    private void updateExpand() {
        isExpand = true;
        searchParentView.setVisibility(View.VISIBLE);
        searchView.addTextChangedListener(PaymentSheetActivity.this);
        LinearLayout.LayoutParams lp = (LinearLayout.LayoutParams) recyclerView.getLayoutParams();
        lp.height = getMaxHeight(recyclerView.getHeight());
        recyclerView.setLayoutParams(lp);
        adapter.setExpandItems(PaymentSheetActivity.this, iapPaymentData.getWallets());
    }

    private int getNormalHeight() {
        return Util.dp2px(getResources(), 430);
    }

    private int getMaxHeight(int defaultHeight) {
        int orientation = getResources().getConfiguration().orientation;
        if (orientation != Configuration.ORIENTATION_PORTRAIT) {
            return defaultHeight;
        }
        int screenHeight = getResources().getDisplayMetrics().heightPixels;
        int available = (int) (0.8 * screenHeight - Util.dp2px(getResources(), 262));
        List<PaymentData.Wallet> wallets = (iapPaymentData == null ? new ArrayList<PaymentData.Wallet>() : iapPaymentData.getWallets());
        wallets = WalletManager.checkWallet(wallets);
        int maxItemsHeight = wallets.size() * Util.dp2px(getResources(), 56);
        defaultHeight += Util.dp2px(getResources(), 15);
        return Math.max(Math.min(maxItemsHeight, available), defaultHeight);
    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        if (id == R.id.iv_close) {
            onSheetEvent(Constants.EVENT_USER_CANCEL, Constants.EVENT_USER_CANCEL_MESSAGE);
            finish();
        } else if (id == R.id.tv_order) {
            if (selectedItem == null || !selectedItem.selected) {
                Util.toast(PaymentSheetActivity.this, R.string.cashier_select_payment_toast);
                logService(Constants.KEY_TYPE_ACTION, "click", "click without selected item");
            } else {
                logService(Constants.KEY_TYPE_ACTION, "click", "click with valid item");
                onSheetEvent(Constants.EVENT_SELECT_AND_PAY, Constants.EVENT_SELECT_AND_PAY_MESSAGE);
                String codeValue = iapPaymentData.getCodeValue();
                IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback = callbackWeakReference == null ? null : callbackWeakReference.get();
                if (CashierUtils.containOrderInfo(selectedItem.wallet)) {
                    AlipayCNPayAction action = new AlipayCNPayAction();
                    Context context = contextWeakReference == null ? null : contextWeakReference.get();
                    if (context != null) {
                        action.handlePay(context, selectedItem.wallet, codeValue, callback);
                    } else {
                        onSheetEvent(Constants.EVENT_PAYMENT_EXCEPTION, Constants.EVENT_THROW_EXCEPTION_MESSAGE);
                    }
                } else {
                    BaseAction action = new BaseAction();
                    action.handlePay(this, selectedItem.wallet, codeValue, callback);
                }
                finish();
            }
        } else if (id == R.id.tv_return){
            logService(Constants.KEY_TYPE_ACTION, "click", "click return");
            onSheetEvent(Constants.EVENT_USER_CANCEL, Constants.EVENT_USER_CANCEL_MESSAGE);
            finish();
        } else if (id == R.id.iv_clear) {
            searchView.setText("");
            logService(Constants.KEY_TYPE_ACTION, "click", "click clear");
        }
    }

    private void logService(String type, String action, String message) {
        if (AlipayCashierClient.logService != null) {
            HashMap<String, String> hashMap = new HashMap<>();
            hashMap.put(action, message);
            AlipayCashierClient.logService.log(type, hashMap);
        }
    }

    private void onSheetEvent(String name, String message) {
        if (callbackWeakReference != null && callbackWeakReference.get() != null) {
            IAPPaymentSheetEvent event = new IAPPaymentSheetEvent();
            event.name = name;
            event.message = message;
            callbackWeakReference.get().onSheetEvent(event);
        }
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {

    }

    @Override
    public void afterTextChanged(Editable editable) {
        String content = editable.toString();
        if (TextUtils.isEmpty(content)) {
            clearView.setVisibility(View.GONE);
            adapter.setExpandItems(this, iapPaymentData.getWallets());
        } else {
            clearView.setVisibility(View.VISIBLE);
            List<PaymentData.Wallet> wallets = WalletManager.matchWallet(content, iapPaymentData.getWallets());
            adapter.setSearchItems(this, wallets);
        }

    }

    @Override
    public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
        Util.hideKeyBoard(this);
        return false;
    }

    @Override
    public void finish() {
        super.finish();
        getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE|View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            getWindow().setStatusBarColor(Color.TRANSPARENT);
        }
        showSheet = false;
        overridePendingTransition(R.anim.fade_in, R.anim.slide_bottom_out);
    }

}