package com.iap.cashier.core;

import android.text.TextUtils;

import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.inner.SearchItem;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 * Author kanguihu
 * Data   2022/3/16
 * Desc
 */
public class WalletManager {

    public static final String TAG = WalletManager.class.getSimpleName();

    public static boolean isWalletsEnable(List<PaymentData.Wallet> wallets) {
        if (wallets == null || wallets.size() == 0) {
            return false;
        }
        boolean enable = false;
        for (PaymentData.Wallet wallet : wallets) {
            if (wallet == null) {
                continue;
            }
            if (wallet.getEnabled()
                    && wallet.getRedirectionInfo() != null
                    && !(TextUtils.isEmpty(wallet.getRedirectionInfo().getSchemeUrl())
                        && TextUtils.isEmpty(wallet.getRedirectionInfo().getApplinkUrl())
                        && TextUtils.isEmpty(wallet.getRedirectionInfo().getNormalUrl())
                        && TextUtils.isEmpty(wallet.getRedirectionInfo().getOrderInfo()))) {
                enable = true;
            }
        }
        return enable;
    }

    public static List<PaymentData.Wallet> checkWallet(List<PaymentData.Wallet> wallets) {
        if (wallets == null) {
            return new ArrayList<>();
        }
        Iterator<PaymentData.Wallet> iterator = wallets.iterator();
        while (iterator != null && iterator.hasNext()) {
            PaymentData.Wallet wallet = iterator.next();
            if (wallet == null
                    || wallet.getRedirectionInfo() == null
                    || (TextUtils.isEmpty(wallet.getRedirectionInfo().getSchemeUrl())
                    && TextUtils.isEmpty(wallet.getRedirectionInfo().getApplinkUrl())
                    && TextUtils.isEmpty(wallet.getRedirectionInfo().getNormalUrl())
                    && TextUtils.isEmpty(wallet.getRedirectionInfo().getOrderInfo()))) {
                iterator.remove();
            }
        }
        return wallets;
    }

    public static List<PaymentData.Wallet> matchWallet(String des, List<PaymentData.Wallet> wallets) {
        wallets = checkWallet(wallets);
        List<PaymentData.Wallet> result = new ArrayList<>();
        if (wallets == null || wallets.size() == 0) {
            return result;
        }
        List<SearchItem> startList = new ArrayList<>();
        List<SearchItem> containList = new ArrayList<>();
        List<SearchItem> fuzzList = new ArrayList<>();
        for (PaymentData.Wallet wallet : wallets) {
            if (wallet == null) {
                continue;
            }
            List<String> targets = new ArrayList<>();
            if (!TextUtils.isEmpty(wallet.getWalletName())) {
                targets.add(wallet.getWalletName());
            }
            if (wallet.getSearchKeywords() != null) {
                targets.addAll(wallet.getSearchKeywords());
            }
            double similarity = 0d;
            boolean contain = false;
            boolean start = false;
            for (String target : targets) {
                double degree = SearchManager.computeSimilarity(des, target);
                similarity = Math.max(similarity, degree);
                if (TextUtils.isEmpty(target)) {
                    target = "";
                }
                if (TextUtils.isEmpty(des)) {
                    des = "";
                }
                if (target.toLowerCase().startsWith(des.toLowerCase())) {
                    start = true;
                } else if (target.toLowerCase().contains(des.toLowerCase())) {
                    contain = true;
                }

            }
            if (similarity > 0) {
                SearchItem item = new SearchItem(similarity, wallet);
                if (start) {
                    startList.add(item);
                } else if (contain) {
                    containList.add(item);
                } else {
                    fuzzList.add(item);
                }
            }
        }

        sortCollections(startList);
        sortCollections(containList);
        sortCollections(fuzzList);

        List<SearchItem> mergeList = new ArrayList<>();
        mergeList.addAll(startList);
        mergeList.addAll(containList);
        mergeList.addAll(fuzzList);
        return bestMatch(mergeList);
    }

    private static void sortCollections(List<SearchItem> searchItems) {
        Collections.sort(searchItems, new Comparator<SearchItem>() {
            @Override
            public int compare(SearchItem o1, SearchItem o2) {
                return (int) (o2.matchDegree - o1.matchDegree);
            }
        });
    }

    private static List<PaymentData.Wallet> bestMatch(List<SearchItem> mergeList) {
        List<PaymentData.Wallet> wallets = new ArrayList<>();
        if (mergeList == null) {
            return wallets;
        }
        for (SearchItem searchItem : mergeList) {
            if (searchItem == null || searchItem.wallet == null) {
                continue;
            }
            if (wallets.size() >= 3) {
                break;
            }
            wallets.add(searchItem.wallet);
        }
        return wallets;
    }

}
