package com.iap.cashier.data.inner;

import android.content.Context;
import android.net.Uri;
import android.util.Log;

import com.iap.alipayplusclient.BuildConfig;
import com.iap.basic.alipay.http.model.HttpMethod;
import com.iap.basic.alipay.http.model.HttpRequest;
import com.iap.basic.alipay.utils.DigestManager;
import com.iap.basic.alipay.utils.SharedPreferenceManager;
import com.iap.cashier.data.Constants;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

/**
 * Author kanguihu
 * Data   2022/3/23
 * Desc
 */
public class RequestCompose {

    private static final String SANDBOX_DEV = "SANDBOX_DEV";
    private static final String SANDBOX_SIT = "SANDBOX_SIT";
    private static final String SANDBOX_PRO = "SANDBOX_PROD";
    private static final String ENV_TYPE = "sandbox_env";

    public static HttpRequest getSandboxReqeust(Context context,
                                                String merchantId,
                                                String acquirerId,
                                                String currency,
                                                String region,
                                                String logoPattern) throws JSONException {
        String url = Constants.CDN_URL_SANDBOX_PROD;
        String envType = SharedPreferenceManager.getInstance(context).fetch(ENV_TYPE);
        if (SANDBOX_DEV.equals(envType)) {
            url = Constants.CDN_URL_SANDBOX_DEV;
        } else if (SANDBOX_SIT.equals(envType)) {
            url = Constants.CDN_URL_SANDBOX_SIT;
        }
        Map<String, String> headers = new HashMap<>();
        headers.put("appId", "SIMULATION_ACQUIRER_AC");
        headers.put("workspaceId", "default");
        headers.put("sofa-group-name", "GROUP_20220317224203");
        headers.put("operation-type", "ac.simulation.wallet.post");
        headers.put("inner_operation_type", "ac.testacquirer.mock.cdn");
        headers.put("load-test", "1");
        JSONArray wrapperArray = new JSONArray();
        JSONObject object = new JSONObject();
        object.put("operationType", "ac.testacquirer.mock.cdn");
        object.put("pspId", "xxx");
        JSONObject requestData = new JSONObject();
        requestData.put("mid", merchantId);
        requestData.put("acquirerId", acquirerId);
        requestData.put("product", "CASHIER_PAYMENT");
        requestData.put("currency", currency);
        requestData.put("region", region);
        requestData.put("logoPattern", logoPattern);
        object.put("requestData", requestData);
        JSONObject envInfo = new JSONObject();
        JSONObject extendInfo = new JSONObject();
        extendInfo.put("instanceId", "xxx");
        envInfo.put("extendInfo", extendInfo);
        object.put("envInfo", envInfo);
        wrapperArray.put(object);
        HttpRequest httpRequest = new HttpRequest(url, HttpMethod.POST, headers, wrapperArray.toString());

        return httpRequest;
    }

    public static HttpRequest getCDNRequest(String merchantId,
                                            String acquirerId,
                                            String currency,
                                            String region,
                                            String logoPattern) {
        String encodeMid = DigestManager.getSHA256(merchantId);
        String url = Constants.CDN_URL_PROD;
        Uri uri = Uri.parse(url);
        Uri.Builder builder = uri.buildUpon();
        builder.appendPath(acquirerId)
                .appendPath(encodeMid)
                .appendPath("CASHIER_PAYMENT")
                .appendPath(currency)
                .appendPath(region)
                .appendPath(logoPattern)
                .appendPath("alipayplus.json")
                .appendQueryParameter("version", String.valueOf(BuildConfig.VERSION_NAME))
                .appendQueryParameter("type", "Android");
        String result = builder.build().toString();
        HttpRequest httpRequest = new HttpRequest(result, HttpMethod.GET, "");
        return httpRequest;
    }

}
