package com.iap.cashier.utils;

import android.content.Context;
import android.text.TextUtils;

import com.iap.alipayplusclient.R;
import com.iap.cashier.core.LanguageManager;
import com.iap.cashier.data.inner.PaymentData;
import com.iap.cashier.data.inner.PaymentOption;
import com.iap.cashier.data.model.IAPPaymentOption;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Author kanguihu
 * Data   2022/3/8
 * Desc
 */
public class JSONConfig {

    public static final String TAG = JSONConfig.class.getSimpleName();

    public static IAPPaymentOption convert2IAPPaymentOption(Context context, JSONObject data) {
        if (context == null
                || data == null) {
            return null;
        }
        IAPPaymentOption iapPaymentOption = new IAPPaymentOption();
        try {
            PaymentOption paymentOption = parsePaymentOption(data);
            if (paymentOption == null) {
                //系统异常
                return null;
            } else if (!paymentOption.enabled) {
                //支付方式不可用
                iapPaymentOption.paymentMethodType = paymentOption.paymentMethodType;
                iapPaymentOption.enabled = paymentOption.enabled;
                iapPaymentOption.logos = new ArrayList<>();
                iapPaymentOption.disableReason = TextUtils.isEmpty(paymentOption.disableReason) ?
                        context.getResources().getString(R.string.cashier_payment_method_not_found) : paymentOption.disableReason;
            } else {
                iapPaymentOption.paymentMethodType = paymentOption.paymentMethodType;
                iapPaymentOption.brandName = paymentOption.brandName;
                iapPaymentOption.logos = getIAPLogos(paymentOption.logos);
                iapPaymentOption.promoNames = LanguageManager.getMatchedPromoName(context, paymentOption.promoNames);
                iapPaymentOption.enabled = paymentOption.enabled;
            }
        } catch (Exception e) {

        }
        return iapPaymentOption;
    }

    public static List<IAPPaymentOption.Logo> getIAPLogos(List<PaymentOption.Logo> logoList) {
        if (logoList == null) {
            return new ArrayList<>();
        }
        List<IAPPaymentOption.Logo> logos = new ArrayList<>();
        for (PaymentOption.Logo logoInnerData : logoList) {
            if (logoInnerData == null) {
                continue;
            }
            IAPPaymentOption.Logo logo = new IAPPaymentOption.Logo();
            logo.logoName = logoInnerData.logoName;
            logo.logoUrl = logoInnerData.logoUrl;
            logo.logoPattern = logoInnerData.logoPattern;
            logo.logoWidth = logoInnerData.logoWidth;
            logo.logoHeight = logoInnerData.logoHeight;
            logos.add(logo);
        }
        return logos;
    }


    public static PaymentOption parsePaymentOption(JSONObject object) throws Exception{
        if (object == null) {
            return null;
        }
        if (object.has("jsonUrl")) {
            object = object.getJSONObject("jsonUrl");
        }
        PaymentOption paymentOption = new PaymentOption();
        if (object.has("promoNames")) {
            JSONArray promoArrays = object.getJSONArray("promoNames");
            paymentOption.promoNames = getStringList(promoArrays);
        }
        if (object.has("enabled")) {
            paymentOption.enabled = object.getBoolean("enabled");
        }
        if (object.has("brandName")) {
            paymentOption.brandName = object.getString("brandName");
        }
        if (object.has("disableReason")) {
            paymentOption.disableReason = object.getString("disableReason");
        }
        if (object.has("paymentMethodType")) {
            paymentOption.paymentMethodType = object.getString("paymentMethodType");
        }
        if (object.has("logos")) {
            JSONArray logoArray = object.getJSONArray("logos");
            List<PaymentOption.Logo> logosList = new ArrayList<>();
            for (int i = 0; i < logoArray.length(); i++) {
                JSONObject logoObject = logoArray.getJSONObject(i);
                if (logoObject == null) {
                    continue;
                }
                PaymentOption.Logo logo = new PaymentOption.Logo();
                if (logoObject.has("logoName")) {
                    logo.logoName = logoObject.getString("logoName");
                }
                if (logoObject.has("logoUrl")) {
                    logo.logoUrl = logoObject.getString("logoUrl");
                }
                if (logoObject.has("logoPattern")) {
                    logo.logoPattern = logoObject.getString("logoPattern");
                }
                if (logoObject.has("logoWidth")) {
                    logo.logoWidth = logoObject.getString("logoWidth");
                }
                if (logoObject.has("logoHeight")) {
                    logo.logoHeight = logoObject.getString("logoHeight");
                }
                logosList.add(logo);
            }
            paymentOption.logos = logosList;
        }
        return paymentOption.isInvalid() ? null : paymentOption;
    }

    private static List<String> getStringList(JSONArray jsonArray) throws Exception {
        List<String> list = new ArrayList<>();
        if (jsonArray == null) return list;
        for (int i = 0; i < jsonArray.length(); i++) {
            String promoItem = jsonArray.getString(i);
            if (!TextUtils.isEmpty(promoItem)) {
                list.add(promoItem);
            }
        }
        return list;
    }

    public static PaymentData parsePaymentData(String data) throws Exception {
        if (TextUtils.isEmpty(data)) {
            return null;
        }
        PaymentData paymentData = new PaymentData();
        JSONObject object = new JSONObject(data);
        if (object == null) {
            return paymentData;
        }
        if (object.has("codeValue")) {
            paymentData.setCodeValue(object.getString("codeValue"));
        }
        if (object.has("displayPaymentAmount")) {
            paymentData.setDisplayPaymentAmount(object.getString("displayPaymentAmount"));
        }
        if (object.has("displayPaymentCurrency")){
            paymentData.setDisplayPaymentCurrency(object.getString("displayPaymentCurrency"));
        }
        if (object.has("reasonTemplate")) {
            JSONObject reasonTemplate = object.getJSONObject("reasonTemplate");
            Iterator<String> iterator = reasonTemplate.keys();
            HashMap<String, String> templateMap = new HashMap<>();
            while (iterator.hasNext()) {
                String key = iterator.next();
                if (TextUtils.isEmpty(key)) {
                    continue;
                }
                String value = reasonTemplate.getString(key);
                if (TextUtils.isEmpty(value)) {
                    continue;
                }
                templateMap.put(key, value);
            }
            paymentData.setReasonTemplate(templateMap);

        }
        if (object.has("wallets")) {
            JSONArray walletArray = object.getJSONArray("wallets");
            paymentData.setWallets(getWalletList(walletArray));
        }
        return paymentData;

    }

    public static List<PaymentData.Wallet> getWalletList(JSONArray jsonArray) throws Exception {
        List<PaymentData.Wallet> wallets = new ArrayList<>();
        if (jsonArray == null) {
            return wallets;
        }
        for (int i = 0; i < jsonArray.length(); i++) {
            JSONObject walletObject = jsonArray.getJSONObject(i);
            if (walletObject == null) continue;
            PaymentData.Wallet wallet = new PaymentData.Wallet();
            if (walletObject.has("enabled")) {
                wallet.setEnabled(walletObject.getBoolean("enabled"));
            }
            if (walletObject.has("walletBrandName")) {
                wallet.setWalletBrandName(walletObject.getString("walletBrandName"));
            }
            if (walletObject.has("disableReason")) {
                wallet.setDisableReason(walletObject.optString("disableReason"));
            }
            if (walletObject.has("searchKeywords")) {
                wallet.setSearchKeywords(getStringList(walletObject.getJSONArray("searchKeywords")));
            }
            if (walletObject.has("promoNames")) {
                wallet.setPromoNames(getStringList(walletObject.getJSONArray("promoNames")));
            }
            if (walletObject.has("walletName")) {
                wallet.setWalletName(walletObject.getString("walletName"));
            }
            if (walletObject.has("walletLogo")) {
                PaymentData.WalletLogo logo =  new PaymentData.WalletLogo();
                JSONObject logoObject = walletObject.getJSONObject("walletLogo");
                if (logoObject.has("logoUrl")) {
                    logo.setLogoUrl(logoObject.getString("logoUrl"));
                }
                if (logoObject.has("logoName")) {
                    logo.setLogoName(logoObject.getString("logoName"));
                }
                wallet.setWalletLogo(logo);
            }
            if (walletObject.has("redirectionInfo")) {
                PaymentData.RedirectionInfo info = new PaymentData.RedirectionInfo();
                JSONObject infoObject = walletObject.getJSONObject("redirectionInfo");
                if (infoObject.has("appIdentifier")) {
                    info.setAppIdentifier(infoObject.getString("appIdentifier"));
                }
                if (infoObject.has("schemeUrl")) {
                    info.setSchemeUrl(infoObject.getString("schemeUrl"));
                }
                if (infoObject.has("applinkUrl")) {
                    info.setApplinkUrl(infoObject.getString("applinkUrl"));
                }
                if (infoObject.has("normalUrl")) {
                    info.setNormalUrl(infoObject.getString("normalUrl"));
                }
                if (infoObject.has("orderInfo")) {
                    info.setOrderInfo(infoObject.getString("orderInfo"));
                }
                wallet.setRedirectionInfo(info);
                wallets.add(wallet);
            }
        }
        return wallets;
    }

    public static IAPPaymentOption getIAPPaymentOption(String data) {
        if (TextUtils.isEmpty(data)) {
            return null;
        }
        IAPPaymentOption iapPaymentOption = new IAPPaymentOption();
        try {
            JSONObject object = new JSONObject(data);
            if (object.has("enabled")) {
                iapPaymentOption.enabled = object.getBoolean("enabled");
            }
            if (object.has("brandName")) {
                iapPaymentOption.brandName = object.getString("brandName");
            }
            if (object.has("paymentMethodType")) {
                iapPaymentOption.paymentMethodType = object.getString("paymentMethodType");
            }
            if (object.has("logos")) {
                List<IAPPaymentOption.Logo> logos = new ArrayList<>();
                JSONArray logoArray = object.getJSONArray("logos");
                for (int i = 0; i < logoArray.length(); i++) {
                    IAPPaymentOption.Logo logo = new IAPPaymentOption.Logo();
                    JSONObject logoObject = logoArray.getJSONObject(i);
                    if (logoObject.has("logoUrl")) {
                        logo.logoUrl = logoObject.getString("logoUrl");
                    }
                    if (logoObject.has("logoName")) {
                        logo.logoName = logoObject.getString("logoName");
                    }
                    if (logoObject.has("logoPattern")) {
                        logo.logoPattern = logoObject.getString("logoPattern");
                    }
                    if (logoObject.has("logoWidth")) {
                        logo.logoWidth = logoObject.getString("logoWidth");
                    }
                    if (logoObject.has("logoHeight")) {
                        logo.logoHeight = logoObject.getString("logoHeight");
                    }
                    logos.add(logo);
                }
                iapPaymentOption.logos = logos;
            }
        } catch (JSONException e) {
            return null;
        }
        return iapPaymentOption;
    }

    public static String getIAPPaymentOptionsStr(IAPPaymentOption iapPaymentOption) {
        if (iapPaymentOption == null) {
            return "";
        }
        JSONObject object = new JSONObject();
        try {
            object.put("enabled", iapPaymentOption.enabled);
            object.put("brandName", iapPaymentOption.brandName);
            object.put("disableReason", iapPaymentOption.disableReason);
            object.put("paymentMethodType", iapPaymentOption.paymentMethodType);
            if (iapPaymentOption.logos != null) {
                JSONArray array = new JSONArray();
                for (IAPPaymentOption.Logo logo : iapPaymentOption.logos) {
                    JSONObject logoObject = new JSONObject();
                    logoObject.put("logoUrl", logo.logoUrl);
                    logoObject.put("logoName", logo.logoName);
                    logoObject.put("logoPattern", logo.logoPattern);
                    logoObject.put("logoWidth", logo.logoWidth);
                    logoObject.put("logoHeight", logo.logoHeight);
                    array.put(logoObject);
                }
                object.put("logos", array);
            }
        } catch (JSONException e) {
            return "";
        }
        return object.toString();
    }

}
