This document introduces the business flow involving Alipay+ ACQP Server Sample Code usage and provides a step-by-step guide on integrating with the ACQP Server Sample Code.
<a name="PqfRC"></a>
# 1. How it works
The source codes will be fully opened to the NDA-signed ACQP. ACQP has 2 alternative modes to integrate with the sample code.

- 1st Mode： jar installation
- 2nd Mode： change & copy specific source codes

![](pics/1.1.jpeg)<br />Notes: <br />The document mainly introduces the step-by-step guide for 1st mode.
<a name="yoXpj"></a>
# 2. Overall business flow
<a name="x8Her"></a>
## 2.1 Make an API call to A+
![](pics/2.1.jpeg)<a name="vMVEG"></a>
## 2.2 Receive an API call from A+
![](pics/2.2.jpeg)<a name="mUSdj"></a>
# 3. Requirements

1. JDK 1.8.0_251 or higher .
1. Maven 3.0.4 or higher if you integrate the sample code via Maven.
<a name="sFrwj"></a>
# 4. Installation
To install the code into your project, complete the following steps:

Build the jar file from the source code and upload it to your private maven repository. Refer to the following sample for details on how to upload the file. Replace placeholders as required.
```shell
#!/usr/bin/env bash

groupId=com.alipay.ap
jarName=acqp-server-sample-code
jarVersion=<acqp-server-sample-code-version>
pathToJarFile=<path-to-jar-file>
repo=<url-of-the-repository-to-deploy>
repoId=<id-to-map-on-server-section-of-settings.xml>

mvn deploy:deploy-file -DgroupId=${groupId} -DartifactId=${jarName} -Dversion=${jarVersion} -Dpackaging=jar -Dfile=${pathToJarFile} -DrepositoryId=${repoId} -Durl=${repo}
```
```shell
set groupId=com.alipay.ap
set jarName=acqp-server-sample-code
set jarVersion=<acqp-server-sample-code-version>
set pathToJarFile=<path-to-jar-file>
set repo=<url-of-the-repository-to-deploy>
set repoId=<id-to-map-on-server-section-of-settings.xml>

mvn deploy:deploy-file -DgroupId=%groupId% -DartifactId=%jarName% -Dversion=%jarVersion% -Dpackaging=jar -Dfile=%pathToJarFile% -DrepositoryId=%repoId% -Durl=%repo%

```

Download POM dependencies from Maven.Refer to the following sample for details on how to download POM dependencies from Maven. 
```xml
<!--Component dependencies-->
<dependency>
  <groupId>com.alipayplus</groupId>
  <artifactId>acqp-java-server-sdk</artifactId>
  <version>${acqp.server.sdk.version}</version>
</dependency>

<!--Open source dependencies-->
<dependency>
  <groupId>org.slf4j</groupId>
  <artifactId>slf4j-api</artifactId>
  <!--Minimum version: 1.7.5-->
  <version>1.7.5</version>
</dependency>
<dependency>
  <groupId>commons-io</groupId>
  <artifactId>commons-io</artifactId>
   <!--Minimum version: 1.0-->
  <version>1.0</version>
</dependency>
<dependency>
  <groupId>com.google.code.gson</groupId>
  <artifactId>gson</artifactId>
  <!--Minimum version: 2.0 -->
  <version>2.0</version>
</dependency>
<dependency>
  <groupId>org.apache.httpcomponents</groupId>
  <artifactId>httpclient</artifactId>
   <!--Minimum version: 4.5 -->
  <version>4.5</version>
</dependency>
<dependency>
  <groupId>org.apache.commons</groupId>
  <artifactId>commons-lang3</artifactId>
   <!--Minimum version: 3.0 -->
  <version>3.0</version>
</dependency>
```
<a name="XRmrV"></a>
# 5. Documentation
<a name="JzAAE"></a>
## 5.1 Quick Start
<a name="EDNHR"></a>
### 5.1.0 Preparing config
Create a file config.properties under the folder src/main/resources/ with the contents below:
```bash
clientId=your_client_id_here
privateKey=your_private_key_here
alipayPlusPublicKey=your_public_key_here
gatewayUrl=https://open-sea.alipayplus.com
```
| **Key** | **Instruction** |
| --- | --- |
| clientId  | Assigned by A+ to represent a ACQP partner.<br />Note：Ask your SA to provide this information |
| privateKey  | Generated by the ACQP and used to generate the API call signature |
| alipayPlusPublicKey<br />  | Provided by A+ and used to verify the signature<br />Note：Ask your SA to provide this information |
| gatewayUrl  | The API gateway URL provided by A+<br />Note：Ask your SA to provide this information |

<a name="TbpPm"></a>
### 5.1.1 Initializing Logging Library
The code used the SLF4J  logging mechanism which serves as a simple facade or abstraction for various logging frameworks (e.g. java.util.logging, logback, log4j) allowing the end user to plug in the desired logging framework at deployment time. Note that SLF4J-enabling your library implies the addition of only a single mandatory dependency, namely slf4j-api.jar. If no binding/provider is found on the class path, then SLF4J will default to a no-operation implementation.<br />Refer to the following steps on how to initialize the Log4j2 logging framworks. 
<a name="hEJXn"></a>
#### 5.1.1.1 Add Log4j2 Dependencies
```xml
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-api</artifactId>
    <version>2.9.1</version>
</dependency>
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-core</artifactId>
    <version>2.9.1</version>
</dependency>
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-slf4j-impl</artifactId>
    <version>2.11.0</version>
</dependency>
<dependency>
    <groupId>com.lmax</groupId>
    <artifactId>disruptor</artifactId>
    <version>3.4.2</version>
</dependency>
```
Note：For other logging frameworks initialization，pls refer to the SLF4J website [https://www.slf4j.org/](https://www.slf4j.org/)
<a name="r0RwR"></a>
#### 5.1.1.2 Add Log4j2 Configuration File
Add your custom Log4j2 configuration file to the classpath. The following configuration is a sample that prints logs to the console. The actual logging configuration depends on your project. 
```xml
<?xml version="1.0" encoding="UTF-8"?>
<!--Sample Log4j2 configuration which just print logs to the console.-->
<Configuration>
    <Appenders>
        <Console name="STDOUT-APPENDER" target="SYSTEM_OUT">
            <PatternLayout pattern="%-5p %c{2} - %m%n%throwable" charset="UTF-8"/>
        </Console>
        
        <Console name="STDERR-APPENDER" target="SYSTEM_ERR">
            <PatternLayout pattern="%-5p %c{2} - %m%n%throwable" charset="UTF-8"/>
        </Console>
        
    </Appenders>
    
    <Loggers>
        <AsyncLogger name="STDOUT" additivity="false" level="info">
            <AppenderRef ref="STDOUT-APPENDER"/>
        </AsyncLogger>
        
        <AsyncLogger name="STDERR" additivity="false" level="info">
            <AppenderRef ref="STDERR-APPENDER"/>
        </AsyncLogger>
        
        <AsyncRoot level="info">
            <AppenderRef ref="STDOUT-APPENDER"/>
        </AsyncRoot>
    </Loggers>
</Configuration>
```
<a name="y97cs"></a>
### 5.1.2 Make an API call for Cashier Payment
```java
{
    //Get the DefaultApacheHttpPostAPSClient
    ApsClient apsClient = DefaultAPS.getInstance().getApsClient();
    //Compose the CashierPaymentPayRequest
    CashierPaymentPayRequest cashierPaymentPayRequest = new CashierPaymentPayRequest();
    //Compose the DefaultCashierPaymentCallback
    HttpSendCallback callback = new DefaultCashierPaymentCallback(){};
    //Make a api call to A+
    apsClient.execute(cashierPaymentPayRequest, callback);
}
```
You also can refer to the full sample code in CashierPayDemoCode.
<a name="AqqmY"></a>
### 5.1.3 Make an API call for Inquiry Payment
```java
{
    //Get the DefaultApacheHttpPostAPSClient
    ApsClient apsClient = DefaultAPS.getInstance().getApsClient();
    //Compose the inquiryPaymentRequest
    InquiryPaymentRequest inquiryPaymentRequest = new InquiryPaymentRequest();
    //Compose the DefaultInquiryPaymentCallback
    HttpSendCallback callback = new DefaultInquiryPaymentCallback(){};
    //Make a api call to A+
    apsClient.execute(inquiryPaymentRequest, callback);
}
```
You also can refer to the full sample code in InquiryPaymentDemoCode
<a name="tVsam"></a>
### 5.1.4 Make an API call for Cancel Payment
```java
{
    //Get the DefaultApacheHttpPostAPSClient
    ApsClient apsClient = DefaultAPS.getInstance().getApsClient();
    //Compose the cancelPaymentRequest
    CancelPaymentRequest cancelPaymentRequest = new CancelPaymentRequest();
    //Compose the DefaultCancelPaymentCallback
    HttpSendCallback callback = new DefaultCancelPaymentCallback(){};
    //Make a api call to A+
    apsClient.execute(cancelPaymentRequest, callback);
}
```
You also can refer to the full sample code in CancelPaymentDemoCode
<a name="eYp9n"></a>
### 5.1.5 Make an API call for Refund Payment
```java
{
    //Get the DefaultApacheHttpPostAPSClient
    ApsClient apsClient = DefaultAPS.getInstance().getApsClient();
    //Compose the refundPaymentRequest
    RefundPaymentRequest refundPaymentRequest = new RefundPaymentRequest();
    //Compose the DefaultRefundPaymentCallback
    HttpSendCallback callback = new DefaultRefundPaymentCallback(){};
    //Make a api call to A+
    apsClient.execute(refundPaymentRequest, callback);
}
```
You also can refer to the full sample code in RefundPaymentDemoCode
<a name="GzZP1"></a>
### 5.1.6 Receive an API call for Payment Notify
```java
{
    //Receive a API call from A+ for Payment Notify
    
    //Compose the httpReceiveRequest
    HttpReceive httpReceiveRequest = new HttpReceive();
    
    //Get the DefaultAPSHttpReceiveListener
    APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();
 
    //Compose DefaultPaymentNotifyCallback
    HttpReceiveCallback callback = new DefaultPaymentNotifyCallback(){};
    
    //Handle the request from A+
    HttpReceiveResponse httpReceiveResponse = listener.onReceive(httpReceiveRequest,callback); 

    //Write the response to A+
}
```
You also can refer to the full sample code in PaymentNotifyDemoCode
<a name="lrje3"></a>
## 5.2 Advanced Topic
<a name="s2vmA"></a>
### 5.2.1 Best Practice Settings

1. Create optimal payment experiences by following these best practices for integration.
1. The codes provide the default implementations following these best practices.

| **Function** | **Instruction** | **API Definition** | **Default Implementation** |
| --- | --- | --- | --- |
| APS Settings | the system settings for APS integration<br />- clientId<br />- cancelInterval<br />- refundInterval<br />- initDelay<br />- ..<br /> | APSSettings | APSSettings |
| HTTP Client | The client which sends the aps request to A+ | APSClient | DefaultApacheHttpPostAPSClient |
| HTTP Listener Client | The client which deals with the request from A+<br /> | APSHttpReceiveListener | DefaultAPSHttpReceiveListener |
| HTTP Send Callback | deals with the response from A+  | HttpSendCallback | For example:<br />DefaultCashierPaymentCallback<br />DefaultCancelPaymentCallback<br /> |
| HTTP Receive Callback | deals with the request from A+  | HttpReceiveCallback | For example:<br />DefaultPaymentNotifyCallback<br />DefaultRegistrationStatusNotifyCallback |
| Job Schedule Framework | Supports scheduling a job to execute a task with a specific interval when the response status is "U". For example, schedule a job to execute the refund API call again when the first refund response is "U".<br />e.g.<br />Schedule a job to execute the refund API call again when 1st refund response is with "U" status | APSScheduleJobExecutor | com.alipay.aps.job.APSScheduleJobExecutor |

<a name="XzgDE"></a>
### 5.2.2 Customization of APS Settings
<a name="rbd5v"></a>
#### 5.2.2.1 Initialize the APS Settings without config.properties
If ACQP have the concerns to put the sensitive data(e.g. privateKey/clientId/...) under the config.properties, ACQP also can get the data from your own system then construct the APS Settings object. Refer to the below sample codes.
```java
String clientId = "get clientId from your system";
String privateKey = "get privateKey from your system";
String alipayPlusPublicKey = "get alipayPlusPublicKey from your system";
String gateWayUrl = "get gateWayUrl from your system";


APSSettings apsSettings = new APSSettings(clientId, privateKey, alipayPlusPublicKey, gateWayUrl);
```
<a name="K0eXl"></a>
#### 5.2.2.2 Overwrite Default Settings Value
```java
//Get default APS Settings
APSSettings apsSettings = DefaultAPS.getApsSettings();

//Overwrite default size of core thread pool 
apsSettings.setCorePoolSizeOfJobExecutor(20);

//Overwrite default intervals for inquiry payment
apsSettings.setInquiryInterval(new int[] { 10, 15, 20});
```
table below shows all default settings 

| **Settings key** | **Default  value** | **instruction** |
| --- | --- | --- |
| apacheHttpConnectionRequestTimeout | 5000 | Max milliseconds for request timeout |
| apacheHttpConnectTimeout | 5000 | Max milliseconds for http connect timeout |
| apacheHttpSocketTimeout | 5000 | Max milliseconds for http socket timeout |
| apacheMaxPoolSize | 20 | Max size of Http Connection Pool |
| corePoolSizeOfJobExecutor | 10 | Core size of Job Executor Thread Pool |
| maxSizeOFDelayQueue | 50 | Max size of  Job Executor queue |
| initDelay | 600 | The initial delay time of Job  Executor<br />interval unit: milliseconds |
| delay | 600 | The fixed delay time of Job  Executor<br />interval unit : milliseconds |
| inquiryIntervalForCashierpay | 20, 30, 40, 50, 60,70, 80 | the interval of inquiryPaymentForCashierPay schedule job execution<br />interval unit : seconds |
| cancelInterval | 5, 5, 5, 5, 10, 10,10, 10 | the interval of cancelPayment schedule job execution<br />interval unit : seconds |
| refundInterval | 2, 4, 8, 16, 32,                                       64, 128, 256,512 | the interval of refundPayment schedule job execution<br />interval unit : seconds |
| inquiryRegistrationStatusInterval | 86400, 86400,86400, 86400,86400, 86400,86400 | the interval of inquiryRegistrationStatus schedule job execution<br />interval unit : seconds |

<a name="SIUVC"></a>
### 5.2.3 Customization of APSClient
If the default implementation of APSClient does not satisfy your requirements, you can create a class inherits the abstract APSClient class and implement the abstract method to replace the default one.```java
public class ACQPCustomizedAPSClient extends APSClient {

     /**
     * 
     * @param request aps request {@link APSRequest}
     * @param callback {@link HttpSendCallback}
     * @throws IOException
     * @throws AlipayPlusApiException
     * @throws Exception
     */
    @Override
    public <R extends APSRequest, P extends APSResponse> void execute(R request,
                                                                      HttpSendCallback<R, P> callback
                                                                     ) throws IOException,AlipayPlusApiException,Exception {
        //TODO: implement the execute method by ACQP
    }
}
```
```java
{
//initialize the aps client
AcqpCustomizedAPSClient apsclient = new AcqpCustomizedAPSClient(XXX);
//inject the aps client
DefaultAPS.instance().setApsClient(apsclient)    
}
```

<a name="ryjyT"></a>
### 5.2.4 Customization of APSHttpReceiveListener
If the default implementation of APSHttpReceiveListener does not satisfy your requirements, you can create a class that inherits the base APSHttpReceiveListener class and implement the abstract method to replace the default one.
```java
public class AcqpCustomizedAPSHttpReceiveListener extends APSHttpReceiveListener {

    /**
     * 
     * @param httpReceive
     * @param httpReceiveCallback
     * @return HttpReceiveResponse
     * @throws AlipayPlusApiException
     * @throws Exception
     */
    @Override
    public HttpReceiveResponse onReceive(HttpReceive httpReceive,
                                         HttpReceiveCallback httpReceiveCallback) 
                                                throws AlipayPlusApiException, Exception {

        //TODO: implement by ACQP
    }
}
```
```java
{
//initialize the aps listener
AcqpCustomizedAPSHttpReceiveListener apsListener= new AcqpCustomizedAPSHttpReceiveListener(XXX);
//inject the aps listener
DefaultAPS.instance().setApsHttpReceiveListener(apsListener)    
}
```
<a name="T0Cnx"></a>
### 5.2.5  Customization of  Callback

- The HttpSendCallback implementation is to handle the response from A+ (the ACQP makes an API call to A+).
- The HttpReceiveCallback implementation is to handle the request from A+ (the ACQP receives an API call from A+).

For each API call/receive, A+ provides the default callback definition in the sample code. The user can use the one provided or create a new customized callback to handle the request/response.
```java
public class AcqpCustomizedApplyTokenCallback extends HttpSendCallback {

    @Override
    public void onFstatus(APSClient apsClient, APSRequest request,
                          APSResponse response) throws Exception {

        //TODO: implement by AQC
    }

    @Override
    public void onUstatus(APSClient apsClient, APSRequest request,
                          APSResponse response) throws Exception {

        //TODO: implement by AQC
    }

    @Override
    public void onSstatus(APSClient apsClient, APSRequest request,
                          APSResponse response) throws Exception {
        //TODO: implement by AQC
    }
```
```java
public class AcqpCustomizedPaymentNotifyCallback extends HttpReceiveCallback {

    @Override
    public HttpReceiveResponse doProcess(HttpReceive httpReceive) throws Exception {

        //TODO: implement by AQC

    }
}
```
| **Default Callback** | **Base Class** |
| --- | --- |
| DefaultApplyTokenCallback | HttpSendCallback |
| DefaultAutoDebitPayCallback | HttpSendCallback |
| DefaultCancelPaymentCallback | HttpSendCallback |
| DefaultCancelTokenCallback | HttpSendCallback |
| DefaultCashierPaymentCallback | HttpSendCallback |
| DefaultInquiryPaymentCallback | HttpSendCallback |
| DefaultInquiryRegistrationStatusCallback | HttpSendCallback |
| DefaultPrepareCallback | HttpSendCallback |
| DefaultRefundPaymentCallback | HttpSendCallback |
| DefaultAuthNotifyCallback | HttpReceiveCallback |
| DefaultConsultUnbindingCallback | HttpReceiveCallback |
| DefaultPaymentNotifyCallback | HttpReceiveCallback |
| DefaultRegistrationStatusNotifyCallback | HttpReceiveCallback |

<a name="PzzTX"></a>
### 5.2.6 Job Schedule Framework
| **Case Scenrios** | **Result.ResultStatus** |  |
| --- | --- | --- |
| Refund Payment | U | Schedule a job to use the same parameter to retry the refundPayment request.  |
| Cancel Payment | U | Schedule a job use the same parameter to retry the cancelPayment request.  |
| Cashier Payment | U | Schedule a job to call the inquiry payment API to get the payment result from Alipay+ |
| .... |  |  |

For some cerntain cases/scenarios A+ require the ACQP to retry the request util you get the final result. The sample code provide a job schedule framework to support. 
<a name="IuPpB"></a>
#### 5.2.6.1 Start the Job Schedule
```java
{
    //Inject ScheduleJobSupport to persist the job in ACQP
    //Implementation by ACQP
    APSScheduleJobExecutor.getInstance().setScheduleJobSupport(new ScheduleJobSupport());

    //Inject schedule callback for refund
    APSScheduleJobExecutor.getInstance().setInquiryCallback(new AcqInquiryPaymentCallback());

    //Inject the aps client
    APSScheduleJobExecutor.getInstance()
        .setApsClient(new DefaultApacheHttpPostAPSClient(APSSettings.instance));

    //Start the job executor when the application start
    //Only need to start once
    APSScheduleJobExecutor.getInstance().startJobExecutor();
}
```
![](pics/5.2.6.1.jpeg)<a name="nHVdy"></a>
#### 5.2.6.2 Execute the job
![](pics/5.2.6.2.jpeg)<a name="CRoyn"></a>
#### 5.2.6.3 Execute runnable task
![](pics/5.2.6.3.jpeg)<a name="x2lTj"></a>
#### 5.2.6.4 Register a job
```java
//register a cancel job
APSScheduleJobExecutor.instance.put(new ScheduleJobRequest(
                    request.getPaymentRequestId(), BizType.CANCEL.name(),
                    GsonUtils.toJSONString(request),
                    DateUtil.triggerTime(APSScheduleJobExecutor.instance.getApsClient()
                        .getApsSettings().getCancelInterval()[CommonConstants.NUMBER_ZERO],
                        TimeUnit.SECONDS)));

//register a refund job
  APSScheduleJobExecutor.instance.put(new ScheduleJobRequest(
                    request.getPaymentRequestId(), BizType.REFUND.toString(),
                    GsonUtils.toJSONString(request),
                    DateUtil.triggerTime(
                        apsClient.getApsSettings().getRefundInterval()[CommonConstants.NUMBER_ZERO],
                        TimeUnit.SECONDS)));

//register a inquiry job for cashier payment
 APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(inquiryPaymentRequest.getPaymentRequestId(),
                        BizType.INQUIRY.toString(), GsonUtils.toJSONString(inquiryPaymentRequest),
                        DateUtil.triggerTime(
                            APSScheduleJobExecutor.instance.getApsClient().getApsSettings()
                                .getInquiryIntervalForCashierpay()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
```
Now only supports  3 scenrios for the job schedule, pls refer to the below tables.

| **Scenrios** | **Default Interval** | **Default Max Retry Times** |
| --- | --- | --- |
| REFUND | { 2, 4, 8, 16, 32, 64, 128, 256,512 }  | 9 |
| CANCEL | { 2, 4, 8, 16, 32,64, 128, 256,512 }  | 9 |
| INQUIRY | { 20, 30, 40, 50, 60,70, 80 }  | 7 |

<a name="uTMIf"></a>
#### 5.2.6.5 Implement the ScheduleJobSupport interface
ScheduleJobSupport is an SPI which need to be implemented by the ACQP,  using it to persist the job in the storage such as Redis or DB to make sure the jobs will be finally executed by the job executor framework even if some unexpected exception occurs.
```java
public class ACQPScheduleJobSupportForDB implements ScheduleJobSupport {

    //insert a job in the storage
    @Override
    public boolean register(ScheduleJobRequest request) {
        //implementation by ACQP
    }

    //remove a job in the storage
    @Override
    public boolean remove(ScheduleJob job) {
        //implementation by ACQP
    }

	//update a job in the storage
    @Override
    public boolean update(ScheduleJob job) {
        //implementation by ACQP
    }

    //lock a job in the storage
    @Override
    public boolean lock(ScheduleJob job) {
        //implementation by ACQP
    }

    //release a job in the storage
    @Override
    public boolean release(ScheduleJob job) {
        //implementation by ACQP
    }

    //get the jobs from the storage
    @Override
    public ScheduleJob[] listJobs() {
        //implementation by ACQP
    }

```
<a name="izRSh"></a>
### 5.2.7  Signature Tool
```java
{
   String signature = SignatureTool.sign("POST", "/aps/api/v1/payments/inquiryPayment", "your_client_id",
                           "2019-04-04T12:08:56+05:30", "request_body","your_private_key");
}
```
```java
{
    boolean isVerify = SignatureTool.verify("POST", "/aps/api/v1/payments/inquiryPayment", "your_client_id"
               "2019-04-04T12:08:56+05:30", "response_body", "sigature","alipayplus_public_key");
}
```
<a name="j0zkf"></a>
# 6. Troubleshooting
You can check logs provided by ACQP Server Sample Code to obtain helpful information for troubleshooting. The log format depends on your logging framework configuration. The following sample shows the logs printed by the ACQP Server Sample Code. In this sample, you can use the log keyword for troubleshooting.
[ALIPAYPLUS_LOG][JOBEXECUTOR_ALREADY_STARTED]  
```
WARN job.APSScheduleJobExecutor -  [ALIPAYPLUS_LOG][JOBEXECUTOR_ALREADY_STARTED] jobExecutor has already been started. 
```
The following table shows the log keywords and suggested solutions:

| **Log Keyword** | **Log Level** | **Condition** | **Solution** |
| --- | --- | --- | --- |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_NOT_READY] | ERROR | JobExecutor not ready to start, scheduleJobSupport or apsClient is not injected successfully | Check if scheduleJobSupport or apsClient is injected or not |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_ALREADY_STARTED] | WARN | The job executor has already started   | Only start the job executor once and suggest to start it when application starts |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_FETCH_TASK_ERROR] | ERROR | Failed to fetch the job/tasks from the storage | Check whether scheduleJobSupport.listJobs can get the jobs successfully |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_RUNNING_TASK_ERROR] | ERROR | Running task occurs unexpected error | Check the error stack to locate the root cause |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_JOB_TYPE_UNKNOWN] | ERROR | Unrecognized job type | Check the job type value |
| [ALIPAYPLUS_LOG][JOB_EXECUTOR_LOCK_FAILED] | WARN | lock job failed | Check if the job is already locked and not released |
| [ALIPAYPLUS_LOG][HTTP_STATUS_NOT_200_ERROR] | ERROR | No HTTP 200 status from A+ | Check the error stack to locate the root cause |
| [ALIPAYPLUS_LOG][VERIFY_SIGNATURE_ERROR] | ERROR | Unexpected error occurred when the ACQP verified the A+ signature   | Check the signature verification standard and error stack to locate the root cause |
| [ALIPAYPLUS_LOG][INVALID_SIGNATURE] | WARN | A+ signature verification by the ACQP failed | Check the signature verification standard and error stack to locate the root cause |
| [ALIPAYPLUS_LOG][CONFIG_FILE_NOT_FOUND] | ERROR | property file not found in the classpath <br />[src/main/resources/config.properties] | Check the file is exist or not |
| [ALIPAYPLUS_LOG][CONFIG_FILE_IO_EXCEPTION] | ERROR | IO exception when reading the config.properties file | Check the error stack to locate the root cause |
| [ALIPAYPLUS_LOG][JOBEXECUTOR_SCHEDULE_CALLBACK_MISSING] | ERROR | The scheduled callback for the job schedule is not injected | Check APSScheduleJobExecutor if the scheduled callback is injected |
| [ALIPAYPLUS_LOG][INVALID_AUTH_NOTIFY_TYPE] | ERROR | Invalid auth notify type  | Check the API request message |
| [ALIPAYPLUS_LOG][INVALID_REGISTRATION_NOTIFY_STATUS] | ERROR | Invalid registration notify status  | Check the API request message |
| [ALIPAYPLUS_LOG][INVALID_PAYMENT_NOTIFY_RESULT_STATUS] | ERROR | Invalid payment notify result status  | Check the API request message |
| [ALIPAYPLUS_LOG][INVALID_RESULT_STATUS] | ERROR | Invalid  result status  | Check the API request message |

<a name="orTXy"></a>
# 7. Integration Sample Code Demo
| **Integration Scenrio** | **Sample Code Class** | **Function** |
| --- | --- | --- |
| Auto Debit | PrepareDemoCode | Make an api call for Prepare |
|  | ApplyTokenDemoCode | Make an api call for Apply Token |
|  | CancelTokenDemoCode | Make an api call for Cancel Token |
|  | AutoDebitPayDemoCode | Make an api call for Auto Debit Payment |
|  | AuthNotifyDemoCode | Receive an api call  for Auth Notify |
|  | ConsultUnbindingDemoCode | Receive an api call  for Consult Unbinding |
| Cashier Payment | CashierPayDemoCode | Make an api call for Cashier Payment |
|  | CashierPayWithScheduleDemoCode | Make an api call for Cashier Payment with register a inquiry job |
| Common | RefundPaymentDemoCode | Make an api call for Refund Payment |
|  | CancelPaymentDemoCode | Make an api call for Cancel Payment |
|  | InquiryPaymentDemoCode | Make an api call for Inquiry Payment |
|  | InquiryRegistrationStatusDemoCode | Make an api call for Inquiry Registeration Status |
|  | PaymentNotifyDemoCode | Receive an api call  for Payment Notify |
|  | RegistrationDemoCode | Make an api call for Merchant Registeration |
|  | RegistrationStatusNotifyDemoCode | Receive an api call  for Registration Status Notify |
|  | CancelPaymentWithScheduleDemoCode | Make an api call for Cancel Payment with register a schedule job |
|  | RefundPaymentWithScheduleDemoCode | Make an api call for Refund Payment with register a schedule job |

<a name="zrcvM"></a>
# 8. FAQs
<a name="x3gdc"></a>
### Which programming languages are supported for the Sample Code?
Only Java is supported.
<a name="wYnAl"></a>
### Does the Sample Code include the GOF scenarios？
No，it only includes GOL scenarios（Cashier Payment and Auto Debit). 
<a name="mkbnS"></a>
### Why does the ACQP need to implement ScheduleJobSupport interface if the ACQP decides to use the Job Schedule Framework?
So as to make sure the schedule job stability, we require the ACQP to persist the scheduled job in the storeage (etc datanbase or redis ...）so that if there are any exception cases that occur during the execution, the ACQP can retry to execute jobs after the system recovery.
<a name="mXj5B"></a>
### What's the logging mechanism in the sample code ?
The code used the SLF4J  logging mechanism which serves as a simple facade or abstraction for various logging frameworks allowing the end user to plug in the desired logging framework. (e.g. java.util.logging, logback, log4j).
<a name="h385b"></a>
### Is the end user allowed to override ApsSettings?  
Yes, you are recommended to follow the best practices which are mentioned in the A+ integration document.
<a name="TlqXJ"></a>
### Does the code collect any private data/sensitive data ？
No.
<a name="OEEEH"></a>
# 9. Legal Disclaimer
**- Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full **<br />**responsibility for any activities or actions under its use.**<br />**- Partners are authorised to use for Approved Purposes.**<br />**- "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).**<br />**- You should check our NDA for detail Legal T&C.**


