/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.enums.AuthorizationNotifyType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.request.AuthNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

/**
 * subclass of {@link HttpReceiveCallback}
 * implements the callback to handle the request from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultAuthNotifyCallback.java, v 0.1 2022‐07-13 17:32 wqy01239909 Exp $
 */
public abstract class DefaultAuthNotifyCallback extends HttpReceiveCallback {
    /** logger */
    private static final Logger logger = LoggerFactory.getLogger(DefaultAuthNotifyCallback.class);

    /**
     * Process the request sent by A+
     * @param httpReceive
     * @return
     * @throws Exception
     */
    @Override
    public HttpReceiveResponse doProcess(HttpReceive httpReceive) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG]Auth notify doProcess [HttpReceive={}]", httpReceive);
        AuthNotifyRequest authNotifyRequest = SerializeUtil.deserializeRequest(httpReceive,
            AuthNotifyRequest.class);
        AuthorizationNotifyType authorizationNotifyType = authNotifyRequest
            .getAuthorizationNotifyType();
        if (authorizationNotifyType == null) {
            logger.error(
                "[ALIPAYPLUS_LOG][INVALID_AUTH_NOTIFY_TYPE] authorizationNotifyType is missing");

            throw new AlipayPlusApiException(String.format("authorizationNotifyType is missing"));
        }
        switch (authorizationNotifyType) {
            case TOKEN_CREATED:
                return this.onTokenCreated(authNotifyRequest);
            case TOKEN_CANCELED:
                return this.onTokenCanceled(authNotifyRequest);
            case AUTHCODE_CREATED:
                return this.onAuthCodeCreated(authNotifyRequest);
            default:

                logger.error(
                    "[ALIPAYPLUS_LOG][INVALID_AUTH_NOTIFY_TYPE] cannot handle the auth notify request ,authorizationNotifyType = {} ",
                    authorizationNotifyType);

                throw new AlipayPlusApiException(String.format(
                    "cannot handle the auth notify request, authorizationNotifyType=%s",
                    authorizationNotifyType));
        }
    }

    /**
     * indicates that the access token is created by the MPP and is provided in the request.
     * @param request
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onTokenCreated(AuthNotifyRequest request);

    /**
     * indicates that the access token in the request is revoked.
     * @param request
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onTokenCanceled(AuthNotifyRequest request);

    /**
     *  indicates that the authorization code is created by the MPP and is provided in the request. 
     * @param request
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onAuthCodeCreated(AuthNotifyRequest request);
}