/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.request.AutoDebitPayRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.AutoDebitPayResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultAutoDebitPayCallback.java, v 0.1 2022‐07-11 17:22 wqy01239909 Exp $
 */
public abstract class DefaultAutoDebitPayCallback extends
                                                  HttpSendCallback<AutoDebitPayRequest, AutoDebitPayResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory.getLogger(DefaultAutoDebitPayCallback.class);

    /**
     * on response.result.resultStatus = S
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onSstatus(APSClient apsClient, AutoDebitPayRequest request,
                          AutoDebitPayResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onSstatus][AutoDebitPayRequest={}],[AutoDebitPayResponse={}],",
            request, response);
        this.onPaymentSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = F
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onFstatus(APSClient apsClient, AutoDebitPayRequest request,
                          AutoDebitPayResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onFstatus][AutoDebitPayRequest={}],[AutoDebitPayResponse={}],",
            request, response);
        this.onPaymentFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onUstatus(APSClient apsClient, AutoDebitPayRequest request,
                          AutoDebitPayResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onUstatus][AutoDebitPayRequest={}],[AutoDebitPayResponse={}],",
            request, response);
        this.onPaymentUnknown(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S & result.resultCode = SUCCESS
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onPaymentSuccess(APSClient apsClient, AutoDebitPayRequest request,
                                          AutoDebitPayResponse response);

    /**
     * on response.result.resultStatus = F
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onPaymentFail(APSClient apsClient, AutoDebitPayRequest request,
                                       AutoDebitPayResponse response);

    /**
     * on response.result.resultStatus = U
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onPaymentUnknown(APSClient apsClient, AutoDebitPayRequest request,
                                          AutoDebitPayResponse response) throws AlipayPlusApiException;

}