/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.CancelPaymentPayResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;
import com.alipay.ap.request.CancelPaymentRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 * 
 * @author wangpeng
 * @version $Id: DefaultCancelPaymentCallback.java, v 0.1 2022‐06-24 10:14 wangpeng Exp $
 */
public abstract class DefaultCancelPaymentCallback extends
                                                   HttpSendCallback<CancelPaymentRequest, CancelPaymentPayResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultCancelPaymentCallback.class);

    /**
     * on response.result.resultStatus = S 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onSstatus(APSClient apsClient, CancelPaymentRequest request,
                          CancelPaymentPayResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onSstatus][CancelPaymentRequest={}],[CancelPaymentPayResponse={}],",
            request, response);
        this.onCancelPaymentSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = F 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onFstatus(APSClient apsClient, CancelPaymentRequest request,
                          CancelPaymentPayResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onFstatus][CancelPaymentRequest={}],[CancelPaymentPayResponse={}],",
            request, response);
        this.onCancelPaymentFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onUstatus(APSClient apsClient, CancelPaymentRequest request,
                          CancelPaymentPayResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onUstatus][CancelPaymentRequest={}],[CancelPaymentPayResponse={}],",
            request, response);
        this.onCancelPaymentUnknown(apsClient, request, response);

    }

    /**
     * on response.result.resultStatus = S 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelPaymentSuccess(APSClient apsClient, CancelPaymentRequest request,
                                                CancelPaymentPayResponse response) throws Exception;

    /**
     * on response.result.resultStatus = F 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelPaymentFail(APSClient apsClient, CancelPaymentRequest request,
                                             CancelPaymentPayResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelPaymentUnknown(APSClient apsClient, CancelPaymentRequest request,
                                                CancelPaymentPayResponse response) throws Exception;
}