/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.response.ConsultUnbindingResponse;
import com.alipay.ap.response.HttpReceiveResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.request.ConsultUnbindingRequest;
import com.alipay.ap.utils.SerializeUtil;

/**
 * subclass of {@link HttpReceiveCallback}
 * implements the callback to handle the response from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultConsultUnbindingCallback.java, v 0.1 2022‐07-13 18:33 wqy01239909 Exp $
 */
public abstract class DefaultConsultUnbindingCallback extends HttpReceiveCallback {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultConsultUnbindingCallback.class);

    /**
     * Process the request sent by A+
     * @param httpReceive
     * @return
     * @throws Exception
     */
    @Override
    public HttpReceiveResponse doProcess(HttpReceive httpReceive) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG]Consult unbinding doProcess [HttpReceive={}]", httpReceive);
        ConsultUnbindingRequest consultUnbindingRequest = SerializeUtil
            .deserializeRequest(httpReceive, ConsultUnbindingRequest.class);
        return this.handleConsultUnbinding(consultUnbindingRequest);

    }

    /**
     * Handle consult unbinding in ACQ
     * 
     * @param request
     * @return ConsultUnbindingResponse
     */
    public abstract ConsultUnbindingResponse handleConsultUnbinding(ConsultUnbindingRequest request);
}