/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.request.MerchantRegistrationStatusInquiryRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.MerchantInquiryRegistrationStatusResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultInquiryRegistrationStatusCallback.java, v 0.1 2022‐07-13 11:44 wqy01239909 Exp $
 */
public abstract class DefaultInquiryRegistrationStatusCallback extends
                                                               HttpSendCallback<MerchantRegistrationStatusInquiryRequest, MerchantInquiryRegistrationStatusResponse> {

    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultInquiryRegistrationStatusCallback.class);

    /**
     *  on response.result.resultStatus = F
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onFstatus(APSClient apsClient, MerchantRegistrationStatusInquiryRequest request,
                          MerchantInquiryRegistrationStatusResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onFstatus][MerchantRegistrationStatusInquiryRequest={}],[MerchantInquiryRegistrationStatusResponse={}],",
            request, response);
        this.onInquiryRegistrationStatusFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onUstatus(APSClient apsClient, MerchantRegistrationStatusInquiryRequest request,
                          MerchantInquiryRegistrationStatusResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onUstatus][MerchantRegistrationStatusInquiryRequest={}],[MerchantInquiryRegistrationStatusResponse={}],",
            request, response);
        this.onInquiryRegistrationStatusUnknown(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onSstatus(APSClient apsClient, MerchantRegistrationStatusInquiryRequest request,
                          MerchantInquiryRegistrationStatusResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onSstatus][MerchantRegistrationStatusInquiryRequest={}],[MerchantInquiryRegistrationStatusResponse={}],",
            request, response);
        this.onInquiryRegistrationStatusSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S 
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onInquiryRegistrationStatusSuccess(APSClient apsClient,
                                                            MerchantRegistrationStatusInquiryRequest request,
                                                            MerchantInquiryRegistrationStatusResponse response) throws Exception;

    /**
     * on response.result.resultStatus = F 
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onInquiryRegistrationStatusFail(APSClient apsClient,
                                                         MerchantRegistrationStatusInquiryRequest request,
                                                         MerchantInquiryRegistrationStatusResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U 
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onInquiryRegistrationStatusUnknown(APSClient apsClient,
                                                            MerchantRegistrationStatusInquiryRequest request,
                                                            MerchantInquiryRegistrationStatusResponse response) throws Exception;
}