/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.enums.ResultStatusType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.request.PayNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

/**
 * subclass of {@link HttpReceiveCallback}
 * implements the callback to handle the request from A+
 *
 * @author mulin.xl
 * @version $Id: DefaultPaymentNotifyCallback.java, v 0.1 2022‐07‐13 13:31 mulin.xl Exp $$
 */
public abstract class DefaultPaymentNotifyCallback extends HttpReceiveCallback {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultPaymentNotifyCallback.class);

    /**
     * Process the request sent by A+
     * @param httpReceive
     * @return
     * @throws Exception
     */
    @Override
    public HttpReceiveResponse doProcess(HttpReceive httpReceive) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG]Payment notify doProcess [HttpReceive={}]", httpReceive);
        PayNotifyRequest payNotifyRequest = SerializeUtil.deserializeRequest(httpReceive,
            PayNotifyRequest.class);

        ResultStatusType resultStatus = payNotifyRequest.getPaymentResult().getResultStatus();
        if (resultStatus == null) {

            logger.error(
                "[ALIPAYPLUS_LOG][INVALID_PAYMENT_NOTIFY_RESULT_STATUS] resultStatus is missing");
            throw new AlipayPlusApiException(String.format("resultStatus is missing"));

        }
        switch (resultStatus) {
            case S:
                return this.onPaymentSuccess(payNotifyRequest);
            case F:
                return this.onPaymentFailure(payNotifyRequest);
            case U:
            default:

                logger.error(
                    "[ALIPAYPLUS_LOG][INVALID_PAYMENT_NOTIFY_RESULT_STATUS] cannot handle the payment notify request ,resultStatus = {} ",
                    resultStatus);

                throw new AlipayPlusApiException(String.format(
                    " cannot handle the payment notify request, resultStatus=%s", resultStatus));
        }
    }

    /**
     * payment notify succeeded
     * @param request request {@link PayNotifyRequest}
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onPaymentSuccess(PayNotifyRequest request);

    /**
     * payment notify failed
     * @param request request {@link PayNotifyRequest}
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onPaymentFailure(PayNotifyRequest request);
}