/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.RefundResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 *
 * @author wangpeng
 * @version $Id: DefaultRefundPaymentCallback.java, v 0.1 2022‐06-28 18:37:18 wangpeng Exp $
 */
public abstract class DefaultRefundPaymentCallback extends

                                                   HttpSendCallback<RefundRequest, RefundResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultRefundPaymentCallback.class);

    /**
     * on response.result.resultStatus = S 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onSstatus(APSClient apsClient, RefundRequest request,
                          RefundResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onSstatus][RefundRequest={}],[RefundResponse={}],", request,
            response);
        // override this method to customise business codes
        this.onRefundPaymentSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = F 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onFstatus(APSClient apsClient, RefundRequest request,
                          RefundResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onFstatus][RefundRequest={}],[RefundResponse={}],", request,
            response);
        // override this method to customise business codes
        this.onRefundPaymentFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    @Override
    public void onUstatus(APSClient apsClient, RefundRequest request,
                          RefundResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onUstatus][RefundRequest={}],[RefundResponse={}],", request,
            response);
        // override this method to customise business codes
        this.onRefundPaymentUnknown(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onRefundPaymentSuccess(APSClient apsClient, RefundRequest request,
                                                RefundResponse response) throws Exception;

    /**
     * on response.result.resultStatus = F 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onRefundPaymentFail(APSClient apsClient, RefundRequest request,
                                             RefundResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U 
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onRefundPaymentUnknown(APSClient apsClient, RefundRequest request,
                                                RefundResponse response) throws Exception;

}