/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.enums.RegistrationStatus;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.request.MerchantRegistrationStatusNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

/**
 * subclass of {@link HttpReceiveCallback}
 * implements the callback to handle the request from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultRegistrationStatusNotifyCallback.java, v 0.1 2022‐07-13 17:32 wqy01239909 Exp $
 */
public abstract class DefaultRegistrationStatusNotifyCallback extends HttpReceiveCallback {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultRegistrationStatusNotifyCallback.class);

    /**
     * Process the request sent by A+
     * @param httpReceive
     * @return
     * @throws Exception
     */
    @Override
    public HttpReceiveResponse doProcess(HttpReceive httpReceive) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG]Registration status notify doProcess [HttpReceive={}]",
            httpReceive);
        MerchantRegistrationStatusNotifyRequest registrationStatusNotifyRequest = SerializeUtil
            .deserializeRequest(httpReceive, MerchantRegistrationStatusNotifyRequest.class);

        RegistrationStatus registrationStatus = registrationStatusNotifyRequest
            .getRegistrationResult().getRegistrationStatus();
        if (registrationStatus == null) {

            logger.error(
                "[ALIPAYPLUS_LOG][INVALID_REGISTRATION_NOTIFY_STATUS] registrationStatus is missing");
            throw new AlipayPlusApiException(String.format("registrationStatus is missing"));

        }
        switch (registrationStatus) {
            case COMPLETED:
                return this.onCompleted(registrationStatusNotifyRequest);
            case PENDING:
                return this.onPending(registrationStatusNotifyRequest);
            default:

                logger.error(
                    "[ALIPAYPLUS_LOG][INVALID_REGISTRATION_NOTIFY_STATUS] cannot handle the registration status notify request ,registrationStatus = {} ",
                    registrationStatus);

                throw new AlipayPlusApiException(String.format(
                    "cannot handle the registration status notify request, registrationStatus=%s",
                    registrationStatus));

        }
    }

    /**
     * registration is completed
     *
     * @param request request {@link MerchantRegistrationStatusNotifyRequest}
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onCompleted(MerchantRegistrationStatusNotifyRequest request);

    /**
     * registration is pending
     *
     * @param request request {@link MerchantRegistrationStatusNotifyRequest}
     * @return HttpReceiveResponse
     */
    public abstract HttpReceiveResponse onPending(MerchantRegistrationStatusNotifyRequest request);

}