/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.response.APSResponse;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * The base class {@code HttpSendCallback} and its subclasses are used to handle the response from A+
 *
 * @author wangpeng
 * @version $Id: HttpSendCallback.java, v 0.1 2022‐06-30 19:04 Callback Exp $
 */
public abstract class HttpSendCallback<R extends APSRequest, P extends APSResponse> {

    /**
     * default construction method
     */
    public HttpSendCallback() {

    }

    /**
     * on response.result.resultStatus = F after calling {@link APSClient}
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    public abstract void onFstatus(APSClient apsClient, R request, P response) throws Exception;

    /**
     * on response.result.resultStatus = U after calling {@link APSClient}
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    public abstract void onUstatus(APSClient apsClient, R request, P response) throws Exception;

    /**
     * on response.result.resultStatus = S  after calling {@link APSClient}
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    public abstract void onSstatus(APSClient apsClient, R request, P response) throws Exception;

}
