/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.enums;

/**
 * This parameter specifies the method through which the auth client can get the access token.
 * @author wqy01239909
 * @version $Id: GrantType.java, v 0.1 2022‐07-08 15:39 wqy01239909 Exp $
 */
public enum GrantType {
                       /**
                        * indicates that the auth client uses the authorization code (authCode) to get an access token.
                        */
                       AUTHORIZATION_CODE("AUTHORIZATION_CODE", "authorization code"),
                       /**
                        * indicates that the auth client uses the refresh token (refreshToken) to get a new access token to replace the existing one. 
                        */
                       REFRESH_TOKEN("REFRESH_TOKEN", "refresh token");

    private String code;

    private String desc;

    /**
     * Gets authorization Notify Type by code.
     *
     * @param code the code
     * @return the authorization Notify Type
     */
    public static GrantType getByCode(String code) {
        for (GrantType value : values()) {
            if (value.getCode().equals(code)) {
                return value;
            }
        }
        return null;
    }

    GrantType(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    /**
     * Getter method for property code.
     *
     * @return property value of code
     */
    public String getCode() {
        return code;
    }

    /**
     * Setter method for property code.
     *
     * @param code value to be assigned to property code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Getter method for property desc.
     *
     * @return property value of desc
     */
    public String getDesc() {
        return desc;
    }

    /**
     * Setter method for property desc.
     *
     * @param desc value to be assigned to property desc
     */
    public void setDesc(String desc) {
        this.desc = desc;
    }
}