/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.enums;

/**
 * The presentation mode that the merchant wants to use to display the payment methods on the cashier page.
 * The value is fixed as UNIFIED: indicates the presentation mode where a merchant presents the unified Alipay+ payment method on the cashier page.
 * @author wqy01239909
 * @version $Id: PresentmentMode.java, v 0.1 2022‐07-11 10:43 wqy01239909 Exp $
 */
public enum PresentmentMode {
                             /**
                              * The value is fixed as UNIFIED: indicates the presentation mode where a merchant presents the unified Alipay+ payment method on the cashier page.
                              */
                             UNIFIED("UNIFIED", "UNIFIED");

    private String code;

    private String desc;

    /**
     * Gets authorization Notify Type by code.
     *
     * @param code the code
     * @return the authorization Notify Type
     */
    public static PresentmentMode getByCode(String code) {
        for (PresentmentMode value : values()) {
            if (value.getCode().equals(code)) {
                return value;
            }
        }
        return null;
    }

    PresentmentMode(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    /**
     * Getter method for property code.
     *
     * @return property value of code
     */
    public String getCode() {
        return code;
    }

    /**
     * Setter method for property code.
     *
     * @param code value to be assigned to property code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Getter method for property desc.
     *
     * @return property value of desc
     */
    public String getDesc() {
        return desc;
    }

    /**
     * Setter method for property desc.
     *
     * @param desc value to be assigned to property desc
     */
    public void setDesc(String desc) {
        this.desc = desc;
    }
}
