/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.enums;

/**
 * Scope of the Authorization, which means what resources or capabilities are authorized to the auth client.
 * @author wqy01239909
 * @version $Id: Scope.java, v 0.1 2022‐07-08 15:26 wqy01239909 Exp $
 */
public enum Scope {
                   /**
                    * indicates that the auth client can use an access token to deduct funds from the user's MPP app for Auto Debit payments.
                    */
                   AGREEMENT_PAY("AGREEMENT_PAY", "agreement pay"),
                   /**
                    *  indicates that the auth client can obtain the desensitized user login ID for the MPP app by using the applyToken API. 
                    */
                   USER_LOGIN_ID("USER_LOGIN_ID", "user login id");

    private String code;

    private String desc;

    /**
     * Gets authorization Notify Type by code.
     *
     * @param code the code
     * @return the authorization Notify Type
     */
    public static Scope getByCode(String code) {
        for (Scope value : values()) {
            if (value.getCode().equals(code)) {
                return value;
            }
        }
        return null;
    }

    Scope(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    /**
     * Getter method for property code.
     *
     * @return property value of code
     */
    public String getCode() {
        return code;
    }

    /**
     * Setter method for property code.
     *
     * @param code value to be assigned to property code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Getter method for property desc.
     *
     * @return property value of desc
     */
    public String getDesc() {
        return desc;
    }

    /**
     * Setter method for property desc.
     *
     * @param desc value to be assigned to property desc
     */
    public void setDesc(String desc) {
        this.desc = desc;
    }
}