/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.job;

import java.io.IOException;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * this executor supplies retry mechanism 
 * with which you can retry to call a+ interface synchronously many times
 * 
 * @author wangpeng
 * @version $Id: RetryExecutor.java, v 0.1 2022‐07-05 20:47 wangpeng Exp $
 */
public class RetryExecutor {

    /**
     * call a+ interface synchronously many times
     * @param retryCount max retry times
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param callback subclasses of {@link HttpSendCallback}
     * @throws IOException if io exception happens
     * @throws AlipayPlusApiException if a+ api exception happens
     * @throws Exception if normal exception happens
     */
    public static void execute(int retryCount, APSClient apsClient, APSRequest request,
                               HttpSendCallback callback) throws IOException,
                                                          AlipayPlusApiException, Exception {
        for (int i = 0; i < retryCount; i++) {

            request.getRetryContext().getRetryCount().incrementAndGet();

            if ((retryCount - 1) == i) {
                request.getRetryContext().setMaxRetry(true);
            }

            apsClient.execute(request, callback);

            if (request.getRetryContext().isContinueRetry()) {
                // reset retry flag
                request.getRetryContext().resetContinueRetryFlag();
                continue;

            } else {
                //No need retry, break the loop
                break;
            }

        }
    }

}
