/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.job;

import com.alipay.ap.request.ScheduleJobRequest;

/**
 * Schedule Job Support
 * 
 * @author mulin.xl
 * @version $Id: ScheduleJobSupport.java, v 0.1 2022‐07‐19 16:26 mulin.xl Exp $$
 */
public interface ScheduleJobSupport {

    /**
     * Insert a new Job to a persistent repository like Redis or DB for future execution
     * @param request Schedule Job Request
     * @return successed or not
     */
    boolean register(ScheduleJobRequest request);

    /**
     * delete Job or set delete flag in a persistent repository like Redis or DB
     * @param job
     * @return successed or not
     */
    boolean remove(ScheduleJob job);

    /**
     * update Job in a persistent repository like Redis or DB for future execution
     * @param job
     * @return successed or not
     */
    boolean update(ScheduleJob job);

    /**
     * lock Job in a persistent repository like Redis or DB for future execution
     * for preventing from running the same job concurrently
     * 
     * @param job
     * @return successed or not
     */
    boolean lock(ScheduleJob job);

    /**
     * release the lock in a persistent repository like Redis or DB for future execution
     * @param job
     * @return
     */
    boolean release(ScheduleJob job);

    /**
     * list all the schedule jobs
     * @return all the schedule jobs
     */
    ScheduleJob[] listJobs();

}