/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.job;

import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;
import com.alipay.ap.utils.DateUtil;

/**
 * Schedule task base class
 * 
 * @author mulin.xl
 * @version $Id: ScheduleRunnable.java, v 0.1 2022‐07‐19 22:25 mulin.xl Exp $$
 */
public class ScheduleRunnable implements Runnable {
    /** logger */
    private static final Logger logger = LoggerFactory.getLogger(ScheduleRunnable.class);
    /**
     * scheduleJobSupport
     */
    private ScheduleJobSupport  scheduleJobSupport;
    /**
     * apsClient
     */
    private APSClient           apsClient;
    /**
     * scheduleJob
     */
    private ScheduleJob         scheduleJob;
    /**
     * scheduledCallback
     */
    private HttpSendCallback    scheduledCallback;
    /**
     * interval
     */
    private int[]               interval;
    /**
     * apsRequest
     */
    private APSRequest          apsRequest;

    /**
     *
     * @param scheduleJobSupport
     * @param apsClient
     * @param apsRequest
     * @param scheduleJob
     * @param scheduledCallback
     * @param interval
     */
    public ScheduleRunnable(ScheduleJobSupport scheduleJobSupport, APSClient apsClient,
                            APSRequest apsRequest, ScheduleJob scheduleJob,
                            HttpSendCallback scheduledCallback, int[] interval) {
        this.scheduleJobSupport = scheduleJobSupport;
        this.apsClient = apsClient;
        this.scheduleJob = scheduleJob;
        this.scheduledCallback = scheduledCallback;
        this.interval = interval;
        this.apsRequest = apsRequest;
    }

    public ScheduleRunnable() {
    }

    @Override
    public void run() {

        boolean isContinue = false;

        boolean isLockSuccess = false;

        try {

            logger.debug("[ALIPAYPLUS_LOG][JOB_EXECUTOR] execute the job , jobId = {}, bizId = {}",
                scheduleJob.getJobId(), scheduleJob.getBizId());

            isLockSuccess = scheduleJobSupport.lock(scheduleJob);

            if (!isLockSuccess) {

                logger.warn("[ALIPAYPLUS_LOG][JOB_EXECUTOR_LOCK_FAILED] jobId = {}, bizId = {}",
                    scheduleJob.getJobId(), scheduleJob.getBizId());

                return;
            }

            apsRequest.getRetryContext().init(scheduleJob.getExecutedTimes() + 1,
                scheduleJob.getExecutedTimes() + 1 >= this.interval.length);

            apsRequest.setJobSchedule(true);

            //reset need retry flag before execute
            apsRequest.getRetryContext().resetContinueRetryFlag();

            apsClient.execute(apsRequest, scheduledCallback);

            isContinue = apsRequest.getRetryContext().isContinueRetry();

            logger.debug(
                "[ALIPAYPLUS_LOG][JOB_EXECUTOR] job execution finished , jobId = {}, bizId = {}, isContinue = {}",
                scheduleJob.getJobId(), scheduleJob.getBizId(),isContinue);

        } catch (Exception e) {
            logger.warn(
                "[ALIPAYPLUS_LOG][JOBEXECUTOR_RUNNING_TASK_ERROR] exception happens when jobExecutor runs a task. {}",
                e);
        } finally {

            if (isLockSuccess) {

                if (!isContinue) {

                    boolean isRemoved = scheduleJobSupport.remove(scheduleJob);

                    logger.debug("[ALIPAYPLUS_LOG][JOB_EXECUTOR] isRemoved = {} , jobId = {}",
                        isRemoved, scheduleJob.getJobId());

                } else {

                    if (scheduleJob.getExecutedTimes() + 1 >= interval.length) {

                        boolean isRemoved = scheduleJobSupport.remove(scheduleJob);
                        logger.debug("[ALIPAYPLUS_LOG][JOB_EXECUTOR] isRemoved = {} , jobId = {}",
                            isRemoved, scheduleJob.getJobId());

                    } else {

                        scheduleJob.setExecutedTimes(scheduleJob.getExecutedTimes() + 1);
                        scheduleJob.setNextExecuteTime(DateUtil.triggerTime(
                            interval[scheduleJob.getExecutedTimes()], TimeUnit.SECONDS));
                        boolean isUpdated = scheduleJobSupport.update(scheduleJob);
                        logger.debug("[ALIPAYPLUS_LOG][JOB_EXECUTOR] isUpdated = {} , jobId = {}",
                            isUpdated, scheduleJob.getJobId());
                    }
                }

                // release lock
                boolean isReleased = scheduleJobSupport.release(scheduleJob);

                logger.debug("[ALIPAYPLUS_LOG][JOB_EXECUTOR] isReleased = {} , jobId = {}",
                    isReleased, scheduleJob.getJobId());
            }
        }
    }

}