/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.net;

import java.io.IOException;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.request.APSRequest;

/**
 * an abstract class that defines an APS client
 * 
 * @author mulin.xl
 * @version $Id: APSClient.java, v 0.1 2022‐06‐23 13:20 mulin.xl Exp $$
 */
public abstract class APSClient {

    /** ap settings **/
    private APSSettings apsSettings;

    /**
     * Construct method
     * 
     * @param apsSettings ap settings {@link APSSettings}
     */
    public APSClient(APSSettings apsSettings) {
        this.apsSettings = apsSettings;
    }

    /**
     * abstract method and need to implement
     * 
     * @param request ap request {@link APSRequest}
     * @param callback {@link HttpSendCallback}
     * @param <R> subclasses of {@link APSRequest}
     * @param <P> subclasses of {@link APSResponse}
     * @throws if alipay plus exception happens
     */
    public abstract <R extends APSRequest, P extends APSResponse> void execute(R request,
                                                                               HttpSendCallback<R, P> callback) throws IOException,
            AlipayPlusApiException,
                                                                                                                Exception;

    /**
     * Getter method for property apsSettings.
     *
     * @return property value of apsSettings
     */
    public APSSettings getApsSettings() {
        return apsSettings;
    }

    /**
     * Setter method for property apsSettings.
     *
     * @param apsSettings value to be assigned to property apsSettings
     */
    public void setApsSettings(APSSettings apsSettings) {
        this.apsSettings = apsSettings;
    }
}