/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.request;

import com.alipay.ap.enums.HttpMethod;
import com.alipay.ap.model.RequestHeader;
import com.alipay.ap.model.RetryContext;
import com.alipay.ap.model.ToString;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.utils.GsonUtils;

/**
 * APS Base Request
 * @author mulin.xl
 * @version $Id: APSRequest.java, v 0.1 2022‐06‐23 11:45 mulin.xl Exp $$
 */
public abstract class APSRequest<T extends APSResponse> extends ToString implements Cloneable {
    /** serialVersionUID */
    private static final long          serialVersionUID = -476563025141467698L;
    /**
     * requestURI
     */
    private transient    String        requestURI;
    /**
     * requestHeader
     */
    private transient    RequestHeader requestHeader    = new RequestHeader();
    /**
     * httpMethod
     */
    private transient    String        httpMethod       = HttpMethod.POST.name();
    /**
     * retryContext
     */
    private transient    RetryContext  retryContext     = new RetryContext();

    /**
     * true if the request is executed by job schedule
     * false if the request is not executed by job schedule
     */
    private transient boolean       isJobSchedule    = false;

    public APSRequest(String requestURI) {
        this.requestURI = requestURI;
    }

    public APSRequest() {

    }

    public String toJSONContent() {
        return GsonUtils.toJSONString(this);

    }

    public abstract Class<T> getResponseClass();

    /**
     * Getter method for property requestURI.
     *
     * @return property value of requestURI
     */
    public String getRequestURI() {
        return requestURI;
    }

    /**
     * Getter method for property requestHeader.
     *
     * @return property value of requestHeader
     */
    public RequestHeader getRequestHeader() {
        return requestHeader;
    }

    /**
     * Setter method for property requestHeader.
     *
     * @param requestHeader value to be assigned to property requestHeader
     */
    public void setRequestHeader(RequestHeader requestHeader) {
        this.requestHeader = requestHeader;
    }

    /**
     * Getter method for property httpMethod.
     *
     * @return property value of httpMethod
     */
    public String getHttpMethod() {
        return httpMethod;
    }

    /**
     * Setter method for property httpMethod.
     *
     * @param httpMethod value to be assigned to property httpMethod
     */
    public void setHttpMethod(String httpMethod) {
        this.httpMethod = httpMethod;
    }

    /**
     * Getter method for property retryContext.
     *
     * @return property value of retryContext
     */
    public RetryContext getRetryContext() {
        return retryContext;
    }

    /**
     * Setter method for property retryContext.
     *
     * @param retryContext value to be assigned to property retryContext
     */
    public void setRetryContext(RetryContext retryContext) {
        this.retryContext = retryContext;
    }

    @Override
    public APSRequest clone() throws CloneNotSupportedException {
        APSRequest apsRequest = (APSRequest) super.clone();
        apsRequest.setRequestHeader(this.getRequestHeader().clone());
        apsRequest.setRetryContext(this.getRetryContext().clone());
        return apsRequest;
    }

    public boolean isJobSchedule() {
        return isJobSchedule;
    }

    public void setJobSchedule(boolean jobSchedule) {
        isJobSchedule = jobSchedule;
    }
}