/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.utils;

import java.util.Map;

import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.model.RequestHeader;
import com.alipay.ap.request.APSRequest;

/**
 * Serialization tool class
 *
 * @author mulin.xl
 * @version $Id: SerializeUtils.java, v 0.1 2022‐07‐13 13:44 mulin.xl Exp $$
 */
public class SerializeUtil {

    /**
     * deserialize http response
     *
     * @param request ap request {@link APSRequest}
     * @param rspBody http response body string
     * @param responseHeader  http response body object               
     * @param <R> subclasses of {@link APSRequest}
     * @param <P> subclasses of {@link APSResponse}
     *
     */
    public static <R extends APSRequest, P extends APSResponse> P deserializeResponse(R request,
                                                                                      String rspBody,
                                                                                      Map<String, String> responseHeader) {

        P apsResponse = (P) GsonUtils.toJavaObject(rspBody, request.getResponseClass());

        apsResponse.getResponseHeader().fromMap(responseHeader);

        return apsResponse;

    }

    /**
     * deserialize httpReceive
     *
     * @param httpReceive httpReceive {@link HttpReceive}
     * @param requestClass the deserialized object
     * @param <T> subclasses of {@link APSRequest}
     * @return
     * @throws Exception
     */
    public static <T extends APSRequest> T deserializeRequest(HttpReceive httpReceive,
                                                              Class<T> requestClass) throws Exception {

        String receiveReqBody = new String(httpReceive.getReceiveReqBody(),
            CommonConstants.DEFAULT_CHARSET);

        T request = GsonUtils.toJavaObject(receiveReqBody, requestClass);

        RequestHeader requestHeader = new RequestHeader();
        requestHeader.fromMap(httpReceive.getReceiveRequestHeader());

        request.setRequestHeader(requestHeader);

        return request;
    }
}