/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultAutoDebitPayCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.enums.OsType;
import com.alipay.ap.enums.PaymentMethodType;
import com.alipay.ap.enums.PresentmentMode;
import com.alipay.ap.enums.TerminalType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.Address;
import com.alipay.ap.model.Amount;
import com.alipay.ap.model.Env;
import com.alipay.ap.model.Merchant;
import com.alipay.ap.model.Order;
import com.alipay.ap.model.PaymentFactor;
import com.alipay.ap.model.PaymentMethod;
import com.alipay.ap.model.SettlementStrategy;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.AutoDebitPayRequest;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.response.AutoDebitPayResponse;
import com.alipay.ap.utils.DateUtil;
import com.alipay.ap.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * After the account binding process is completed, the merchant as the auth client,
 * obtains the user authorization (the access token). Then the merchant can initiate a payment request
 * according to the business scenario, which can be an individual payment request or one of the subscription payment requests.
 * The ACQP then uses the access token to assemble a request to call the pay interface.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: AutoDebitPayDemoCode.java, v 0.1 2022‐07-11 17:16 wqy01239909 Exp $
 */
public class AutoDebitPayDemoCode {
    private static final Logger logger             = LoggerFactory
        .getLogger(AutoDebitPayDemoCode.class);
    public static final String  PAYMENT_REQUEST_ID = StringUtil.genRequestID("payment");

    private static final int    AUTO_DEBIT_INQUIRY = 5;

    public static void main(String[] args) {
        APSClient apsClient = DefaultAPS.getInstance().getApsClient();

        AutoDebitPayDemoCode demoCode = new AutoDebitPayDemoCode();

        AutoDebitPayRequest autoDebitPayRequest = demoCode.buildAutoDebitPayRequest();
        HttpSendCallback autoDebitPayCallback = demoCode.buildAutoDebitPayCallback();

        demoCode.autoDebitPay(apsClient, autoDebitPayRequest, autoDebitPayCallback);
    }

    private void autoDebitPay(APSClient apsClient, AutoDebitPayRequest autoDebitPayRequest,
                              HttpSendCallback autoDebitPayCallback) {
        try {

            //TODO: ACQ handling the autoDebit pay request before calling A+ api

            //Calling ApsClient.execute to initiate a autoDebit payment
            apsClient.execute(autoDebitPayRequest, autoDebitPayCallback);

            //TODO: ACQ handling the autoDebit pay request after calling A+ api

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] autoDebitPayRequest]={},stackError={}",
                autoDebitPayRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] autoDebitPayRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                autoDebitPayRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] autoDebitPayRequest={},stackError={}",
                autoDebitPayRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private AutoDebitPayRequest buildAutoDebitPayRequest() {
        AutoDebitPayRequest autoDebitPayRequest = new AutoDebitPayRequest();
        // The unique ID that is assigned by the ACQP to identify a payment order.
        autoDebitPayRequest.setPaymentRequestId(PAYMENT_REQUEST_ID);

        Amount paymentAmount = new Amount();
        paymentAmount.setCurrency("JPY");
        paymentAmount.setValue(100);
        autoDebitPayRequest.setPaymentAmount(paymentAmount);

        PaymentMethod paymentMethod = new PaymentMethod();
        // fixed value: CONNECT_WALLET
        paymentMethod.setPaymentMethodType(PaymentMethodType.CONNECT_WALLET);
        paymentMethod.setPaymentMethodId("281006050000000000125733DAHJ1212");
        autoDebitPayRequest.setPaymentMethod(paymentMethod);

        PaymentFactor paymentFactor = new PaymentFactor();
        paymentFactor.setAgreementPayment(true);
        paymentFactor.setPresentmentMode(PresentmentMode.UNIFIED);
        autoDebitPayRequest.setPaymentFactor(paymentFactor);

        Order order = new Order();
        order.setReferenceOrderId("102775765075669");
        order.setOrderDescription("SHOES");

        Amount orderAmount = new Amount();
        orderAmount.setCurrency("JPY");
        // The value of the amount as a natural number. By default, the value of this parameter is in the smallest currency unit.
        orderAmount.setValue(100);
        Merchant merchant = new Merchant();
        merchant.setMerchantMCC("1405");
        merchant.setReferenceMerchantId("M000000012121");
        merchant.setMerchantName("UGG");
        Address merchantAddress = new Address();
        merchantAddress.setRegion("JP");
        merchantAddress.setCity("xxx");
        merchant.setMerchantAddress(merchantAddress);
        order.setMerchant(merchant);
        order.setOrderAmount(orderAmount);
        Env env = new Env();
        env.setTerminalType(TerminalType.APP);
        env.setOsType(OsType.IOS);
        order.setEnv(env);
        autoDebitPayRequest.setOrder(order);

        // Required if the settlement currency is different from the payment currency that is specified for the paymentAmount.currency parameter.
        SettlementStrategy settlementStrategy = new SettlementStrategy();
        settlementStrategy.setSettlementCurrency("JPY");// following the contract
        autoDebitPayRequest.setSettlementStrategy(settlementStrategy);

        autoDebitPayRequest.setPaymentNotifyUrl(
            "https://xmock.inc.alipay.net/api/Ipay/globalSite/automtion/paymentNotify.htm");

        // Required if the ACQP wants the payment order to be completed before a specific time.
        // In this scenario, the default value of this parameter is 1 minute.
        // The specified expiration time must be shorter than the default expiration time; otherwise, the default expiration time takes effect instead of the specified expiration time.
        // The value follows the ISO 8601 standard format. For example, "2019-11-27T12:01:01+08:00".
        Calendar nowTime = Calendar.getInstance();
        nowTime.add(Calendar.SECOND, 30);
        autoDebitPayRequest
            .setPaymentExpiryTime(DateUtil.getISODateTimeStr(new Date(nowTime.getTimeInMillis())));
        return autoDebitPayRequest;
    }

    private HttpSendCallback buildAutoDebitPayCallback() {
        return new DefaultAutoDebitPayCallback() {
            @Override
            public void onPaymentSuccess(APSClient apsClient, AutoDebitPayRequest request,
                                         AutoDebitPayResponse response) {
                logger.debug("[onPaymentSuccess] autoDebitPayRequest={},autoDebitPayResponse={}",
                    request, response);
                // Payment succeeds, which means the money is deducted from the user.
                // TODO: Update the status from merchant side.

            }

            @Override
            public void onPaymentFail(APSClient apsClient, AutoDebitPayRequest request,
                                      AutoDebitPayResponse response) {
                logger.debug("[onPaymentFail] autoDebitPayRequest={},autoDebitPayResponse={}",
                    request, response);
                // Payment fails.
                // TODO: Take actions according to the error message in result.resultCode.
            }

            @Override
            public void onPaymentUnknown(APSClient apsClient, AutoDebitPayRequest request,
                                         AutoDebitPayResponse response) throws AlipayPlusApiException {
                logger.debug("[onPaymentUnknown] autoDebitPayRequest={},autoDebitPayResponse={}",
                    request, response);
                // Payment in processing. 
            }
        };
    }

    private InquiryPaymentRequest buildInquiryPaymentRequest() {

        InquiryPaymentRequest inquiryPaymentRequest = new InquiryPaymentRequest();
        inquiryPaymentRequest.setPaymentRequestId(PAYMENT_REQUEST_ID);

        return inquiryPaymentRequest;
    }

}