/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import com.alipay.ap.callbacks.DefaultCancelPaymentCallback;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CancelPaymentRequest;
import com.alipay.ap.response.CancelPaymentPayResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * CancelPayment Callback With Schedule
 * @author wangpeng
 * @version $Id: CancelPaymentCallbackWithSchedule.java, v 0.1 2022‐08-23 16:08 wangpeng Exp $
 */
public class CancelPaymentCallbackWithSchedule extends DefaultCancelPaymentCallback {

    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(CancelPaymentCallbackWithSchedule.class);

    @Override
    public void onCancelPaymentSuccess(APSClient apsClient, CancelPaymentRequest request,
                                       CancelPaymentPayResponse response) {

        logger.debug(
            "[onCancelPaymentSuccess][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously   */
        // 1. TODO Take actions for payment cancel success in ACQ on asynchronous execution scene.
        // 2. TODO Notify the result to upstream system/partner system.
        /** The merchant can reinitiate a new payment request */

    }

    @Override
    public void onCancelPaymentFail(APSClient apsClient, CancelPaymentRequest request,
                                    CancelPaymentPayResponse response) {

        logger.debug(
            "[onCancelPaymentFail][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously   */
        // 1. TODO Take actions according to the error code in result.resultCode.
        // 2. TODO If the result.resultCode value returns as CANCEL_WINDOW_EXCEED，please call the refund interface.
        // 3. TODO Notify result to upstream system/partner system

    }

    @Override
    public void onCancelPaymentUnknown(APSClient apsClient, CancelPaymentRequest request,
                                       CancelPaymentPayResponse response) throws Exception {

        logger.debug(
            "[onCancelPaymentUnknown][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously */
        // TODO Take actions for payment cancel unknown.
        if (request.getRetryContext().isMaxRetry()) {
            // TODO If keep receiving the result that indicates the status is U contact connect_support@service.alipay.com.
        } else {
            /** register the schedule for cancel retry asynchronously */
            //need retry flag
            request.getRetryContext().setContinueRetry(true);
        }
    }
}