/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import com.alipay.ap.callbacks.DefaultInquiryPaymentCallback;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.response.InquiryPaymentResponse;
import com.alipay.ap.utils.GsonUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author wqy01239909
 * @version $Id: InquiryPaymentCallbackWithSchedule.java, v 0.1 2022‐07-06 10:29 wqy01239909 Exp $
 */
public class InquiryPaymentCallbackWithSchedule extends DefaultInquiryPaymentCallback {

    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(InquiryPaymentCallbackWithSchedule.class);

    @Override
    public void onPaymentInquiryFail(APSClient apsClient, InquiryPaymentRequest request,
                                     InquiryPaymentResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onPaymentInquiryFail][isJobSchedule={}] inquiryPaymentResponse={} , inquiryPaymentResponse={}",
            request.isJobSchedule(), request, response);

        if (ResultCode.ORDER_NOT_EXIST == ResultCode
            .getByCode(response.getResult().getResultCode())) {
            // For the cases where the order does not exist (result.resultStatus == F && resultcode == ORDER_NOT_EXIST)
            // keep querying for 10 minutes with an incremental frequency

            /** implement the callback if this request is executed asynchronously */
            if (request.getRetryContext().isMaxRetry()) {
                // If keep receiving the same result, call cancel payment interface.
                // 1. TODO ACQ call the cancel payment interface.  
            } else {
                /** register the schedule for inquiry payment retry */
                //APSJobExecutor.instance.put(apsClient, this, request,
                //    apsClient.getApsSettings().inquiryInterval);
                //need retry flag
                request.getRetryContext().setContinueRetry(true);
            }
        } else {
            // For the cases where the payment request fails (result.resultStatus == F && resultcode != ORDER_NOT_EXIST), 
            // TODO take actions according to the error message in result.resultCode or contact connect_support@service.alipay.com.

        }

    }

    @Override
    public void onPaymentInquiryUnknown(APSClient apsClient, InquiryPaymentRequest request,
                                        InquiryPaymentResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onPaymentInquiryUnknown][isJobSchedule={}] inquiryPaymentResponse={},inquiryPaymentResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously */
        if (request.getRetryContext().isMaxRetry()) {
            // If keep receiving the same result
            // TODO Take actions for inquiry payment unknown 

        } else {
            /** this is optional to register the schedule for inquiry payment retry */
            //APSJobExecutor.instance.put(apsClient, this, request,
            //    apsClient.getApsSettings().inquiryInterval);
            //need retry flag
            request.getRetryContext().setContinueRetry(true);
        }

    }

    @Override
    public void onPaymentStatusSuccess(APSClient apsClient, InquiryPaymentRequest request,
                                       InquiryPaymentResponse response) {
        logger.debug(
            "[ALIPAYPLUS_LOG][onPaymentStatusSuccess][isJobSchedule={}] inquiryPaymentResponse={},inquiryPaymentResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously */
        // TODO Update the status from your server side.
        // TODO Notify the result to upstream/partner system

    }

    @Override
    public void onPaymentStatusFail(APSClient apsClient, InquiryPaymentRequest request,
                                    InquiryPaymentResponse response) {
        logger.info("[ALIPAYPLUS_LOG][onPaymentStatusFail][isJobSchedule={}] inquiryPaymentResponse={}",
            request.isJobSchedule(), GsonUtils.toJSONString(response));

        /** If the paymentResult.resultCode value is ORDER_IS_CLOSED, the order is closed or cancelled; otherwise, the payment fails.*/

        /** implement the callback if this request is executed asynchronously */
        // TODO: Take actions according to the error message in result.resultCode
        // TODO Notify the result to upstream/partner system and The merchant can reinitiate a payment request in a new API call
    }

    @Override
    public void onPaymentStatusProcessing(APSClient apsClient, InquiryPaymentRequest request,
                                          InquiryPaymentResponse response) throws Exception {

        logger.info(
            "[ALIPAYPLUS_LOG][onPaymentStatusProcessing][isJobSchedule={}] inquiryPaymentResponse={}",
            request.isJobSchedule(), GsonUtils.toJSONString(response));
        /** implement the callback if this request is executed asynchronously */
        // TODO Take actions for inquiry payment resultStatus = S and resultStatus = U
        if (request.getRetryContext().isMaxRetry()) {

            // TODO Take actions for payment status is unknown after reaching the max retry times
            // TODO: call the cancelPayment interface

        } else {
            /** register the schedule for inquiry payment retry */
            /** It is recommended to call the inquiryPayment API in the form of polling: keep querying for 10 minutes with an incremental frequency, after your server side gets the response of the pay API.*/
            //APSJobExecutor.instance.put(apsClient, this, request,
            //    apsClient.getApsSettings().inquiryInterval);
            //need retry flag
            request.getRetryContext().setContinueRetry(true);
        }

    }
}