/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.callbacks.DefaultRefundPaymentCallback;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.response.RefundResponse;

/**
 * Refund Payment Callback With Schedule
 * @author wangpeng
 * @version $Id: RefundPaymentCallbackWithSchedule.java, v 0.1 2022‐08-23 16:17 wangpeng Exp $
 */
public class RefundPaymentCallbackWithSchedule extends DefaultRefundPaymentCallback {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(RefundPaymentCallbackWithSchedule.class);

    @Override
    public void onRefundPaymentSuccess(APSClient apsClient, RefundRequest request,
                                       RefundResponse response) {
        logger.debug(
            "[onRefundPaymentSuccess][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously   */
        // 1. TODO Update the order status in your side to the value such as refunded.
        // 2. TODO Notify the result to upstream system/partner system.
    }

    @Override
    public void onRefundPaymentFail(APSClient apsClient, RefundRequest request,
                                    RefundResponse response) {
        logger.debug(
            "[onPaymentRefundFail][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously   */
        // 1. TODO Take actions according to the error message in the result.resultMessage parameter.
        // 2. TODO Notify the result to upstream system/partner system.
    }

    @Override
    public void onRefundPaymentUnknown(APSClient apsClient, RefundRequest request,
                                       RefundResponse response) throws Exception {
        logger.debug(
            "[onRefundPaymentUnknown][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
            request.isJobSchedule(), request, response);

        /** implement the callback if this request is executed asynchronously */
        // TODO Take actions for refund payment unknown.
        if (request.getRetryContext().isMaxRetry()) {
            // TODO contact connect_support@service.alipay.com.
        } else {
            /** register the schedule for refund retry asynchronously
             *  Retry the same request until getting the response. Ensure that the parameters must be the same with the previous one */
            //APSJobExecutor.instance.put(apsClient, this, request,
            //    apsClient.getApsSettings().refundInterval);
            //need retry flag
            request.getRetryContext().setContinueRetry(true);
        }
    }
}