/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.job.APSScheduleJobExecutor;
import com.alipay.ap.job.ScheduleJob;
import com.alipay.ap.job.ScheduleJobSupport;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.response.APSResponse;

/**
 * This is for the schedule job initialization Sample
 *
 * @author mulin.xl
 * @version $Id: ScheduleJobInitializeDemoCode.java, v 0.1 2022‐08‐16 10:54 mulin.xl Exp $$
 */
public class ScheduleJobInitializeDemoCode {

    /**
     * APSScheduleJobExecutor need to be initialized when application start 
     * APSScheduleJobExecutor need to be initialized before the APSScheduleJobExecutor start
     */
    public static void init() {

        //inject implementation of ScheduleJobSupport
        APSScheduleJobExecutor.getInstance().setScheduleJobSupport(implementScheduleJobSupport());
        //inject implementation of APSClient
        APSScheduleJobExecutor.getInstance().setApsClient(DefaultAPS.getInstance().getApsClient());
        //inject implementation of Callback
        APSScheduleJobExecutor.getInstance()
            .setScheduledCancelCallback(implementHttpSendCallback());
        APSScheduleJobExecutor.getInstance()
            .setScheduledInquiryCallback(implementHttpSendCallback());
        APSScheduleJobExecutor.getInstance()
            .setScheduledRefundCallback(implementHttpSendCallback());

    }

    /**
     * APSScheduleJobExecutor only need to start once.
     * APSScheduleJobExecutor need to be started when application start
     */
    public static void start() {

        //start JobExecutor
        APSScheduleJobExecutor.getInstance().startJobExecutor();
    }

    static ScheduleJobSupport implementScheduleJobSupport() {

        ScheduleJobSupport scheduleJobSupport = new ScheduleJobSupport() {
            @Override
            public boolean register(ScheduleJobRequest request) {
                //...
                return false;
            }

            @Override
            public boolean remove(ScheduleJob job) {
                return false;
            }

            @Override
            public boolean update(ScheduleJob job) {
                return false;
            }

            @Override
            public boolean lock(ScheduleJob job) {
                return false;
            }

            @Override
            public boolean release(ScheduleJob job) {
                return false;
            }

            @Override
            public ScheduleJob[] listJobs() {
                return new ScheduleJob[0];
            }
        };

        return scheduleJobSupport;
    }

    static HttpSendCallback implementHttpSendCallback() {

        return new HttpSendCallback() {
            @Override
            public void onFstatus(APSClient apsClient, APSRequest request,
                                  APSResponse response) throws Exception {

            }

            @Override
            public void onUstatus(APSClient apsClient, APSRequest request,
                                  APSResponse response) throws Exception {

            }

            @Override
            public void onSstatus(APSClient apsClient, APSRequest request,
                                  APSResponse response) throws Exception {

            }
        };
    }

}