/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.TimeUnit;

import com.alipay.ap.job.ScheduleJob;
import com.alipay.ap.job.ScheduleJobSupport;
import com.alipay.ap.request.ScheduleJobRequest;

/**
 * A demo that implements ScheduleJobSupport and manages jobs in memory
 * Forbidden to use it in prod
 * 
 * @author wangpeng
 * @version $Id: ScheduleJobSupportInMemDemo.java, v 0.1 2022‐07-20 10:22 wangpeng Exp $
 */
public class ScheduleJobSupportInMemDemo implements ScheduleJobSupport {
    private List<ScheduleJob>        jobRepo     = new CopyOnWriteArrayList<>();
    private Map<ScheduleJob, Object> jobLockRepo = new ConcurrentHashMap<ScheduleJob, Object>();
    private final Object             value       = new Object();

    @Override
    public boolean register(ScheduleJobRequest request) {
        ScheduleJob job = new ScheduleJob();
        job.setBizId(request.getBizId());
        job.setBizContext(request.getBizContext());
        job.setBizType(request.getBizType());
        job.setBizContext(request.getBizContext());
        job.setExecutedTimes(request.getExecutedTimes());
        job.setNextExecuteTime(request.getNextExecuteTime());
        job.setJobId(String.valueOf(UUID.randomUUID()));
        jobRepo.add(job);
        return true;
    }

    @Override
    public boolean remove(ScheduleJob job) {
        return jobRepo.remove(job);
    }

    @Override
    public boolean update(ScheduleJob job) {
        return true;
    }

    @Override
    public boolean lock(ScheduleJob job) {
        return jobLockRepo.putIfAbsent(job, value) == null;
    }

    @Override
    public boolean release(ScheduleJob job) {
        return jobLockRepo.remove(job) != null;
    }

    @Override
    public ScheduleJob[] listJobs() {
        return jobRepo.toArray(new ScheduleJob[jobRepo.size()]);
    }

    /**
     * Getter method for property <tt>jobRepo</tt>.
     *
     * @return property value of jobRepo
     */
    public List<ScheduleJob> getJobRepo() {
        return jobRepo;
    }

    /**
     * Setter method for property <tt>jobRepo</tt>.
     *
     * @param jobRepo value to be assigned to property jobRepo
     */
    public void setJobRepo(List<ScheduleJob> jobRepo) {
        this.jobRepo = jobRepo;
    }

    /**
     * Returns the trigger time of a delayed action.
     */
    private long triggerTime(long delay, TimeUnit unit) {
        return triggerTime(unit.toNanos((delay < 0) ? 0 : delay));
    }

    /**
     * Returns the trigger time of a delayed action.
     */
    long triggerTime(long delay) {
        return now() + delay;
    }

    /**
     * Returns current nanosecond time.
     */
    final long now() {
        return System.nanoTime();
    }
}