/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest;

import static org.mockito.Mockito.doReturn;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.DefaultAPSHttpReceiveListener;
import com.alipay.ap.response.HttpReceiveResponse;

/**
 *
 * @author wqy01239909
 * @version $Id: DefaultAPSHttpReceiveListenerTest.java, v 0.1 2022‐07-14 19:38 wqy01239909 Exp $
 */
@RunWith(MockitoJUnitRunner.Silent.class)
public class DefaultAPSHttpReceiveListenerTest {

    @Mock
    private HttpReceiveCallback           mockHttpReceiveCallback;

    @InjectMocks
    private DefaultAPSHttpReceiveListener defaultAPSHttpReceiveListener;

    @Test
    public void testOnReceive() throws Exception {

        APSSettings apsSettings = new APSSettings();
        apsSettings.setDevMode(true);
        defaultAPSHttpReceiveListener = new DefaultAPSHttpReceiveListener(apsSettings);
        HttpReceiveResponse httpReceiveResponse = new HttpReceiveResponse();
        httpReceiveResponse.setResult(Result.S);
        doReturn(httpReceiveResponse).when(mockHttpReceiveCallback)
            .doProcess(ArgumentMatchers.any());

        HttpReceive httpReceive = buildHttpReceive();

        try {
            defaultAPSHttpReceiveListener.onReceive(httpReceive, mockHttpReceiveCallback);
            Assert.assertEquals(ResultCode.SUCCESS.name(),
                httpReceiveResponse.getResult().getResultCode());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testOnReceiveVerifyFail() throws Exception {

        APSSettings apsSettings = new APSSettings();
        apsSettings.setDevMode(false);
        defaultAPSHttpReceiveListener = new DefaultAPSHttpReceiveListener(apsSettings);
        HttpReceive httpReceive = buildHttpReceiveVerifyFail();

        try {
            defaultAPSHttpReceiveListener.onReceive(httpReceive, mockHttpReceiveCallback);
        } catch (Exception e) {
            Assert.assertTrue(e instanceof AlipayPlusApiException);
        }
    }

    private HttpReceive buildHttpReceive() throws IOException {
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, "SANDBOX_xxx");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, "2022-07-19T10:59:39+08:00");
        hashMap.put(CommonConstants.REQ_SIGN_HEADER,
            "algorithm=RSA256,keyVersion=1,signature=GgJ7aAtPG5axTSBNPfU67pEY1SBxt1LXAlw3tUV3ruwy%2F05eOHz4doOfP%2BRdbPSkjxUSp6qLDz8JoIp%2FOSh0ou4JbuC9mYQdZQvDGLB1DjbL5mcX3y0Mc28UUuM7Hqfx%2F8RyFOaugudeHITQquTlVa4u6rXIxnPLubeGup9NCkso2BJNw%2FtjD13RQfPMhw%2Frrhjxpws%2B8XTC8lTM5w%2Fh9i2LIoz956VOgrwIVMd5CwvHL%2F8MV3spXxYC48LQTuy52LTg6Ym0EuKF6D%2BySL%2Fq4LMWXt8mVIY9xMTI0pme9JFv5kYsZkMCmv%2Fefph0aizgc5LpIqKjBGwfeXXhi7PEdQ%3D%3D");
        String reqBody = "{\"test\": \"test\",\"paymentResult\": {\"resultCode\": \"SUCCESS\",\"resultStatus\": \"S\",\"resultMessage\": "
                         + "\"success\"}}";
        InputStream reqBodyInputStream = new ByteArrayInputStream(reqBody.getBytes());

        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        httpReceive.setRequestURI("/pay/notify");
        httpReceive.setReceiveReqBody(IOUtils.toByteArray(reqBodyInputStream));
        httpReceive.setReceiveType(HttpReceiveType.PAYMENT_NOTIFY);
        return httpReceive;
    }

    private HttpReceive buildHttpReceiveVerifyFail() throws IOException {
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, "SANDBOX_xxx");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, "2022-07-19T10:59:39+08:00");
        hashMap.put(CommonConstants.REQ_SIGN_HEADER,
            "algorithm=RSA256,keyVersion=1,signature=GgJ7aAtPG5axTSBNPfU67pEY1SBxt1LXAlw3tUV3ruwy%2F05eOHz4doOfP%2BRdbPSkjxUSp6qLDz8JoIp%2FOSh0ou4JbuC9mYQdZQvDGLB1DjbL5mcX3y0Mc28UUuM7Hqfx%2F8RyFOaugudeHITQquTlVa4u6rXIxnPLubeGup9NCkso2BJNw%2FtjD13RQfPMhw%2Frrhjxpws%2B8XTC8lTM5w%2Fh9i2LIoz956VOgrwIVMd5CwvHL%2F8MV3spXxYC48LQTuy52LTg6Ym0EuKF6D%2BySL%2Fq4LMWXt8mVIY9xMTI0pme9JFv5kYsZkMCmv%2Fefph0aizgc5LpIqKjBGwfeXXhi7PEdQ%3D%3D");
        String reqBody = "test";
        InputStream reqBodyInputStream = new ByteArrayInputStream(reqBody.getBytes());

        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        httpReceive.setRequestURI("/pay/notify");
        httpReceive.setReceiveReqBody(IOUtils.toByteArray(reqBodyInputStream));
        httpReceive.setReceiveType(HttpReceiveType.PAYMENT_NOTIFY);
        return httpReceive;
    }
}