/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.impl.client.CloseableHttpClient;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.DefaultApacheHttpPostAPSClient;
import com.alipay.ap.request.CashierPaymentPayRequest;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.SignatureTool;

/**
 *
 * @author wqy01239909
 * @version $Id: DefaultApacheHttpPostAPSClientTest.java, v 0.1 2022‐07-14 15:14 wqy01239909 Exp $
 */
@RunWith(MockitoJUnitRunner.class)
public class DefaultApacheHttpPostAPSClientTest {

    @Mock
    private CloseableHttpClient            httpClient;

    @Spy
    @InjectMocks
    private DefaultApacheHttpPostAPSClient defaultApacheHttpPostAPSClient = new DefaultApacheHttpPostAPSClient(
        new APSSettings());

    @Test
    public void testExecute_U() {
        mockHttpClient("api_mocks/response/payment-response_u.json", 200);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try (MockedStatic<SignatureTool> serializeUtilMockedStatic = Mockito
            .mockStatic(SignatureTool.class)) {
            serializeUtilMockedStatic.when(() -> SignatureTool.verify(anyString(), anyString(),
                anyString(), anyString(), anyString(), anyString(), anyString())).thenReturn(true);

            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(cashierPaymentPayRequest != null);

            defaultApacheHttpPostAPSClient.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testExecute_S() {
        mockHttpClient("api_mocks/response/payment-response_s.json", 200);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try (MockedStatic<SignatureTool> serializeUtilMockedStatic = Mockito
            .mockStatic(SignatureTool.class)) {
            serializeUtilMockedStatic.when(() -> SignatureTool.verify(anyString(), anyString(),
                anyString(), anyString(), anyString(), anyString(), anyString())).thenReturn(true);

            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(cashierPaymentPayRequest != null);

            defaultApacheHttpPostAPSClient.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testExecute_F() {
        mockHttpClient("api_mocks/response/payment-response_f.json", 200);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try (MockedStatic<SignatureTool> serializeUtilMockedStatic = Mockito
            .mockStatic(SignatureTool.class)) {
            serializeUtilMockedStatic.when(() -> SignatureTool.verify(anyString(), anyString(),
                anyString(), anyString(), anyString(), anyString(), anyString())).thenReturn(true);

            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(cashierPaymentPayRequest != null);

            defaultApacheHttpPostAPSClient.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testExecute_verifySignException() {
        mockHttpClient("api_mocks/response/payment-response_f.json", 200);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try (MockedStatic<SignatureTool> serializeUtilMockedStatic = Mockito
            .mockStatic(SignatureTool.class)) {
            serializeUtilMockedStatic.when(() -> SignatureTool.verify(anyString(), anyString(),
                anyString(), anyString(), anyString(), anyString(), anyString()))
                .thenThrow(new Exception());

            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(cashierPaymentPayRequest != null);

            defaultApacheHttpPostAPSClient.close();
        } catch (Exception e) {
            Assert.assertTrue(e instanceof AlipayPlusApiException);
        }
    }

    @Test
    public void testExecute_verifySignFail() {
        mockHttpClient("api_mocks/response/payment-response_f.json", 200);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try (MockedStatic<SignatureTool> serializeUtilMockedStatic = Mockito
            .mockStatic(SignatureTool.class)) {
            serializeUtilMockedStatic.when(() -> SignatureTool.verify(anyString(), anyString(),
                anyString(), anyString(), anyString(), anyString(), anyString())).thenReturn(false);

            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(cashierPaymentPayRequest != null);

            defaultApacheHttpPostAPSClient.close();
        } catch (Exception e) {
            Assert.assertTrue(e instanceof AlipayPlusApiException);
        }
    }

    @Test
    public void testExecuteNot200() {
        mockHttpClient("api_mocks/response/payment-response_u.json", 400);
        CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
            MockUtil.getFileContents("api_mocks/request/payment-request.json"),
            CashierPaymentPayRequest.class);

        try {
            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
        } catch (Exception e) {
            Assert.assertTrue(e instanceof AlipayPlusApiException);
        }

    }

    @Test
    public void testExecuteNullResponse() {
        try {
            CashierPaymentPayRequest cashierPaymentPayRequest = GsonUtils.toJavaObject(
                MockUtil.getFileContents("api_mocks/request/payment-request.json"),
                CashierPaymentPayRequest.class);
            doReturn(null).when(httpClient).execute(ArgumentMatchers.any());
            defaultApacheHttpPostAPSClient.execute(cashierPaymentPayRequest, createMockCallback());
            Assert.assertTrue(true);
        } catch (Exception e) {
        }
    }

    @Test
    public void testSetApsSettings() {
        defaultApacheHttpPostAPSClient.setApsSettings(new APSSettings());
        Assert.assertTrue(defaultApacheHttpPostAPSClient.getApsSettings() != null);
    }

    protected HttpSendCallback createMockCallback() {
        HttpSendCallback mockCallback = mock(HttpSendCallback.class);
        return mockCallback;
    }

    protected void mockHttpClient(String fileName, int httpStatusCode) {
        try {

            CloseableHttpResponse closeableHttpResponse = mock(CloseableHttpResponse.class);
            StatusLine statusLine = mock(StatusLine.class);
            doReturn(httpStatusCode).when(statusLine).getStatusCode();
            doReturn(statusLine).when(closeableHttpResponse).getStatusLine();

            doReturn(closeableHttpResponse).when(httpClient).execute(ArgumentMatchers.any());

            Header header = mock(Header.class);
            doReturn("Client-Id").when(header).getName();
            doReturn("SANDBOX_xxx").when(header).getValue();

            Header header1 = mock(Header.class);
            doReturn("Response-Time").when(header1).getName();
            doReturn("2022-07-25T16:40:02+08:00").when(header1).getValue();

            Header header2 = mock(Header.class);
            doReturn("Signature").when(header2).getName();
            doReturn(
                "R9JLo09%2Bl7hE0nxP9%2BTHJ62WldRtr9R6iFyEWnC9EV4XJ7AhJ9t9bQLxC3xcNGwRFbof%2Fqrwu3o1fX457I9LitbJ50ve2j1IqrtWYayJh4Kf5YHMipod9TjU6tonzdpUp3Qx1m1Kh4JrMworyW4T7EkpXF%2BDh3KkOih6%2FNw13Hjuk20n0lAipnVr4q8MJEEQrCdOvQoRqja0QQGd4tZf0eBYVEOZX%2FQFaREnv3Hfp2YGi%2FdJCilOOH5FUPX7PU%2FbU%2FAKuS%2FZydQBL3OkU9Y70MSOjIKmqPnxssuzvOFKYmJvdJcn%2BcCflbF6MUPilxl8FPpiW98awvxFxRsBxTYryQ%3D%3D")
                    .when(header2).getValue();

            Header[] headers = { header, header1, header2 };
            doReturn(headers).when(closeableHttpResponse).getAllHeaders();

            HttpEntity HttpEntity = mock(HttpEntity.class);
            doReturn(HttpEntity).when(closeableHttpResponse).getEntity();

            InputStream contentInputStream = new ByteArrayInputStream(
                MockUtil.getFileContents(fileName).getBytes());
            doReturn(contentInputStream).when(HttpEntity).getContent();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}