/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.DefaultAuthNotifyCallback;
import com.alipay.ap.enums.AuthorizationNotifyType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.request.AuthNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

@RunWith(MockitoJUnitRunner.class)
public class DefaultAuthNotifyCallbackTest {

    @Spy
    @InjectMocks
    private DefaultAuthNotifyCallback defaultAuthNotifyCallback = new DefaultAuthNotifyCallback() {
        @Override
        public HttpReceiveResponse onTokenCreated(AuthNotifyRequest request) {
            return new HttpReceiveResponse();
        }

        @Override
        public HttpReceiveResponse onTokenCanceled(AuthNotifyRequest request) {
            return new HttpReceiveResponse();
        }

        @Override
        public HttpReceiveResponse onAuthCodeCreated(AuthNotifyRequest request) {
            return new HttpReceiveResponse();
        }
    };

    @Test
    public void doProcess() {

        AuthNotifyRequest authNotifyRequest = new AuthNotifyRequest();
        try (MockedStatic<SerializeUtil> serializeUtilMockedStatic = Mockito
            .mockStatic(SerializeUtil.class)) {
            serializeUtilMockedStatic.when(() -> SerializeUtil.deserializeRequest(any(), any()))
                .thenReturn(authNotifyRequest);
            // TOKEN_CREATED
            authNotifyRequest.setAuthorizationNotifyType(AuthorizationNotifyType.TOKEN_CREATED);
            HttpReceiveResponse httpReceiveResponse1 = defaultAuthNotifyCallback.doProcess(any());
            verify(defaultAuthNotifyCallback, times(1))
                .onTokenCreated(any(AuthNotifyRequest.class));
            Assert.assertTrue(httpReceiveResponse1 != null);

            // TOKEN_CANCELED
            authNotifyRequest.setAuthorizationNotifyType(AuthorizationNotifyType.TOKEN_CANCELED);
            HttpReceiveResponse httpReceiveResponse2 = defaultAuthNotifyCallback.doProcess(any());
            verify(defaultAuthNotifyCallback, times(1))
                .onTokenCanceled(any(AuthNotifyRequest.class));
            Assert.assertTrue(httpReceiveResponse2 != null);

            // AUTHCODE_CREATED
            authNotifyRequest.setAuthorizationNotifyType(AuthorizationNotifyType.AUTHCODE_CREATED);
            HttpReceiveResponse httpReceiveResponse3 = defaultAuthNotifyCallback.doProcess(any());
            verify(defaultAuthNotifyCallback, times(1))
                .onAuthCodeCreated(any(AuthNotifyRequest.class));
            Assert.assertTrue(httpReceiveResponse3 != null);

            // null
            authNotifyRequest.setAuthorizationNotifyType(null);
            try {
                HttpReceiveResponse httpReceiveResponse = defaultAuthNotifyCallback.doProcess(any());
            } catch (Exception e) {
                Assert.assertTrue(e instanceof AlipayPlusApiException);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}