/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.DefaultCancelTokenCallback;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.model.Result;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CancelTokenRequest;
import com.alipay.ap.response.CancelTokenResponse;

@RunWith(MockitoJUnitRunner.class)
public class DefaultCancelTokenCallbackTest {

    @Spy
    private DefaultCancelTokenCallback callback = new DefaultCancelTokenCallback() {
        @Override
        public void onCancelTokenSuccess(APSClient apsClient, CancelTokenRequest request, CancelTokenResponse response) throws Exception {

        }

        @Override
        public void onCancelTokenFail(APSClient apsClient, CancelTokenRequest request, CancelTokenResponse response) throws Exception {

        }

        @Override
        public void onCancelTokenUnknown(APSClient apsClient, CancelTokenRequest request, CancelTokenResponse response) throws Exception {

        }
    };

    @Test
    public void onSstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onSstatus(any(), any(), any());
            verify(callback, times(1)).onCancelTokenSuccess(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onFstatus_INVALID_TOKEN() {
        try {
            Assert.assertTrue(callback != null);
            CancelTokenResponse response = new CancelTokenResponse();
            Result result = new Result();
            result.setResultCode(ResultCode.INVALID_TOKEN.name());
            response.setResult(result);
            callback.onFstatus(null, null, response);
            verify(callback, times(1)).onCancelTokenSuccess(any(), any(), any());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onFstatus_EXPIRED_ACCESS_TOKEN() {
        try {
            Assert.assertTrue(callback != null);
            CancelTokenResponse response = new CancelTokenResponse();
            Result result = new Result();
            result.setResultCode(ResultCode.EXPIRED_ACCESS_TOKEN.name());
            response.setResult(result);
            callback.onFstatus(null, null, response);
            verify(callback, times(1)).onCancelTokenSuccess(any(), any(), any());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onFstatus() {
        try {
            Assert.assertTrue(callback != null);
            CancelTokenResponse response = new CancelTokenResponse();
            Result result = new Result();
            result.setResultCode(ResultCode.ORDER_NOT_EXIST.name());
            response.setResult(result);
            callback.onFstatus(null, null, response);
            verify(callback, times(1)).onCancelTokenFail(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onUstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onUstatus(any(), any(), any());
            verify(callback, times(1)).onCancelTokenUnknown(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}