/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultCashierPaymentCallback;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.model.Result;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CashierPaymentPayRequest;
import com.alipay.ap.response.CashierPaymentResponse;

@RunWith(MockitoJUnitRunner.class)
public class DefaultCashierPaymentCallbackTest {

    @Spy
    private DefaultCashierPaymentCallback callback = new DefaultCashierPaymentCallback() {
        @Override
        public void onPaymentFail(APSClient apsClient, CashierPaymentPayRequest request,
                                  CashierPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentInProcess(APSClient apsClient, CashierPaymentPayRequest request,
                                       CashierPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentUnknown(APSClient apsClient, CashierPaymentPayRequest request,
                                     CashierPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentSuccess(APSClient apsClient, CashierPaymentPayRequest request,
                                     CashierPaymentResponse response) throws Exception {

        }
    };

    @Test
    public void onSstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onSstatus(any(), any(), any());
            verify(callback, times(1)).onPaymentSuccess(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onFstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onFstatus(any(), any(), any());
            verify(callback, times(1)).onPaymentFail(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onPaymentInProcess() {
        try {
            Assert.assertTrue(callback != null);
            CashierPaymentPayRequest request = new CashierPaymentPayRequest();
            CashierPaymentResponse response = new CashierPaymentResponse();
            Result result = new Result();
            result.setResultCode(ResultCode.PAYMENT_IN_PROCESS.name());
            response.setResult(result);
            callback.onUstatus(DefaultAPS.getInstance().getApsClient(), request, response);
            verify(callback, times(1)).onPaymentInProcess(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onPaymentUnknown() {
        try {
            Assert.assertTrue(callback != null);
            CashierPaymentPayRequest request = new CashierPaymentPayRequest();
            CashierPaymentResponse response = new CashierPaymentResponse();
            Result result = new Result();
            result.setResultCode(ResultCode.UNKNOWN_EXCEPTION.name());
            response.setResult(result);
            callback.onUstatus(DefaultAPS.getInstance().getApsClient(), request, response);
            verify(callback, times(1)).onPaymentUnknown(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}