/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultInquiryPaymentCallback;
import com.alipay.ap.enums.ResultStatusType;
import com.alipay.ap.model.Result;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.response.InquiryPaymentResponse;

@RunWith(MockitoJUnitRunner.class)
public class DefaultInquiryPaymentCallbackTest {

    @Spy
    private DefaultInquiryPaymentCallback callback = new DefaultInquiryPaymentCallback() {
        @Override
        public void onPaymentInquiryFail(APSClient apsClient, InquiryPaymentRequest request,
                                         InquiryPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentInquiryUnknown(APSClient apsClient, InquiryPaymentRequest request,
                                            InquiryPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentStatusSuccess(APSClient apsClient, InquiryPaymentRequest request,
                                           InquiryPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentStatusFail(APSClient apsClient, InquiryPaymentRequest request,
                                        InquiryPaymentResponse response) throws Exception {

        }

        @Override
        public void onPaymentStatusProcessing(APSClient apsClient, InquiryPaymentRequest request,
                                              InquiryPaymentResponse response) throws Exception {

        }
    };

    @Test
    public void onPaymentStatusSuccess() {
        try {
            Assert.assertTrue(callback != null);
            InquiryPaymentRequest request = new InquiryPaymentRequest();
            InquiryPaymentResponse response = new InquiryPaymentResponse();
            Result paymentResult = new Result();
            paymentResult.setResultStatus(ResultStatusType.S);
            response.setPaymentResult(paymentResult);
            callback.onSstatus(DefaultAPS.getInstance().getApsClient(), request, response);
            verify(callback, times(1)).onPaymentStatusSuccess(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onPaymentStatusFail() {
        try {
            Assert.assertTrue(callback != null);
            InquiryPaymentRequest request = new InquiryPaymentRequest();
            InquiryPaymentResponse response = new InquiryPaymentResponse();
            Result paymentResult = new Result();
            paymentResult.setResultStatus(ResultStatusType.F);
            response.setPaymentResult(paymentResult);
            callback.onSstatus(DefaultAPS.getInstance().getApsClient(), request, response);
            verify(callback, times(1)).onPaymentStatusFail(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onPaymentStatusProcessing() {
        try {
            Assert.assertTrue(callback != null);
            InquiryPaymentRequest request = new InquiryPaymentRequest();
            InquiryPaymentResponse response = new InquiryPaymentResponse();
            Result paymentResult = new Result();
            paymentResult.setResultStatus(ResultStatusType.U);
            response.setPaymentResult(paymentResult);
            callback.onSstatus(DefaultAPS.getInstance().getApsClient(), request, response);
            verify(callback, times(1)).onPaymentStatusProcessing(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Test
    public void onFstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onFstatus(any(), any(), any());
            verify(callback, times(1)).onPaymentInquiryFail(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void onUstatus() {
        try {
            Assert.assertTrue(callback != null);
            callback.onUstatus(any(), any(), any());
            verify(callback, times(1)).onPaymentInquiryUnknown(any(), any(), any());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}