/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.DefaultPaymentNotifyCallback;
import com.alipay.ap.enums.ResultStatusType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.Result;
import com.alipay.ap.request.PayNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

@RunWith(MockitoJUnitRunner.class)
public class DefaultPaymentNotifyCallbackTest {

    @Spy
    private DefaultPaymentNotifyCallback callback = new DefaultPaymentNotifyCallback() {
        @Override
        public HttpReceiveResponse onPaymentSuccess(PayNotifyRequest request) {
            return null;
        }

        @Override
        public HttpReceiveResponse onPaymentFailure(PayNotifyRequest request) {
            return null;
        }
    };

    @Test
    public void doProcess() {
        PayNotifyRequest payNotifyRequest = new PayNotifyRequest();

        try (MockedStatic<SerializeUtil> serializeUtilMockedStatic = Mockito
            .mockStatic(SerializeUtil.class)) {
            serializeUtilMockedStatic.when(() -> SerializeUtil.deserializeRequest(any(), any()))
                .thenReturn(payNotifyRequest);
            Result paymentResult = new Result();
            paymentResult.setResultStatus(ResultStatusType.S);
            payNotifyRequest.setPaymentResult(paymentResult);
            // onPaymentSuccess
            callback.doProcess(any());
            verify(callback, times(1)).onPaymentSuccess(payNotifyRequest);
            Assert.assertEquals(ResultStatusType.S,
                payNotifyRequest.getPaymentResult().getResultStatus());

            // onPaymentFailure
            payNotifyRequest.getPaymentResult().setResultStatus(ResultStatusType.F);
            callback.doProcess(any());
            verify(callback, times(1)).onPaymentFailure(payNotifyRequest);
            Assert.assertEquals(ResultStatusType.F,
                payNotifyRequest.getPaymentResult().getResultStatus());

            // onPaymentUnknown
            payNotifyRequest.getPaymentResult().setResultStatus(ResultStatusType.U);
            try {
                callback.doProcess(any());
            } catch (Exception e) {
                Assert.assertTrue(e instanceof AlipayPlusApiException);
            }

            // null
            payNotifyRequest.getPaymentResult().setResultStatus(null);
            try {
                callback.doProcess(any());
            } catch (Exception e) {
                Assert.assertTrue(e instanceof AlipayPlusApiException);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}