/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.callbacks;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.DefaultRegistrationStatusNotifyCallback;
import com.alipay.ap.enums.RegistrationStatus;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.RegistrationResult;
import com.alipay.ap.request.MerchantRegistrationStatusNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.SerializeUtil;

@RunWith(MockitoJUnitRunner.class)
public class DefaultRegistrationStatusNotifyCallbackTest {

    @Spy
    private DefaultRegistrationStatusNotifyCallback callback = new DefaultRegistrationStatusNotifyCallback() {
        @Override
        public HttpReceiveResponse onCompleted(MerchantRegistrationStatusNotifyRequest request) {
            return null;
        }

        @Override
        public HttpReceiveResponse onPending(MerchantRegistrationStatusNotifyRequest request) {
            return null;
        }
    };

    @Test
    public void doProcess() {
        Assert.assertTrue(callback != null);
        MerchantRegistrationStatusNotifyRequest registrationStatusNotifyRequest = new MerchantRegistrationStatusNotifyRequest();

        try (MockedStatic<SerializeUtil> serializeUtilMockedStatic = Mockito
            .mockStatic(SerializeUtil.class)) {
            serializeUtilMockedStatic.when(() -> SerializeUtil.deserializeRequest(any(), any()))
                .thenReturn(registrationStatusNotifyRequest);
            // onCompleted
            RegistrationResult registrationResult = new RegistrationResult();
            registrationResult.setRegistrationStatus(RegistrationStatus.COMPLETED);
            registrationStatusNotifyRequest.setRegistrationResult(registrationResult);
            callback.doProcess(any());
            verify(callback, times(1)).onCompleted(registrationStatusNotifyRequest);
            Assert.assertEquals(RegistrationStatus.COMPLETED,
                registrationStatusNotifyRequest.getRegistrationResult().getRegistrationStatus());

            // onPending
            registrationStatusNotifyRequest.getRegistrationResult()
                .setRegistrationStatus(RegistrationStatus.PENDING);
            callback.doProcess(any());
            verify(callback, times(1)).onPending(registrationStatusNotifyRequest);
            Assert.assertEquals(RegistrationStatus.PENDING,
                registrationStatusNotifyRequest.getRegistrationResult().getRegistrationStatus());

            // null
            registrationStatusNotifyRequest.getRegistrationResult().setRegistrationStatus(null);
            try {
                callback.doProcess(any());
            } catch (Exception e) {
                Assert.assertTrue(e instanceof AlipayPlusApiException);
            }

            // other type
            registrationStatusNotifyRequest.getRegistrationResult().setRegistrationStatus(RegistrationStatus.CANCELLED);
            try {
                callback.doProcess(any());
            } catch (Exception e) {
                Assert.assertTrue(e instanceof AlipayPlusApiException);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}