/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.job;

import java.util.concurrent.TimeUnit;

import org.junit.Assert;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultCancelPaymentCallback;
import com.alipay.ap.callbacks.DefaultInquiryPaymentCallback;
import com.alipay.ap.callbacks.DefaultRefundPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.BizType;
import com.alipay.ap.enums.TransactionType;
import com.alipay.ap.example.schedule.ScheduleJobSupportInMemDemo;
import com.alipay.ap.job.APSScheduleJobExecutor;
import com.alipay.ap.model.Amount;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.net.DefaultApacheHttpPostAPSClient;
import com.alipay.ap.request.CancelPaymentRequest;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.response.CancelPaymentPayResponse;
import com.alipay.ap.response.InquiryPaymentResponse;
import com.alipay.ap.response.RefundResponse;
import com.alipay.ap.utils.DateUtil;
import com.alipay.ap.utils.GsonUtils;

public class APSScheduleJobExecutorTest {

    private static final Logger logger = LoggerFactory
        .getLogger(DefaultRefundPaymentCallback.class);

    @Test
    public void startJobExecutor_refund() {
        try {
            HttpSendCallback callback = buildRefundPaymentCallBack();
            APSScheduleJobExecutor.getInstance().setScheduledRefundCallback(callback);
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setRefundInterval(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            RefundRequest request = buildRefundRequest();

            Assert.assertTrue(request != null);
            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        TransactionType.REFUND.toString(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getRefundInterval()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(15 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void startJobExecutor_cancel() {
        try {
            HttpSendCallback callback = buildCancelPaymentCallBack();
            APSScheduleJobExecutor.getInstance().setScheduledCancelCallback(callback);
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setCancelInterval(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            CancelPaymentRequest request = buildCancelPaymentRequest();
            Assert.assertTrue(request != null);

            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        BizType.CANCEL.name(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getCancelInterval()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(15 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void startJobExecutor_inquiry() {
        try {
            APSScheduleJobExecutor.getInstance().setScheduledInquiryCallback(buildInquiryPaymentCallback());
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setInquiryIntervalForCashierpay(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            InquiryPaymentRequest request = buildInquiryRequest();
            Assert.assertTrue(request != null);

            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        BizType.INQUIRY.name(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getInquiryIntervalForCashierpay()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(15 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void startJobExecutor_refund_nullRefundCallback() {
        try {
            APSScheduleJobExecutor.getInstance().setScheduledRefundCallback(null);
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setRefundInterval(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            RefundRequest request = buildRefundRequest();
            Assert.assertTrue(request != null);

            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        TransactionType.REFUND.toString(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getRefundInterval()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(10 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void startJobExecutor_cancel_nullCancelCallback() {
        try {
            APSScheduleJobExecutor.getInstance().setScheduledCancelCallback(null);
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setCancelInterval(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            CancelPaymentRequest request = buildCancelPaymentRequest();
            Assert.assertTrue(request != null);

            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        BizType.CANCEL.name(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getCancelInterval()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(10 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void startJobExecutor_inquiry_nullInquiryCallback() {
        try {
            APSScheduleJobExecutor.getInstance().setScheduledInquiryCallback(null);
            APSScheduleJobExecutor.getInstance()
                .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

            APSSettings instance = new APSSettings();
            instance.setInquiryIntervalForCashierpay(new int[] { 5, 5 });
            APSScheduleJobExecutor.getInstance()
                .setApsClient(new DefaultApacheHttpPostAPSClient(instance));

            APSScheduleJobExecutor.getInstance().startJobExecutor();

            InquiryPaymentRequest request = buildInquiryRequest();
            Assert.assertTrue(request != null);

            // U status
            for (int i = 0; i < 3; i++) {
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(request.getPaymentRequestId(),
                        BizType.INQUIRY.name(), GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(instance.getInquiryIntervalForCashierpay()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
            }
            try {
                Thread.sleep(10 * 1000L);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private InquiryPaymentRequest buildInquiryRequest() {
        InquiryPaymentRequest inquiryPaymentRequest = new InquiryPaymentRequest();
        inquiryPaymentRequest.setPaymentRequestId("4654564541541");
        return inquiryPaymentRequest;
    }

    private static HttpSendCallback buildCancelPaymentCallBack() {

        return new DefaultCancelPaymentCallback() {

            @Override
            public void onCancelPaymentSuccess(APSClient apsClient, CancelPaymentRequest request,
                                               CancelPaymentPayResponse response) {

                logger.debug(
                    "# Cancel onCancelPaymentSuccess [isJobSchedule={}] , [cancelPaymentRequest={}],[cancelPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

            }

            @Override
            public void onCancelPaymentFail(APSClient apsClient, CancelPaymentRequest request,
                                            CancelPaymentPayResponse response) {

                logger.debug(
                    "# Cancel onCancelPaymentFail [isJobSchedule={}] , [cancelPaymentRequest={}],[cancelPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

            }

            @Override
            public void onCancelPaymentUnknown(APSClient apsClient, CancelPaymentRequest request,
                                               CancelPaymentPayResponse response) throws Exception {

                logger.debug(
                    "# cancel onCancelPaymentUnknown [isJobSchedule={}] , [cancelPaymentRequest={}],[cancelPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

                APSScheduleJobExecutor.instance.put(new ScheduleJobRequest(
                        request.getPaymentRequestId(), BizType.CANCEL.name(),
                        GsonUtils.toJSONString(request),
                        DateUtil.triggerTime(APSScheduleJobExecutor.instance.getApsClient()
                                        .getApsSettings().getCancelInterval()[CommonConstants.NUMBER_ZERO],
                                TimeUnit.SECONDS)));
            }
        };

    }

    private CancelPaymentRequest buildCancelPaymentRequest() {
        CancelPaymentRequest cancelPaymentRequest = new CancelPaymentRequest();
        cancelPaymentRequest.setPaymentRequestId("6544115156");
        return cancelPaymentRequest;
    }

    private RefundRequest buildRefundRequest() {

        RefundRequest refundRequest = new RefundRequest();
        Amount refundAmount = new Amount();
        refundAmount.setCurrency("PHP");
        refundRequest.setRefundAmount(refundAmount);
        refundAmount.setValue(30000);
        refundRequest.setRefundReason("refund reason");
        refundRequest.setRefundRequestId("14141555");
        refundRequest.setPaymentRequestId("465564151515");

        return refundRequest;
    }

    private static HttpSendCallback buildRefundPaymentCallBack() {

        return new DefaultRefundPaymentCallback() {

            @Override
            public void onRefundPaymentSuccess(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) {
                logger.debug(
                    "# Refund onRefundPaymentSuccess [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

            }

            @Override
            public void onRefundPaymentFail(APSClient apsClient, RefundRequest request,
                                            RefundResponse response) {
                logger.debug(
                    "# Refund onPaymentRefundFail [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

            }

            @Override
            public void onRefundPaymentUnknown(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) throws Exception {
                logger.debug(
                    "# Refund onRefundPaymentUnknown [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

                if (request.getRetryContext().isMaxRetry()) {
                    // TODO contact connect_support@service.alipay.com.

                } else {
                    request.getRetryContext().setContinueRetry(true);
                }

            }
        };
    }

    private HttpSendCallback buildInquiryPaymentCallback() {
        return new DefaultInquiryPaymentCallback() {
            @Override
            public void onPaymentInquiryFail(APSClient apsClient, InquiryPaymentRequest request,
                                             InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "# inquiry onPaymentInquiryFail [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);
            }

            @Override
            public void onPaymentInquiryUnknown(APSClient apsClient, InquiryPaymentRequest request,
                                                InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "# inquiry onPaymentInquiryUnknown [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);
                /** implement the callback if this request is executed asynchronously */
                if (request.getRetryContext().isMaxRetry()) {
                    // If keep receiving the same result
                    // TODO Take actions for inquiry payment unknown

                } else {
                    /** this is optional to register the schedule for inquiry payment retry */
                    request.getRetryContext().setContinueRetry(true);
                }
            }

            @Override
            public void onPaymentStatusSuccess(APSClient apsClient, InquiryPaymentRequest request,
                                               InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "# inquiry onPaymentStatusSuccess [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);
            }

            @Override
            public void onPaymentStatusFail(APSClient apsClient, InquiryPaymentRequest request,
                                            InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "# inquiry onPaymentStatusFail [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);
            }

            @Override
            public void onPaymentStatusProcessing(APSClient apsClient,
                                                  InquiryPaymentRequest request,
                                                  InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "# inquiry onPaymentStatusProcessing [isJobSchedule={}] , [refundPaymentRequest={}],[refundPaymentPayResponse={}],",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously */
                if (request.getRetryContext().isMaxRetry()) {
                    // If keep receiving the same result
                    // TODO Take actions for inquiry payment unknown

                } else {
                    /** this is optional to register the schedule for inquiry payment retry */
                    request.getRetryContext().setContinueRetry(true);
                }
            }
        };

    }

}