/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.job;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.job.ScheduleJob;
import com.alipay.ap.job.ScheduleJobSupport;
import com.alipay.ap.job.ScheduleRunnable;
import com.alipay.ap.model.RetryContext;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 *
 * @author wqy01239909
 * @version $Id: ScheduleRunnableTest.java, v 0.1 2022‐07-27 15:32 wqy01239909 Exp $
 */
@RunWith(MockitoJUnitRunner.class)
public class ScheduleRunnableTest {

    @Mock
    private ScheduleJobSupport scheduleJobSupport;
    @Mock
    protected APSClient        apsClient;
    @Mock
    protected ScheduleJob      scheduleJob;
    @Mock
    protected HttpSendCallback scheduledCallback;
    @Mock
    private APSRequest         apsRequest;
    @Mock
    private RetryContext       retryContext;
    @Spy
    @InjectMocks
    private ScheduleRunnable   scheduleRunnable = new ScheduleRunnable(scheduleJobSupport,
        apsClient, apsRequest, scheduleJob, scheduledCallback, new int[] { 2, 2, 2 });

    @Test
    public void run_lockFail() {

        doReturn(false).when(scheduleJobSupport).lock(any());
        scheduleRunnable.run();

        try {
            verify(scheduleJobSupport, never()).remove(any());
            verify(scheduleJobSupport, never()).update(any());
            Assert.assertTrue(true);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void run_lockSuccess_notContinue() {
        doReturn(false).when(retryContext).isContinueRetry();
        doReturn(retryContext).when(apsRequest).getRetryContext();

        doReturn(2).when(scheduleJob).getExecutedTimes();
        doReturn(true).when(scheduleJobSupport).lock(any());
        scheduleRunnable.run();

        verify(scheduleJobSupport, times(1)).remove(any());
        Assert.assertTrue(true);
    }

    @Test
    public void run_lockSuccess_Continue_removeJob() {
        doReturn(true).when(retryContext).isContinueRetry();
        doReturn(retryContext).when(apsRequest).getRetryContext();

        doReturn(true).when(scheduleJobSupport).lock(any());

        doReturn(2).when(scheduleJob).getExecutedTimes();
        scheduleRunnable.run();

        verify(scheduleJobSupport, times(1)).remove(any());
        Assert.assertTrue(true);
    }

    @Test
    public void run_lockSuccess_Continue_updateJob() {

        doReturn(true).when(retryContext).isContinueRetry();
        doReturn(retryContext).when(apsRequest).getRetryContext();

        doReturn(true).when(scheduleJobSupport).lock(any());

        doReturn(1).when(scheduleJob).getExecutedTimes();
        scheduleRunnable.run();

        verify(scheduleJobSupport, times(1)).update(any());
        Assert.assertTrue(true);
    }
}