/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.utils;

import static org.junit.Assert.assertEquals;

import java.util.Date;
import java.util.TimeZone;

import javax.xml.bind.DatatypeConverter;

import org.apache.commons.lang3.time.DateFormatUtils;
import org.junit.Assert;
import org.junit.Test;

import com.alipay.ap.utils.DateUtil;

public class DateUtilTest {

    /** ISO format of date **/
    public final static String ISO8601 = "yyyy-MM-dd'T'HH:mm:ssZZZ";

    /** ISO format of date **/
    public final static String SIMPLE  = "yyyyMMddHHmmss";

    @Test
    public void getCurISO8601Time() {
        Assert.assertEquals(DateFormatUtils.format(new Date().getTime(), ISO8601),
            DateUtil.getCurISO8601Time());

    }

    @Test
    public void getSimpleFormatTime() {
        Assert.assertEquals(DateFormatUtils.format(new Date().getTime(), SIMPLE),
            DateUtil.getSimpleFormatTime());
    }

    @Test
    public void getISODateTimeStr() {
        Date date = new Date();

        Assert.assertEquals(DateFormatUtils.format(date.getTime(), ISO8601),
            DateUtil.getISODateTimeStr(date));

        Assert.assertEquals(null, DateUtil.getISODateTimeStr(null));

        Assert.assertEquals(DateFormatUtils.format(date.getTime(), ISO8601),
            DateUtil.getISODateTimeStr(date, TimeZone.getDefault()));

        Assert.assertEquals(null, DateUtil.getISODateTimeStr(null, TimeZone.getDefault()));
    }

    @Test
    public void parseISODateTimeStr() {
        String dateTimeStr = "2022-08-23T10:14:48+08:00";
        assertEquals(DatatypeConverter.parseDate(dateTimeStr).getTime(),
            DateUtil.parseISODateTimeStr(dateTimeStr));
    }

}