/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.unittest.utils;

import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;

import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.request.ApplyTokenRequest;
import com.alipay.ap.request.PayNotifyRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.ApplyTokenResponse;
import com.alipay.ap.utils.SerializeUtil;

public class SerializeUtilTest {

    @Test
    public void deserializeResponse() {
        ApplyTokenRequest applyTokenRequest  = new ApplyTokenRequest();
        String rspBody = "{\"acquirerId\":\"Z02TE4250000000A\",\"pspId\":\"102217000000000000A\","
                + "\"result\":{\"resultCode\":\"UNKNOWN_EXCEPTION\",\"resultMessage\":\"Unknown exception\",\"resultStatus\":\"U\"}}";
        Map<String, String> responseHeader = new HashMap<>();
        responseHeader.put("client-id", "test");
        APSResponse apsResponse = SerializeUtil.deserializeResponse(applyTokenRequest, rspBody, responseHeader);
        assertTrue(apsResponse instanceof ApplyTokenResponse);
        Assert.assertEquals("UNKNOWN_EXCEPTION", apsResponse.getResult().getResultCode());
        Assert.assertEquals("test", apsResponse.getResponseHeader().getClientId());

    }

    @Test
    public void deserialzeRequest() throws Exception {
        HttpReceive httpReceive  = new HttpReceive();
        httpReceive.setReceiveType(HttpReceiveType.PAYMENT_NOTIFY);
        httpReceive.setRequestURI("/payment/notify");
        httpReceive.setReceiveReqBody(("{\"test\": \"test\",\"paymentResult\": {\"resultCode\": \"SUCCESS\",\"resultStatus\": \"S\","
                + "\"resultMessage\": \"success\"}}").getBytes());
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, "SANDBOX_112231458452");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, "2022-07-19T10:59:39+08:00");
        hashMap.put(CommonConstants.REQ_SIGN_HEADER,
                "algorithm=RSA256,keyVersion=1,signature=GgJ7aAtPG5axTSBNPfU67pEY1SBxt1LXAlw3tUV3ruwy%2F05eOHz4doOfP%2BRdbPSkjxUSp6qLDz8JoIp%2FOSh0ou4JbuC9mYQdZQvDGLB1DjbL5mcX3y0Mc28UUuM7Hqfx%2F8RyFOaugudeHITQquTlVa4u6rXIxnPLubeGup9NCkso2BJNw%2FtjD13RQfPMhw%2Frrhjxpws%2B8XTC8lTM5w%2Fh9i2LIoz956VOgrwIVMd5CwvHL%2F8MV3spXxYC48LQTuy52LTg6Ym0EuKF6D%2BySL%2Fq4LMWXt8mVIY9xMTI0pme9JFv5kYsZkMCmv%2Fefph0aizgc5LpIqKjBGwfeXXhi7PEdQ%3D%3D");
        httpReceive.setReceiveRequestHeader(hashMap);
        PayNotifyRequest payNotifyRequest = SerializeUtil.deserializeRequest(httpReceive, PayNotifyRequest.class);
        Assert.assertEquals("SUCCESS", payNotifyRequest.getPaymentResult().getResultCode());
        Assert.assertEquals("SANDBOX_112231458452", payNotifyRequest.getRequestHeader().getClientId());
    }
}